"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _records;
Object.defineProperty(exports, "__esModule", { value: true });
const cli_1 = require("./cli");
const GarbageMap_1 = __importDefault(require("./common/GarbageMap"));
const util_1 = require("./common/util");
const core_1 = require("./interfaces/core");
class ChannelConfigManager {
    constructor() {
        _records.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(1000 * 60 * 30)));
    }
    async getChannelConfig(channelConfigUrl) {
        const a = __classPrivateFieldGet(this, _records).get(channelConfigUrl);
        if (a) {
            const elapsed = core_1.elapsedSince(a.timestamp);
            if (elapsed < 1000 * 60 * 3) {
                return a.channelConfig;
            }
            else {
                const b = await this._fetchChannelConfigRecord(channelConfigUrl);
                if (b)
                    return b.channelConfig;
                else
                    return a.channelConfig;
            }
        }
        else {
            const b = await this._fetchChannelConfigRecord(channelConfigUrl);
            if (b)
                return b.channelConfig;
            else
                return null;
        }
    }
    getChannelConfigSync(channelConfigUrl) {
        this.getChannelConfig(channelConfigUrl); // trigger a get
        const a = __classPrivateFieldGet(this, _records).get(channelConfigUrl);
        if (a) {
            return a.channelConfig;
        }
        else
            return null;
    }
    async _fetchChannelConfigRecord(channelConfigUrl) {
        try {
            const config = await loadConfig(channelConfigUrl.toString());
            if (config) {
                if (cli_1.isChannelConfig(config)) {
                    config.authorizedNodes;
                    const r = {
                        channelConfig: config,
                        timestamp: core_1.nowTimestamp(),
                        channelConfigUrl
                    };
                    __classPrivateFieldGet(this, _records).set(channelConfigUrl, r);
                    return r;
                }
                else {
                    console.warn(`Invalid channel config for: ${channelConfigUrl}`);
                    const r = {
                        channelConfig: null,
                        timestamp: core_1.nowTimestamp(),
                        channelConfigUrl
                    };
                    __classPrivateFieldGet(this, _records).set(channelConfigUrl, r);
                    return r;
                }
            }
            else
                return null;
        }
        catch (err) {
            return null;
        }
    }
}
_records = new WeakMap();
const loadConfig = async (url) => {
    return await util_1.loadYamlFromUrl(url);
};
exports.default = ChannelConfigManager;
//# sourceMappingURL=ChannelConfigManager.js.map