"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _keyPair, _nodeId, _joinedChannels, _feedManager, _mutableManager, _remoteNodeManager, _kacheryStorageManager, _channelConfigManager, _proxyConnectionsToClients, _proxyConnectionsToServers, _downloadStreamManager, _publicUdpSocketAddress, _publicUdpSocketServer, _downloadOptimizer, _onProxyConnectionToServerCallbacks, _stats, _mirrorSources, _clientAuthCode, _downloadStreamInfos;
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const process_1 = require("process");
const ChannelConfigManager_1 = __importDefault(require("./ChannelConfigManager"));
const crypto_util_1 = require("./common/crypto_util");
const DataStreamy_1 = __importDefault(require("./common/DataStreamy"));
const GarbageMap_1 = __importDefault(require("./common/GarbageMap"));
const util_1 = require("./common/util");
const DownloadOptimizer_1 = __importDefault(require("./downloadOptimizer/DownloadOptimizer"));
const FeedManager_1 = __importDefault(require("./feeds/FeedManager"));
const MutableManager_1 = __importDefault(require("./mutables/MutableManager"));
const getStats_1 = require("./getStats");
const core_1 = require("./interfaces/core");
const NodeToNodeRequest_1 = require("./interfaces/NodeToNodeRequest");
const NodeStats_1 = __importDefault(require("./NodeStats"));
const handleCheckAliveRequest_1 = require("./nodeToNodeRequestHandlers/handleCheckAliveRequest");
const handleCheckForFileRequest_1 = require("./nodeToNodeRequestHandlers/handleCheckForFileRequest");
const handleCheckForLiveFeedRequest_1 = require("./nodeToNodeRequestHandlers/handleCheckForLiveFeedRequest");
const handleDownloadFileDataRequest_1 = require("./nodeToNodeRequestHandlers/handleDownloadFileDataRequest");
const handleDownloadSubfeedMessagesRequest_1 = require("./nodeToNodeRequestHandlers/handleDownloadSubfeedMessagesRequest");
const handleFallbackUdpPacketRequest_1 = require("./nodeToNodeRequestHandlers/handleFallbackUdpPacketRequest");
const handleGetChannelInfoRequest_1 = require("./nodeToNodeRequestHandlers/handleGetChannelInfoRequest");
const handleReportSubfeedMessages_1 = require("./nodeToNodeRequestHandlers/handleReportSubfeedMessages");
const handleStartStreamViaUdpRequest_1 = require("./nodeToNodeRequestHandlers/handleStartStreamViaUdpRequest");
const handleSubmitMessageToLiveFeedRequest_1 = require("./nodeToNodeRequestHandlers/handleSubmitMessageToLiveFeedRequest");
const handleSubscribeToSubfeed_1 = require("./nodeToNodeRequestHandlers/handleSubscribeToSubfeed");
const protocolVersion_1 = require("./protocolVersion");
const RemoteNodeManager_1 = __importDefault(require("./RemoteNodeManager"));
class KacheryP2PNode {
    constructor(p) {
        this.p = p;
        _keyPair.set(this, void 0);
        _nodeId.set(this, void 0);
        _joinedChannels.set(this, []);
        _feedManager.set(this, void 0);
        _mutableManager.set(this, void 0);
        _remoteNodeManager.set(this, void 0);
        _kacheryStorageManager.set(this, void 0);
        _channelConfigManager.set(this, new ChannelConfigManager_1.default());
        _proxyConnectionsToClients.set(this, new Map());
        _proxyConnectionsToServers.set(this, new Map());
        _downloadStreamManager.set(this, new DownloadStreamManager);
        _publicUdpSocketAddress.set(this, null);
        _publicUdpSocketServer.set(this, null);
        _downloadOptimizer.set(this, void 0);
        _onProxyConnectionToServerCallbacks.set(this, []);
        _stats.set(this, new NodeStats_1.default());
        _mirrorSources.set(this, []);
        _clientAuthCode.set(this, { current: '', previous: '' });
        __classPrivateFieldSet(this, _kacheryStorageManager, p.externalInterface.createKacheryStorageManager());
        if ((p.configDir) && (fs_1.default.existsSync(`${p.configDir.toString()}/private.pem`))) {
            migrateKeypairToStorageDir(p.configDir, __classPrivateFieldGet(this, _kacheryStorageManager).storageDir());
        }
        __classPrivateFieldSet(this, _keyPair, __classPrivateFieldGet(this, _kacheryStorageManager).storageDir() ? _loadKeypair(__classPrivateFieldGet(this, _kacheryStorageManager).storageDir()) : crypto_util_1.createKeyPair());
        __classPrivateFieldSet(this, _nodeId, core_1.publicKeyHexToNodeId(crypto_util_1.publicKeyToHex(__classPrivateFieldGet(this, _keyPair).publicKey))); // get the node id from the public key
        const storageDir = __classPrivateFieldGet(this, _kacheryStorageManager).storageDir();
        if (storageDir) {
            fs_1.default.writeFileSync(`${storageDir}/kachery-node-id`, `${__classPrivateFieldGet(this, _nodeId)}`);
        }
        __classPrivateFieldSet(this, _mutableManager, new MutableManager_1.default(storageDir)
        // The feed manager -- each feed is a collection of append-only logs
        );
        // The feed manager -- each feed is a collection of append-only logs
        const localFeedManager = this.p.externalInterface.createLocalFeedManager(__classPrivateFieldGet(this, _mutableManager));
        __classPrivateFieldSet(this, _feedManager, new FeedManager_1.default(this, localFeedManager));
        __classPrivateFieldSet(this, _remoteNodeManager, new RemoteNodeManager_1.default(this));
        __classPrivateFieldSet(this, _downloadOptimizer, new DownloadOptimizer_1.default(this));
    }
    nodeId() {
        return __classPrivateFieldGet(this, _nodeId);
    }
    keyPair() {
        return __classPrivateFieldGet(this, _keyPair);
    }
    joinedChannels() {
        return [...__classPrivateFieldGet(this, _joinedChannels)];
    }
    joinedChannelConfigUrls() {
        return __classPrivateFieldGet(this, _joinedChannels).map(x => (x.channelConfigUrl));
    }
    hasJoinedChannel(channelConfigUrl) {
        return (__classPrivateFieldGet(this, _joinedChannels).filter(x => (x.channelConfigUrl === channelConfigUrl)).length > 0);
    }
    setJoinedChannels(x) {
        __classPrivateFieldSet(this, _joinedChannels, x);
    }
    setMirrorSources(x) {
        __classPrivateFieldSet(this, _mirrorSources, x);
    }
    mirrorSources() {
        return [...__classPrivateFieldGet(this, _mirrorSources)];
    }
    remoteNodeManager() {
        return __classPrivateFieldGet(this, _remoteNodeManager);
    }
    isBootstrapNode() {
        return this.p.opts.isBootstrapNode;
    }
    useMulticastUdp() {
        return (this.p.opts.multicastUdpAddress !== null);
    }
    kacheryStorageManager() {
        return __classPrivateFieldGet(this, _kacheryStorageManager);
    }
    downloadStreamManager() {
        return __classPrivateFieldGet(this, _downloadStreamManager);
    }
    downloadOptimizer() {
        return __classPrivateFieldGet(this, _downloadOptimizer);
    }
    getDefects() {
        return this.p.opts.getDefects();
    }
    stats() {
        return __classPrivateFieldGet(this, _stats);
    }
    async getChannelConfig(channelConfigUrl) {
        return await __classPrivateFieldGet(this, _channelConfigManager).getChannelConfig(channelConfigUrl);
    }
    getChannelConfigSync(channelConfigUrl) {
        return __classPrivateFieldGet(this, _channelConfigManager).getChannelConfigSync(channelConfigUrl);
    }
    nodeIsAuthorizedOnChannel(nodeId, channelConfigUrl) {
        const channelConfig = this.getChannelConfigSync(channelConfigUrl);
        if (!channelConfig)
            return false;
        if (channelConfig.authorizedNodes.find(an => (an.nodeId === nodeId)))
            return true;
        else
            return false;
    }
    nodeIsOnChannel(nodeId, channelConfigUrl) {
        if (nodeId === __classPrivateFieldGet(this, _nodeId)) {
            return __classPrivateFieldGet(this, _joinedChannels).find(a => (a.channelConfigUrl === channelConfigUrl)) ? true : false;
        }
        else {
            const rn = __classPrivateFieldGet(this, _remoteNodeManager).getRemoteNode(nodeId);
            if (!rn)
                return false;
            return rn.getJoinedChannelConfigUrls().find(u => (u === channelConfigUrl)) ? true : false;
        }
    }
    nodeIsPublicOnChannel(nodeId, channelConfigUrl) {
        const channelConfig = this.getChannelConfigSync(channelConfigUrl);
        if (!channelConfig)
            return false;
        const x = channelConfig.authorizedNodes.find(an => (an.nodeId === nodeId));
        if (!x)
            return false;
        if (!x.isPublic)
            return false;
        if (nodeId === __classPrivateFieldGet(this, _nodeId)) {
            const joinedChannelConfig = __classPrivateFieldGet(this, _joinedChannels).find(a => (a.channelConfigUrl === channelConfigUrl));
            if (!joinedChannelConfig)
                return false;
            return joinedChannelConfig.isPublic ? true : false;
        }
        else {
            const remoteChannelNodeInfo = this.getRemoteChannelNodeInfo(nodeId, channelConfigUrl);
            if (!remoteChannelNodeInfo)
                return false;
            if (remoteChannelNodeInfo.body.isPublic)
                return true;
        }
        return false;
    }
    findFile(args) {
        let onFoundCallbacks = [];
        let onFinishedCallbacks = [];
        let cancelled = false;
        let handleCancel = null;
        setTimeout(() => {
            if (cancelled)
                return;
            const requestData = {
                requestType: 'checkForFile',
                fileKey: args.fileKey
            };
            // first check on this node
            handleCheckForFileRequest_1.handleCheckForFileRequest(this, __classPrivateFieldGet(this, _nodeId), requestData).then((thisResponse) => {
                const s = thisResponse.size;
                if ((thisResponse.found) && (s !== null)) {
                    onFoundCallbacks.forEach(cb => {
                        cb({
                            nodeId: __classPrivateFieldGet(this, _nodeId),
                            fileKey: args.fileKey,
                            fileSize: s
                        });
                    });
                }
                const { onResponse, onFinished, onErrorResponse, cancel } = __classPrivateFieldGet(this, _remoteNodeManager).sendRequestToAllNodes(requestData, { timeoutMsec: args.timeoutMsec });
                handleCancel = cancel;
                onResponse((nodeId, responseData) => {
                    if (!NodeToNodeRequest_1.isCheckForFileResponseData(responseData)) {
                        /* istanbul ignore next */
                        throw Error(`Unexpected response type: ${responseData.requestType} <> 'checkForFile'`);
                    }
                    const { found, size } = responseData;
                    if ((found) && (size !== null)) {
                        onFoundCallbacks.forEach(cb => {
                            cb({
                                nodeId,
                                fileKey: args.fileKey,
                                fileSize: size
                            });
                        });
                    }
                });
                onFinished(() => {
                    onFinishedCallbacks.forEach(cb => {
                        cb();
                    });
                });
                onErrorResponse((nodeId, err) => {
                    // todo
                });
            }).catch((err) => {
                throw Error('Unexpected problem trying to find file on local node');
            });
        }, 10);
        return {
            onFound: (cb) => {
                onFoundCallbacks.push(cb);
            },
            onFinished: (cb) => {
                onFinishedCallbacks.push(cb);
            },
            cancel: () => {
                if (cancelled)
                    return;
                handleCancel && handleCancel();
            }
        };
    }
    cleanup() {
        __classPrivateFieldGet(this, _proxyConnectionsToClients).forEach(c => {
            c.close();
        });
        __classPrivateFieldGet(this, _proxyConnectionsToClients).clear();
        __classPrivateFieldGet(this, _proxyConnectionsToServers).forEach(c => {
            c.close();
        });
        __classPrivateFieldGet(this, _proxyConnectionsToServers).clear();
    }
    externalInterface() {
        return this.p.externalInterface;
    }
    feedManager() {
        return __classPrivateFieldGet(this, _feedManager);
    }
    mutableManager() {
        return __classPrivateFieldGet(this, _mutableManager);
    }
    setProxyConnectionToClient(nodeId, c) {
        if (__classPrivateFieldGet(this, _proxyConnectionsToClients).has(nodeId)) {
            // we already have this connection
            c.close();
            return;
        }
        __classPrivateFieldGet(this, _proxyConnectionsToClients).set(nodeId, c);
        c.onClosed(() => {
            if (__classPrivateFieldGet(this, _proxyConnectionsToClients).get(nodeId) === c) {
                __classPrivateFieldGet(this, _proxyConnectionsToClients).delete(nodeId);
            }
        });
    }
    setProxyConnectionToServer(nodeId, c) {
        if (__classPrivateFieldGet(this, _proxyConnectionsToServers).has(nodeId)) {
            // we already have this connection
            c.close();
            return;
        }
        __classPrivateFieldGet(this, _proxyConnectionsToServers).set(nodeId, c);
        c.onClosed(() => {
            if (__classPrivateFieldGet(this, _proxyConnectionsToServers).get(nodeId) === c) {
                __classPrivateFieldGet(this, _proxyConnectionsToServers).delete(nodeId);
            }
        });
        __classPrivateFieldGet(this, _onProxyConnectionToServerCallbacks).forEach(cb => { cb(); });
    }
    onProxyConnectionToServer(callback) {
        __classPrivateFieldGet(this, _onProxyConnectionToServerCallbacks).push(callback);
    }
    getProxyConnectionToServer(nodeId) {
        return __classPrivateFieldGet(this, _proxyConnectionsToServers).get(nodeId) || null;
    }
    getProxyConnectionToClient(nodeId) {
        return __classPrivateFieldGet(this, _proxyConnectionsToClients).get(nodeId) || null;
    }
    getChannelInfo(channelConfigUrl) {
        const remoteNodesInChannel = __classPrivateFieldGet(this, _remoteNodeManager).getRemoteNodesInChannel(channelConfigUrl, { includeOffline: false });
        const x = {
            nodes: remoteNodesInChannel.map(n => {
                return n.getRemoteChannelNodeInfo(channelConfigUrl);
            }).filter(channelInfo => (channelInfo !== null))
                .map(channelInfo => {
                if (channelInfo === null) {
                    throw Error('Unexpected channelInfo === null should have been filtered out');
                }
                return channelInfo;
            })
        };
        return x;
    }
    hostName() {
        if (this.p.hostName)
            return this.p.hostName;
        if (this.p.externalInterface.isMock) {
            return core_1.hostName(this.nodeId().toString());
        }
        else {
            /* istanbul ignore next */
            return null;
        }
    }
    publicUrl() {
        return this.p.publicUrl;
    }
    httpAddress() {
        const u = this.publicUrl();
        if (u) {
            return {
                url: u
            };
        }
        else {
            const h = this.hostName();
            return (h !== null) && (this.p.httpListenPort !== null) ? { hostName: h, port: this.p.httpListenPort } : null;
        }
    }
    webSocketAddress() {
        const h = this.hostName();
        return (h !== null) && (this.p.webSocketListenPort !== null) ? { hostName: h, port: this.p.webSocketListenPort } : null;
    }
    udpSocketPort() {
        return this.p.udpSocketPort;
    }
    publicUdpSocketAddress() {
        if (__classPrivateFieldGet(this, _publicUdpSocketAddress) !== null) {
            return __classPrivateFieldGet(this, _publicUdpSocketAddress);
        }
        else {
            const h = this.hostName();
            if ((h !== null) && (this.p.udpSocketPort !== null)) {
                return {
                    hostName: h,
                    port: this.p.udpSocketPort
                };
            }
            else {
                return null;
            }
        }
    }
    setPublicUdpSocketAddress(a) {
        __classPrivateFieldSet(this, _publicUdpSocketAddress, a);
    }
    setPublicUdpSocketServer(s) {
        __classPrivateFieldSet(this, _publicUdpSocketServer, s);
    }
    publicUdpSocketServer() {
        return __classPrivateFieldGet(this, _publicUdpSocketServer);
    }
    getJoinedChannelConfig(channelConfigUrl) {
        return __classPrivateFieldGet(this, _joinedChannels).find(x => (x.channelConfigUrl === channelConfigUrl));
    }
    getRemoteChannelNodeInfo(nodeId, channelConfigUrl) {
        const rn = __classPrivateFieldGet(this, _remoteNodeManager).getRemoteNode(nodeId);
        if (!rn)
            return null;
        return rn.getRemoteChannelNodeInfo(channelConfigUrl);
    }
    async getChannelNodeInfo(channelConfigUrl) {
        const channelConfig = await this.getChannelConfig(channelConfigUrl);
        const joinedChannelConfig = this.getJoinedChannelConfig(channelConfigUrl);
        if (!joinedChannelConfig) {
            console.warn('Unexpected: joinedChannelConfig is not defined in getChannelNodeInfo');
        }
        const messageProxyWebsocketNodeIds = [];
        const dataProxyWebsocketNodeIds = [];
        if (channelConfig) {
            channelConfig.authorizedNodes.forEach(an => {
                if ((an.isMessageProxy) || (an.isDataProxy)) {
                    const rcni = this.getRemoteChannelNodeInfo(an.nodeId, channelConfigUrl);
                    if (rcni) {
                        if ((rcni.body.isMessageProxy) && (an.isMessageProxy)) {
                            if (__classPrivateFieldGet(this, _proxyConnectionsToServers).has(an.nodeId)) {
                                messageProxyWebsocketNodeIds.push(an.nodeId);
                            }
                        }
                        if ((rcni.body.isDataProxy) && (an.isDataProxy)) {
                            if (__classPrivateFieldGet(this, _proxyConnectionsToServers).has(an.nodeId)) {
                                dataProxyWebsocketNodeIds.push(an.nodeId);
                            }
                        }
                    }
                }
            });
        }
        const body = {
            channelConfigUrl,
            nodeId: __classPrivateFieldGet(this, _nodeId),
            nodeLabel: this.p.label,
            httpAddress: this.httpAddress(),
            webSocketAddress: this.webSocketAddress(),
            publicUdpSocketAddress: __classPrivateFieldGet(this, _publicUdpSocketAddress),
            isMessageProxy: joinedChannelConfig ? joinedChannelConfig.isMessageProxy : undefined,
            isDataProxy: joinedChannelConfig ? joinedChannelConfig.isDataProxy : undefined,
            isPublic: joinedChannelConfig ? joinedChannelConfig.isPublic : undefined,
            messageProxyWebsocketNodeIds,
            dataProxyWebsocketNodeIds,
            timestamp: core_1.nowTimestamp()
        };
        return {
            body,
            signature: crypto_util_1.getSignature(body, __classPrivateFieldGet(this, _keyPair))
        };
    }
    async submitMessageToRemoteLiveFeed({ nodeId, feedId, subfeedHash, message, timeoutMsec }) {
        const requestData = {
            requestType: 'submitMessageToLiveFeed',
            feedId,
            subfeedHash,
            message
        };
        const responseData = await __classPrivateFieldGet(this, _remoteNodeManager).sendRequestToNode(nodeId, requestData, { timeoutMsec: timeoutMsec, method: 'default' });
        if (!NodeToNodeRequest_1.isSubmitMessageToLiveFeedResponseData(responseData)) {
            /* istanbul ignore next */
            throw Error(`Error submitting message to remote live feed: Unexpected response data.`);
        }
        if (!responseData.success) {
            throw Error(`Error submitting message to remote live feed: ${responseData.errorMessage}`);
        }
    }
    async findLiveFeed(args) {
        const { feedId, timeoutMsec } = args;
        return new Promise((resolve, reject) => {
            const requestData = {
                requestType: 'checkForLiveFeed',
                feedId
            };
            const { onResponse, onFinished, onErrorResponse, cancel } = __classPrivateFieldGet(this, _remoteNodeManager).sendRequestToAllNodes(requestData, { timeoutMsec });
            let found = false;
            onResponse((nodeId, responseData) => {
                if (found)
                    return;
                if (!NodeToNodeRequest_1.isCheckForLiveFeedResponseData(responseData)) {
                    /* istanbul ignore next */
                    throw Error('Unexpected response type.');
                }
                if (responseData.found) {
                    found = true;
                    resolve({
                        nodeId
                    });
                }
            });
            onFinished(() => {
                if (!found) {
                    resolve(null);
                }
            });
            onErrorResponse((nodeId, error) => {
                // todo
            });
        });
    }
    async handleNodeToNodeRequest(request) {
        const { requestId, fromNodeId, toNodeId, timestamp, requestData } = request.body;
        if (!crypto_util_1.verifySignature(request.body, request.signature, core_1.nodeIdToPublicKey(fromNodeId))) {
            // think about banning the node here
            /* istanbul ignore next */
            throw Error('Invalid signature in node-to-node request');
        }
        if (toNodeId !== __classPrivateFieldGet(this, _nodeId)) {
            // redirect request to a different node
            const p = __classPrivateFieldGet(this, _proxyConnectionsToClients).get(toNodeId);
            if (!p) {
                throw Error(`No proxy connection to node: ${fromNodeId.slice(0, 6)} -> ${toNodeId.slice(0, 6)}`);
            }
            return await p.sendRequest(request, { timeoutMsec: core_1.addDurations(request.body.timeoutMsec, core_1.scaledDurationMsec(1000)) });
        }
        let responseData;
        if (NodeToNodeRequest_1.isCheckAliveRequestData(requestData)) {
            responseData = await handleCheckAliveRequest_1.handleCheckAliveRequest(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isGetChannelInfoRequestData(requestData)) {
            responseData = await handleGetChannelInfoRequest_1.handleGetChannelInfoRequest(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isAnnounceRequestData(requestData)) {
            responseData = await this.remoteNodeManager().handleAnnounceRequest({ fromNodeId, requestData, localUdpAddress: null });
        }
        else if (NodeToNodeRequest_1.isCheckForFileRequestData(requestData)) {
            responseData = await handleCheckForFileRequest_1.handleCheckForFileRequest(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isCheckForLiveFeedRequestData(requestData)) {
            responseData = await handleCheckForLiveFeedRequest_1.handleCheckForLiveFeedRequest(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isSubscribeToSubfeedRequestData(requestData)) {
            responseData = await handleSubscribeToSubfeed_1.handleSubscribeToSubfeed(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isReportNewSubfeedMessagesRequestData(requestData)) {
            responseData = await handleReportSubfeedMessages_1.handleReportNewSubfeedMessages(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isDownloadFileDataRequestData(requestData)) {
            responseData = await handleDownloadFileDataRequest_1.handleDownloadFileDataRequest(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isDownloadSubfeedMessagesRequestData(requestData)) {
            responseData = await handleDownloadSubfeedMessagesRequest_1.handleDownloadSubfeedMessagesRequest(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isSubmitMessageToLiveFeedRequestData(requestData)) {
            responseData = await handleSubmitMessageToLiveFeedRequest_1.handleSubmitMessageToLiveFeedRequest(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isStartStreamViaUdpRequestData(requestData)) {
            responseData = await handleStartStreamViaUdpRequest_1.handleStartStreamViaUdpRequest(this, fromNodeId, requestData);
        }
        else if (NodeToNodeRequest_1.isFallbackUdpPacketRequestData(requestData)) {
            responseData = await handleFallbackUdpPacketRequest_1.handleFallbackUdpPacketRequest(this, fromNodeId, requestData);
        }
        else {
            /* istanbul ignore next */
            {
                console.warn(requestData);
                throw Error('Unexpected error: unrecognized request data.');
            }
        }
        const body = {
            protocolVersion: protocolVersion_1.protocolVersion(),
            requestId,
            fromNodeId: __classPrivateFieldGet(this, _nodeId),
            toNodeId: fromNodeId,
            timestamp: core_1.nowTimestamp(),
            responseData: responseData
        };
        return {
            body,
            signature: crypto_util_1.getSignature(body, __classPrivateFieldGet(this, _keyPair))
        };
    }
    async streamDataForStreamId(fromNodeId, streamId) {
        if (fromNodeId !== __classPrivateFieldGet(this, _nodeId)) {
            // redirect to a different node
            const p = __classPrivateFieldGet(this, _proxyConnectionsToClients).get(fromNodeId);
            if (!p) {
                /* istanbul ignore next */
                throw Error(`No proxy connection to node: ${fromNodeId.slice(0, 6)} <> ${__classPrivateFieldGet(this, _nodeId).slice(0, 6)}`);
            }
            return p.streamDataForStreamId(streamId);
        }
        const s = __classPrivateFieldGet(this, _downloadStreamManager).get(streamId);
        if (!s) {
            /* istanbul ignore next */
            throw Error(`Unable to find download info for stream: ${streamId}: (node: ${__classPrivateFieldGet(this, _nodeId).slice(0, 6)})`);
        }
        if (NodeToNodeRequest_1.isDownloadFileDataRequestData(s)) {
            const { startByte, endByte } = s;
            /* istanbul ignore next */
            if (endByte === null)
                throw Error('Unexpected in streamDataForStreamId');
            const ret = new DataStreamy_1.default();
            const dataStream = await __classPrivateFieldGet(this, _kacheryStorageManager).getFileReadStream(s.fileKey);
            ret.producer().onCancelled(() => {
                dataStream.cancel();
            });
            try {
                dataStream.onStarted(size => { ret.producer().start(size); });
                dataStream.onData(b => { ret.producer().data(b); });
                dataStream.onFinished(() => { ret.producer().end(); });
                dataStream.onError(err => { ret.producer().error(err); });
            }
            catch (err) {
                ret.producer().error(err);
            }
            return ret;
        }
        else if (NodeToNodeRequest_1.isDownloadSubfeedMessagesRequestData(s)) {
            const ret = new DataStreamy_1.default();
            const { feedId, subfeedHash, position, numMessages } = s;
            let subfeed;
            try {
                subfeed = await this.feedManager()._loadSubfeed(feedId, subfeedHash);
            }
            catch (err) {
                ret.producer().error(new Error('Problem loading subfeed'));
                return ret;
            }
            const numLocalMessages = subfeed.getNumLocalMessages();
            if (core_1.messageCountToNumber(numLocalMessages) < core_1.subfeedPositionToNumber(s.position) + core_1.messageCountToNumber(s.numMessages)) {
                ret.producer().error(new Error('Not enough subfeed messages locally'));
                return ret;
            }
            let signedMessages;
            try {
                signedMessages = subfeed.getLocalSignedMessages({ position, numMessages });
            }
            catch (err) {
                ret.producer().error(new Error('Problem loading signed messages'));
                return ret;
            }
            process_1.nextTick(() => {
                const x = Buffer.from(crypto_util_1.JSONStringifyDeterministic(signedMessages));
                ret.producer().start(core_1.byteCount(x.byteLength));
                ret.producer().data(x);
                ret.producer().end();
            });
            return ret;
        }
        else {
            throw Error('Unexpected');
        }
    }
    receiveFallbackUdpPacket(fromNodeId, packetId, packet) {
        if (!__classPrivateFieldGet(this, _publicUdpSocketServer)) {
            /* istanbul ignore next */
            throw Error(`Cannot receive fallback udp packet from ${fromNodeId.slice(0, 6)}. No public udp socket server set.`);
        }
        __classPrivateFieldGet(this, _publicUdpSocketServer).receiveFallbackUdpPacket(fromNodeId, packetId, packet);
    }
    getStats(o) {
        return getStats_1.getStats(this, o);
    }
    nodeLabel() {
        return this.p.label;
    }
    setClientAuthCode(code, previousCode) {
        __classPrivateFieldSet(this, _clientAuthCode, {
            current: code,
            previous: previousCode
        });
    }
    verifyClientAuthCode(code) {
        if (code === __classPrivateFieldGet(this, _clientAuthCode).current)
            return true;
        if ((__classPrivateFieldGet(this, _clientAuthCode).previous) && (code === __classPrivateFieldGet(this, _clientAuthCode).previous))
            return true;
        return false;
    }
}
_keyPair = new WeakMap(), _nodeId = new WeakMap(), _joinedChannels = new WeakMap(), _feedManager = new WeakMap(), _mutableManager = new WeakMap(), _remoteNodeManager = new WeakMap(), _kacheryStorageManager = new WeakMap(), _channelConfigManager = new WeakMap(), _proxyConnectionsToClients = new WeakMap(), _proxyConnectionsToServers = new WeakMap(), _downloadStreamManager = new WeakMap(), _publicUdpSocketAddress = new WeakMap(), _publicUdpSocketServer = new WeakMap(), _downloadOptimizer = new WeakMap(), _onProxyConnectionToServerCallbacks = new WeakMap(), _stats = new WeakMap(), _mirrorSources = new WeakMap(), _clientAuthCode = new WeakMap();
const migrateKeypairToStorageDir = (configDir, storageDir) => {
    const oldPublicKeyPath = `${configDir.toString()}/public.pem`;
    const oldPrivateKeyPath = `${configDir.toString()}/private.pem`;
    const newPublicKeyPath = `${storageDir.toString()}/public.pem`;
    const newPrivateKeyPath = `${storageDir.toString()}/private.pem`;
    if (fs_1.default.existsSync(newPublicKeyPath)) {
        throw Error(`Unexpected problem migrating keypair. File exists: ${newPublicKeyPath}`);
    }
    if (fs_1.default.existsSync(newPrivateKeyPath)) {
        throw Error(`Unexpected problem migrating keypair. File exists: ${newPrivateKeyPath}`);
    }
    if (!fs_1.default.existsSync(oldPublicKeyPath)) {
        throw Error(`Unexpected problem migrating keypair. File does not exists: ${oldPublicKeyPath}`);
    }
    if (!fs_1.default.existsSync(oldPrivateKeyPath)) {
        throw Error(`Unexpected problem migrating keypair. File exists: ${oldPrivateKeyPath}`);
    }
    console.info('Migrating keypair to kachery storage directory');
    fs_1.default.renameSync(oldPublicKeyPath, newPublicKeyPath);
    fs_1.default.renameSync(oldPrivateKeyPath, newPrivateKeyPath);
    fs_1.default.chmodSync(newPublicKeyPath, fs_1.default.constants.S_IRUSR | fs_1.default.constants.S_IWUSR);
    fs_1.default.chmodSync(newPrivateKeyPath, fs_1.default.constants.S_IRUSR | fs_1.default.constants.S_IWUSR);
};
class DownloadStreamManager {
    constructor() {
        _downloadStreamInfos.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
    }
    set(streamId, info) {
        __classPrivateFieldGet(this, _downloadStreamInfos).set(streamId, info);
    }
    get(streamId) {
        return __classPrivateFieldGet(this, _downloadStreamInfos).get(streamId) || null;
    }
}
_downloadStreamInfos = new WeakMap();
const _loadKeypair = (storageDir) => {
    if (!fs_1.default.existsSync(storageDir.toString())) {
        /* istanbul ignore next */
        throw Error(`Storage directory does not exist: ${storageDir}`);
    }
    const publicKeyPath = `${storageDir.toString()}/public.pem`;
    const privateKeyPath = `${storageDir.toString()}/private.pem`;
    if (fs_1.default.existsSync(publicKeyPath)) {
        /* istanbul ignore next */
        if (!fs_1.default.existsSync(privateKeyPath)) {
            throw Error(`Public key file exists, but secret key file does not.`);
        }
    }
    else {
        const { publicKey, privateKey } = crypto_util_1.createKeyPair();
        fs_1.default.writeFileSync(publicKeyPath, publicKey.toString(), { encoding: 'utf-8' });
        fs_1.default.writeFileSync(privateKeyPath, privateKey.toString(), { encoding: 'utf-8', mode: fs_1.default.constants.S_IRUSR | fs_1.default.constants.S_IWUSR });
        fs_1.default.chmodSync(publicKeyPath, fs_1.default.constants.S_IRUSR | fs_1.default.constants.S_IWUSR);
        fs_1.default.chmodSync(privateKeyPath, fs_1.default.constants.S_IRUSR | fs_1.default.constants.S_IWUSR);
    }
    if (util_1.isReadableByOthers(privateKeyPath)) {
        throw Error(`Invalid permissions for private key file: ${privateKeyPath}`);
    }
    const keyPair = {
        publicKey: fs_1.default.readFileSync(publicKeyPath, { encoding: 'utf-8' }),
        privateKey: fs_1.default.readFileSync(privateKeyPath, { encoding: 'utf-8' }),
    };
    if (!core_1.isKeyPair(keyPair)) {
        /* istanbul ignore next */
        throw Error('Invalid keyPair');
    }
    testKeyPair(keyPair);
    return keyPair;
};
const testKeyPair = (keyPair) => {
    const signature = crypto_util_1.getSignature({ test: 1 }, keyPair);
    if (!crypto_util_1.verifySignature({ test: 1 }, signature, keyPair.publicKey)) {
        /* istanbul ignore next */
        throw new Error('Problem testing public/private keys. Error verifying signature.');
    }
    if (crypto_util_1.hexToPublicKey(crypto_util_1.publicKeyToHex(keyPair.publicKey)) !== keyPair.publicKey) {
        /* istanbul ignore next */
        throw new Error('Problem testing public/private keys. Error converting public key to/from hex.');
    }
    if (crypto_util_1.hexToPrivateKey(crypto_util_1.privateKeyToHex(keyPair.privateKey)) !== keyPair.privateKey) {
        /* istanbul ignore next */
        throw new Error('Problem testing public/private keys. Error converting private key to/from hex.');
    }
};
exports.default = KacheryP2PNode;
//# sourceMappingURL=KacheryP2PNode.js.map