"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _remoteNodeId, _channelNodeInfoByChannel, _isBootstrap, _bootstrapAddress, _bootstrapWebSocketAddress, _bootstrapUdpSocketAddress, _localUdpAddressForRemoteNode, _numUdpBytesSent, _numUdpBytesLost, _numHttpBytesSent, _numRequestsSent, _numResponsesReceived, _isOnline, _sendMessageMethodOptimizer, _downloadFileDataMethodOptimizer;
Object.defineProperty(exports, "__esModule", { value: true });
exports.channelNodeInfoIsExpired = void 0;
const constants_1 = require("./common/constants");
const crypto_util_1 = require("./common/crypto_util");
const core_1 = require("./interfaces/core");
const NodeToNodeRequest_1 = require("./interfaces/NodeToNodeRequest");
const DownloadFileDataMethodOptimizer_1 = __importDefault(require("./methodOptimizers/DownloadFileDataMethodOptimizer"));
const SendMessageMethodOptimizer_1 = __importDefault(require("./methodOptimizers/SendMessageMethodOptimizer"));
const protocolVersion_1 = require("./protocolVersion");
const PublicUdpSocketServer_1 = require("./services/PublicUdpSocketServer");
class RemoteNode {
    constructor(node, remoteNodeId, opts) {
        _node.set(this, void 0);
        _remoteNodeId.set(this, void 0);
        _channelNodeInfoByChannel.set(this, new Map());
        _isBootstrap.set(this, void 0);
        _bootstrapAddress.set(this, void 0);
        _bootstrapWebSocketAddress.set(this, void 0);
        _bootstrapUdpSocketAddress.set(this, void 0);
        _localUdpAddressForRemoteNode.set(this, null);
        _numUdpBytesSent.set(this, core_1.byteCount(0));
        _numUdpBytesLost.set(this, core_1.byteCount(0));
        _numHttpBytesSent.set(this, core_1.byteCount(0));
        _numRequestsSent.set(this, 0);
        _numResponsesReceived.set(this, 0);
        _isOnline.set(this, false);
        _sendMessageMethodOptimizer.set(this, void 0);
        _downloadFileDataMethodOptimizer.set(this, void 0);
        __classPrivateFieldSet(this, _node, node);
        __classPrivateFieldSet(this, _remoteNodeId, remoteNodeId);
        __classPrivateFieldSet(this, _isBootstrap, opts.isBootstrap);
        __classPrivateFieldSet(this, _bootstrapAddress, opts.bootstrapAddress);
        __classPrivateFieldSet(this, _bootstrapWebSocketAddress, opts.bootstrapWebSocketAddress);
        __classPrivateFieldSet(this, _bootstrapUdpSocketAddress, opts.bootstrapUdpSocketAddress);
        __classPrivateFieldSet(this, _sendMessageMethodOptimizer, new SendMessageMethodOptimizer_1.default(__classPrivateFieldGet(this, _node), this));
        __classPrivateFieldSet(this, _downloadFileDataMethodOptimizer, new DownloadFileDataMethodOptimizer_1.default(__classPrivateFieldGet(this, _node), this));
    }
    remoteNodeId() {
        return __classPrivateFieldGet(this, _remoteNodeId);
    }
    remoteNodeLabel() {
        const cni = this._getMostRecentChannelNodeInfo();
        return cni ? cni.body.nodeLabel : null;
    }
    isBootstrap() {
        return __classPrivateFieldGet(this, _isBootstrap);
    }
    bootstrapAddress() {
        return __classPrivateFieldGet(this, _bootstrapAddress);
    }
    bootstrapWebSocketAddress() {
        return __classPrivateFieldGet(this, _bootstrapWebSocketAddress);
    }
    setChannelNodeInfoIfMoreRecent(channelConfigUrl, channelNodeInfo) {
        if (exports.channelNodeInfoIsExpired(channelNodeInfo))
            return;
        if (__classPrivateFieldGet(this, _channelNodeInfoByChannel).has(channelConfigUrl)) {
            const x = __classPrivateFieldGet(this, _channelNodeInfoByChannel).get(channelConfigUrl);
            if (!x)
                throw Error('Unexpected.');
            if (x.body.timestamp > channelNodeInfo.body.timestamp) {
                // already have more recent
                return;
            }
        }
        if ((!__classPrivateFieldGet(this, _isOnline)) && (this.isAuthorizedToCommunicate())) {
            // let's check to see if it is online
            const requestData = {
                requestType: 'checkAlive'
            };
            this.sendRequest(requestData, { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, method: 'default' }).then(responseData => {
                if (!NodeToNodeRequest_1.isCheckAliveResponseData(responseData))
                    throw Error('Unexpected response for checkAlive request');
                if (responseData.alive) {
                    this._setOnline(true);
                }
            }).catch(err => {
                // maybe not online
            });
        }
        __classPrivateFieldGet(this, _channelNodeInfoByChannel).set(channelConfigUrl, channelNodeInfo);
    }
    getJoinedChannelConfigUrls() {
        let ret = [];
        __classPrivateFieldGet(this, _channelNodeInfoByChannel).forEach((channelNodeInfo, channelConfigUrl) => {
            ret.push(channelConfigUrl);
        });
        return ret;
    }
    getRemoteChannelNodeInfo(channelConfigUrl) {
        return __classPrivateFieldGet(this, _channelNodeInfoByChannel).get(channelConfigUrl) || null;
    }
    getRemoteNodeWebSocketAddress() {
        let ret = this.bootstrapWebSocketAddress();
        if (ret)
            return ret;
        __classPrivateFieldGet(this, _channelNodeInfoByChannel).forEach((cni) => {
            if (cni.body.webSocketAddress) {
                ret = cni.body.webSocketAddress;
            }
        });
        return ret;
    }
    setLocalUdpAddressForRemoteNode(address) {
        __classPrivateFieldSet(this, _localUdpAddressForRemoteNode, address);
    }
    getLocalUdpAddressForRemoteNode() {
        return __classPrivateFieldGet(this, _localUdpAddressForRemoteNode);
    }
    getPublicUdpAddressForRemoteNode() {
        let ret = null;
        __classPrivateFieldGet(this, _channelNodeInfoByChannel).forEach((channelNodeInfo) => {
            if (ret === null) {
                if (channelNodeInfo.body.publicUdpSocketAddress) {
                    ret = channelNodeInfo.body.publicUdpSocketAddress;
                }
            }
        });
        return ret;
    }
    getUdpAddressForRemoteNode() {
        return (__classPrivateFieldGet(this, _bootstrapUdpSocketAddress)) || (this.getLocalUdpAddressForRemoteNode()) || (this.getPublicUdpAddressForRemoteNode());
    }
    getStats() {
        const channelNodeInfos = [];
        __classPrivateFieldGet(this, _channelNodeInfoByChannel).forEach((cni) => {
            channelNodeInfos.push(cni);
        });
        const s = {
            remoteNodeId: __classPrivateFieldGet(this, _remoteNodeId),
            channelNodeInfos,
            isBootstrap: __classPrivateFieldGet(this, _isBootstrap),
            bootstrapAddress: __classPrivateFieldGet(this, _bootstrapAddress),
            bootstrapWebSocketAddress: __classPrivateFieldGet(this, _bootstrapWebSocketAddress),
            bootstrapUdpSocketAddress: __classPrivateFieldGet(this, _bootstrapUdpSocketAddress),
            localUdpAddressForRemoteNode: __classPrivateFieldGet(this, _localUdpAddressForRemoteNode),
            numUdpBytesSent: __classPrivateFieldGet(this, _numUdpBytesSent),
            numUdpBytesLost: __classPrivateFieldGet(this, _numUdpBytesLost),
            numHttpBytesSent: __classPrivateFieldGet(this, _numHttpBytesSent),
            numRequestsSent: __classPrivateFieldGet(this, _numRequestsSent),
            numResponsesReceived: __classPrivateFieldGet(this, _numResponsesReceived)
        };
        return s;
    }
    ageOfChannelNodeInfo(channelConfigUrl) {
        const cni = __classPrivateFieldGet(this, _channelNodeInfoByChannel).get(channelConfigUrl);
        if (!cni)
            return null;
        return core_1.unscaledDurationMsec(core_1.elapsedSince(cni.body.timestamp));
    }
    pruneChannelNodeInfos() {
        const channelConfigUrls = __classPrivateFieldGet(this, _channelNodeInfoByChannel).keys();
        for (let channelConfigUrl of channelConfigUrls) {
            const cni = __classPrivateFieldGet(this, _channelNodeInfoByChannel).get(channelConfigUrl);
            /* istanbul ignore next */
            if (!cni)
                throw Error('Unexpected in pruneChannelNodeInfos');
            if (exports.channelNodeInfoIsExpired(cni)) {
                __classPrivateFieldGet(this, _channelNodeInfoByChannel).delete(channelConfigUrl);
            }
        }
    }
    _formRequestFromRequestData(requestData, opts) {
        const requestId = core_1.createRequestId();
        const requestBody = {
            protocolVersion: protocolVersion_1.protocolVersion(),
            requestId,
            fromNodeId: __classPrivateFieldGet(this, _node).nodeId(),
            toNodeId: __classPrivateFieldGet(this, _remoteNodeId),
            timestamp: core_1.nowTimestamp(),
            timeoutMsec: opts.timeoutMsec,
            requestData
        };
        const request = {
            body: requestBody,
            signature: crypto_util_1.getSignature(requestBody, __classPrivateFieldGet(this, _node).keyPair())
        };
        return request;
    }
    getRemoteNodeHttpAddress() {
        if (__classPrivateFieldGet(this, _isBootstrap) && __classPrivateFieldGet(this, _bootstrapAddress)) {
            return __classPrivateFieldGet(this, _bootstrapAddress);
        }
        else {
            const channelNodeData = this._getMostRecentChannelNodeInfo();
            if (channelNodeData === null) {
                return null;
            }
            const remoteInfo = channelNodeData.body;
            if (remoteInfo.httpAddress) {
                return remoteInfo.httpAddress;
            }
            else {
                return null;
            }
        }
    }
    getRemoteNodeMessageProxyNode() {
        const channelConfigUrls = this.getJoinedChannelConfigUrls();
        for (let channelConfigUrl of channelConfigUrls) {
            const channelNodeInfo = __classPrivateFieldGet(this, _channelNodeInfoByChannel).get(channelConfigUrl);
            if (channelNodeInfo) {
                for (let proxyNodeId of channelNodeInfo.body.messageProxyWebsocketNodeIds) {
                    if (proxyNodeId !== __classPrivateFieldGet(this, _remoteNodeId)) {
                        const prn = __classPrivateFieldGet(this, _node).remoteNodeManager().getRemoteNode(proxyNodeId);
                        if (prn) {
                            if ((prn.isOnline()) && (prn.getRemoteNodeHttpAddress())) {
                                const channelConfig = __classPrivateFieldGet(this, _node).getChannelConfigSync(channelConfigUrl);
                                const channelConfigProxyNode = ((channelConfig || {}).authorizedNodes || []).find(an => (an.nodeId === prn.remoteNodeId()));
                                if (channelConfigProxyNode) {
                                    if (channelConfigProxyNode.isMessageProxy) {
                                        return prn;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return null;
    }
    getRemoteNodeDataProxyNode() {
        const channelConfigUrls = this.getJoinedChannelConfigUrls();
        for (let channelConfigUrl of channelConfigUrls) {
            const channelNodeInfo = __classPrivateFieldGet(this, _channelNodeInfoByChannel).get(channelConfigUrl);
            if (channelNodeInfo) {
                for (let proxyNodeId of channelNodeInfo.body.dataProxyWebsocketNodeIds) {
                    if (proxyNodeId !== __classPrivateFieldGet(this, _remoteNodeId)) {
                        const prn = __classPrivateFieldGet(this, _node).remoteNodeManager().getRemoteNode(proxyNodeId);
                        if (prn) {
                            if ((prn.isOnline()) && (prn.getRemoteNodeHttpAddress())) {
                                const channelConfig = __classPrivateFieldGet(this, _node).getChannelConfigSync(channelConfigUrl);
                                const channelConfigProxyNode = ((channelConfig || {}).authorizedNodes || []).find(an => (an.nodeId === prn.remoteNodeId()));
                                if (channelConfigProxyNode) {
                                    if (channelConfigProxyNode.isDataProxy) {
                                        return prn;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return null;
    }
    isAuthorizedToCommunicate() {
        if (__classPrivateFieldGet(this, _node).isBootstrapNode()) {
            // we are a bootstrap node
            return true;
        }
        if (__classPrivateFieldGet(this, _isBootstrap)) {
            // remote node is a bootstrap node
            return true;
        }
        // Other than bootstrap, there are two ways we can be authorized to communicate with the remote node
        // (1) we are joined to a channel where this node and the remote node are both authorized nodes
        // (2) we are a public node on a channel where the remote node has joined
        // (3) the remote node is a public node on a channel where we have joined
        const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
        for (let channelConfigUrl of channelConfigUrls) { // loop through the joined channels
            // (1) we are joined to a channel where this node and the remote node are both authorized nodes
            if (__classPrivateFieldGet(this, _node).nodeIsAuthorizedOnChannel(__classPrivateFieldGet(this, _node).nodeId(), channelConfigUrl) && (__classPrivateFieldGet(this, _node).nodeIsAuthorizedOnChannel(__classPrivateFieldGet(this, _remoteNodeId), channelConfigUrl))) {
                return true;
            }
            // (2) we are a public node on a channel where the remote node has joined
            if ((__classPrivateFieldGet(this, _node).nodeIsPublicOnChannel(__classPrivateFieldGet(this, _node).nodeId(), channelConfigUrl)) && (__classPrivateFieldGet(this, _node).nodeIsOnChannel(__classPrivateFieldGet(this, _remoteNodeId), channelConfigUrl))) {
                return true;
            }
            // (3) the remote node is a public node on a channel where we have joined
            if (__classPrivateFieldGet(this, _node).nodeIsPublicOnChannel(__classPrivateFieldGet(this, _remoteNodeId), channelConfigUrl)) {
                return true;
            }
        }
        return false;
    }
    canSendRequest(method) {
        if (!this.isAuthorizedToCommunicate()) {
            return false;
        }
        let method2 = __classPrivateFieldGet(this, _sendMessageMethodOptimizer).determineSendRequestMethod(method);
        return method2 === null ? false : true;
    }
    canSendData(method) {
        if (!this.isAuthorizedToCommunicate()) {
            return false;
        }
        return __classPrivateFieldGet(this, _downloadFileDataMethodOptimizer).determineDownloadFileDataMethod(method) ? true : false;
    }
    async downloadFileData(streamId, opts) {
        if (!this.isAuthorizedToCommunicate()) {
            throw Error('Cannot download file data. Not authorized to communicate.');
        }
        const method = __classPrivateFieldGet(this, _downloadFileDataMethodOptimizer).determineDownloadFileDataMethod(opts.method);
        if (method === null) {
            throw Error('No method available to download stream');
        }
        if (method === 'http') {
            const address = this.getRemoteNodeHttpAddress();
            if (!address) {
                throw Error('Unable to download file data... no http address found.');
            }
            const ds = await __classPrivateFieldGet(this, _node).externalInterface().httpGetDownload(address, core_1.urlPath(`/download/${this.remoteNodeId()}/${__classPrivateFieldGet(this, _node).nodeId()}/${streamId}`), __classPrivateFieldGet(this, _node).stats(), { fromNodeId: __classPrivateFieldGet(this, _remoteNodeId) });
            return {
                method,
                dataStream: ds
            };
        }
        else if (method === 'http-proxy') {
            const proxyRemoteNode = await this.getRemoteNodeDataProxyNode();
            if (!proxyRemoteNode) {
                /* istanbul ignore next */
                throw Error('Unexpected. Unable to download file data... no proxy remote node found.');
            }
            const address = proxyRemoteNode.getRemoteNodeHttpAddress();
            if (!address) {
                /* istanbul ignore next */
                throw Error('Unexpected. Unable to download file data... no http address for proxy remote node.');
            }
            const ds = await __classPrivateFieldGet(this, _node).externalInterface().httpGetDownload(address, core_1.urlPath(`/download/${this.remoteNodeId()}/${__classPrivateFieldGet(this, _node).nodeId()}/${streamId}`), __classPrivateFieldGet(this, _node).stats(), { fromNodeId: null });
            return {
                method,
                dataStream: ds
            };
        }
        else if (method === 'udp') {
            const ds = await this._streamDataViaUdpFromRemoteNode(streamId);
            return {
                method,
                dataStream: ds
            };
        }
        else {
            /* istanbul ignore next */
            throw Error('Unexpected');
        }
    }
    async _streamDataViaUdpFromRemoteNode(streamId) {
        if (!this.isAuthorizedToCommunicate()) {
            throw Error('In _streamDataViaUdpFromRemoteNode: Not authorized to communicate.');
        }
        const udpS = __classPrivateFieldGet(this, _node).publicUdpSocketServer();
        if (!udpS) {
            /* istanbul ignore next */
            throw Error('Cannot stream from remote node without udp socket server');
        }
        const requestData = {
            requestType: 'startStreamViaUdp',
            streamId
        };
        const incomingDataStream = udpS.getIncomingDataStream(streamId);
        incomingDataStream.producer().onCancelled(() => {
            const stopReqData = {
                requestType: "stopStreamViaUdp",
                streamId
            };
            this.sendRequest(stopReqData, { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, method: 'default' }).then(stopResponseData => {
                /* istanbul ignore next */
                if (!NodeToNodeRequest_1.isStopStreamViaUdpResponseData(stopResponseData))
                    throw Error('Unexpected in _streamDataViaUdpFromRemoteNode');
                // okay
            }).catch(err => {
                throw Error(`Error sending request to stop stream via udp: ${err.message}`);
            });
        });
        try {
            const responseData = await this.sendRequest(requestData, { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, method: 'default' });
            /* istanbul ignore next */
            if (!NodeToNodeRequest_1.isStartStreamViaUdpResponseData(responseData))
                throw Error('Unexpected in _streamDataViaUdpFromRemoteNode');
            /* istanbul ignore next */
            if (!responseData.success)
                throw Error(`Error starting stream via udp: ${responseData.errorMessage}`);
        }
        catch (err) {
            incomingDataStream.producer().error(err);
        }
        return incomingDataStream;
    }
    async startStreamViaUdpToRemoteNode(streamId) {
        if (!this.isAuthorizedToCommunicate()) {
            throw Error('In startStreamViaUdpToRemoteNode: Not authorized to communicate.');
        }
        const r = __classPrivateFieldGet(this, _node).downloadStreamManager().get(streamId);
        if (r === null) {
            /* istanbul ignore next */
            throw Error('Stream not found');
        }
        const udpS = __classPrivateFieldGet(this, _node).publicUdpSocketServer();
        const udpA = this.getUdpAddressForRemoteNode();
        /* istanbul ignore next */
        if (!udpS)
            throw Error('Cannot use stream via udp when there is no udp socket server');
        /* istanbul ignore next */
        if (!udpA)
            throw Error('Cannot stream via udp when there is no udp address');
        let ds = await __classPrivateFieldGet(this, _node).streamDataForStreamId(__classPrivateFieldGet(this, _node).nodeId(), streamId);
        const fallbackAddress = PublicUdpSocketServer_1.nodeIdFallbackAddress(__classPrivateFieldGet(this, _remoteNodeId));
        udpS.setOutgoingDataStream(udpA, fallbackAddress, streamId, ds, { toNodeId: __classPrivateFieldGet(this, _remoteNodeId) });
    }
    async sendRequest(requestData, opts) {
        if (!this.isAuthorizedToCommunicate()) {
            throw Error('In sendRequest: Not authorized to communicate.');
        }
        try {
            const ret = await this._trySendRequest(requestData, opts);
            this._setOnline(true);
            return ret;
        }
        catch (err) {
            this._setOnline(false);
            throw err;
        }
    }
    _setOnline(val) {
        if (val === __classPrivateFieldGet(this, _isOnline))
            return;
        __classPrivateFieldSet(this, _isOnline, val);
    }
    isOnline() {
        return __classPrivateFieldGet(this, _isOnline);
    }
    async _trySendRequest(requestData, opts) {
        const request = this._formRequestFromRequestData(requestData, { timeoutMsec: opts.timeoutMsec });
        const requestId = request.body.requestId;
        const method = await __classPrivateFieldGet(this, _sendMessageMethodOptimizer).determineSendRequestMethod(opts.method);
        if (method === null) {
            /* istanbul ignore next */
            throw Error(`Method not available for sending request ${requestData.requestType}: ${opts.method} (from ${__classPrivateFieldGet(this, _node).nodeId().slice(0, 6)} to ${__classPrivateFieldGet(this, _remoteNodeId).slice(0, 6)})`);
        }
        const requestSize = core_1.byteCount(JSON.stringify(request).length);
        __classPrivateFieldGet(this, _sendMessageMethodOptimizer).reportSendRequestStart(request.body.requestId, method, opts.timeoutMsec);
        let response;
        if ((method === 'http') || (method === 'http-proxy')) {
            let address;
            if (method === 'http') {
                address = this.getRemoteNodeHttpAddress();
                if (!address) {
                    /* istanbul ignore next */
                    throw Error('Unexpected. Unable to send request... no http address found.');
                }
            }
            else if (method === 'http-proxy') {
                const proxyRemoteNode = await this.getRemoteNodeMessageProxyNode();
                if (!proxyRemoteNode) {
                    /* istanbul ignore next */
                    throw Error('Unexpected. Unable to send request... no proxy remote node found.');
                }
                address = proxyRemoteNode.getRemoteNodeHttpAddress();
                if (!address) {
                    /* istanbul ignore next */
                    throw Error('Unexpected. Unable to send request... no http address for proxy remote node.');
                }
            }
            else {
                /* istanbul ignore next */
                throw Error('Unexpected method in _trySendRequest');
            }
            __classPrivateFieldSet(this, _numHttpBytesSent, core_1.addByteCount(__classPrivateFieldGet(this, _numHttpBytesSent), requestSize));
            __classPrivateFieldSet(this, _numRequestsSent, +__classPrivateFieldGet(this, _numRequestsSent) + 1);
            __classPrivateFieldGet(this, _node).stats().reportBytesSent('http', __classPrivateFieldGet(this, _remoteNodeId), core_1.byteCount(JSON.stringify(request).length));
            const R = await __classPrivateFieldGet(this, _node).externalInterface().httpPostJson(address, core_1.urlPath('/NodeToNodeRequest'), request, { timeoutMsec: opts.timeoutMsec });
            if (!NodeToNodeRequest_1.isNodeToNodeResponse(R)) {
                // ban the node?
                /* istanbul ignore next */
                throw Error('Invalid response from node.');
            }
            __classPrivateFieldSet(this, _numResponsesReceived, +__classPrivateFieldGet(this, _numResponsesReceived) + 1);
            response = R;
        }
        else if (method === 'udp') {
            const udpS = __classPrivateFieldGet(this, _node).publicUdpSocketServer();
            const udpA = this.getUdpAddressForRemoteNode();
            if (!udpS) {
                /* istanbul ignore next */
                throw Error('Cannot use udp method when there is no udp socket server');
            }
            if (!udpA) {
                /* istanbul ignore next */
                throw Error('Cannot use udp method when there is no udp address');
            }
            __classPrivateFieldSet(this, _numUdpBytesSent, core_1.addByteCount(__classPrivateFieldGet(this, _numUdpBytesSent), requestSize));
            __classPrivateFieldSet(this, _numRequestsSent, +__classPrivateFieldGet(this, _numRequestsSent) + 1);
            const R = await udpS.sendRequest(udpA, request, { timeoutMsec: opts.timeoutMsec });
            __classPrivateFieldSet(this, _numResponsesReceived, +__classPrivateFieldGet(this, _numResponsesReceived) + 1);
            response = R.response;
            const udpHeader = R.header;
            if (__classPrivateFieldGet(this, _isBootstrap)) {
                // only trust the reported public udp address if it is a bootstrap node
                const publicUdpAddress = udpHeader.body.toAddress;
                if (publicUdpAddress) {
                    __classPrivateFieldGet(this, _node).setPublicUdpSocketAddress(publicUdpAddress);
                }
            }
        }
        else if (method === 'websocket') {
            const c1 = __classPrivateFieldGet(this, _node).getProxyConnectionToServer(__classPrivateFieldGet(this, _remoteNodeId));
            const c2 = __classPrivateFieldGet(this, _node).getProxyConnectionToClient(__classPrivateFieldGet(this, _remoteNodeId));
            if (c1) {
                response = await c1.sendRequest(request, { timeoutMsec: opts.timeoutMsec });
            }
            else if (c2) {
                response = await c2.sendRequest(request, { timeoutMsec: opts.timeoutMsec });
            }
            else {
                throw Error('Unexpected: no websocket proxy connection');
            }
        }
        else {
            /* istanbul ignore next */
            throw Error('Unexpected');
        }
        __classPrivateFieldGet(this, _sendMessageMethodOptimizer).reportSendRequestEnd(request.body.requestId, method);
        if (response.body.responseData.requestType !== request.body.requestData.requestType) {
            /* istanbul ignore next */
            throw Error('Unexpected requestType in response.');
        }
        if (response.body.fromNodeId !== __classPrivateFieldGet(this, _remoteNodeId)) {
            /* istanbul ignore next */
            throw Error('Unexpected fromNodeId in response.');
        }
        if (response.body.toNodeId !== __classPrivateFieldGet(this, _node).nodeId()) {
            /* istanbul ignore next */
            throw Error('Unexpected toNodeId in response.');
        }
        if (response.body.requestId !== requestId) {
            /* istanbul ignore next */
            throw Error('Unexpected requestId in response.');
        }
        if (Number(response.body.timestamp) < Number(request.body.timestamp) - 1000) {
            /* istanbul ignore next */
            throw Error('Unexpected early timestamp in response.');
        }
        if (!crypto_util_1.verifySignature(response.body, response.signature, core_1.nodeIdToPublicKey(__classPrivateFieldGet(this, _remoteNodeId)))) {
            // ban the node?
            /* istanbul ignore next */
            throw Error('Invalid signature in response.');
        }
        return response.body.responseData;
    }
    _getMostRecentChannelNodeInfo() {
        let result = null;
        __classPrivateFieldGet(this, _channelNodeInfoByChannel).forEach((channelNodeInfo) => {
            if ((result === null) || (channelNodeInfo.body.timestamp > result.body.timestamp)) {
                result = channelNodeInfo;
            }
        });
        return result;
    }
}
_node = new WeakMap(), _remoteNodeId = new WeakMap(), _channelNodeInfoByChannel = new WeakMap(), _isBootstrap = new WeakMap(), _bootstrapAddress = new WeakMap(), _bootstrapWebSocketAddress = new WeakMap(), _bootstrapUdpSocketAddress = new WeakMap(), _localUdpAddressForRemoteNode = new WeakMap(), _numUdpBytesSent = new WeakMap(), _numUdpBytesLost = new WeakMap(), _numHttpBytesSent = new WeakMap(), _numRequestsSent = new WeakMap(), _numResponsesReceived = new WeakMap(), _isOnline = new WeakMap(), _sendMessageMethodOptimizer = new WeakMap(), _downloadFileDataMethodOptimizer = new WeakMap();
const channelNodeInfoIsExpired = (cni) => {
    const elapsed = core_1.unscaledDurationMsec(core_1.elapsedSince(cni.body.timestamp));
    return (core_1.durationGreaterThan(elapsed, core_1.scaledDurationMsec(2 * 60 * 1000)));
};
exports.channelNodeInfoIsExpired = channelNodeInfoIsExpired;
exports.default = RemoteNode;
//# sourceMappingURL=RemoteNode.js.map