"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _remoteNodes, _onNodeChannelAddedCallbacks, _onBootstrapNodeAddedCallbacks, _recentWarnings;
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_util_1 = require("./common/crypto_util");
const GarbageMap_1 = __importDefault(require("./common/GarbageMap"));
const core_1 = require("./interfaces/core");
const RemoteNode_1 = __importStar(require("./RemoteNode"));
class RemoteNodeManager {
    constructor(node) {
        _node.set(this, void 0);
        _remoteNodes.set(this, new Map());
        _onNodeChannelAddedCallbacks.set(this, []);
        _onBootstrapNodeAddedCallbacks.set(this, []);
        _recentWarnings.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(1000 * 30)));
        __classPrivateFieldSet(this, _node, node);
    }
    async handleAnnounceRequest({ fromNodeId, requestData, localUdpAddress }) {
        // only handle this if we belong to this channel or we are a bootstrap node
        if (!__classPrivateFieldGet(this, _node).isBootstrapNode()) {
            if (!__classPrivateFieldGet(this, _node).hasJoinedChannel(requestData.channelNodeInfo.body.channelConfigUrl)) {
                return {
                    requestType: 'announce',
                    success: false,
                    errorMessage: core_1.errorMessage('Not a bootstrap and not a member of this channel.')
                };
            }
        }
        const { channelNodeInfo } = requestData;
        await this.setChannelNodeInfo(channelNodeInfo);
        if (localUdpAddress) {
            const n = __classPrivateFieldGet(this, _remoteNodes).get(channelNodeInfo.body.nodeId);
            if (n) {
                n.setLocalUdpAddressForRemoteNode(localUdpAddress);
            }
        }
        return {
            requestType: 'announce',
            success: true,
            errorMessage: null
        };
    }
    _printWarning(w) {
        if (__classPrivateFieldGet(this, _recentWarnings).has(w))
            return;
        __classPrivateFieldGet(this, _recentWarnings).set(w, true);
        console.warn(w);
    }
    async setChannelNodeInfo(channelNodeInfo) {
        const { body, signature } = channelNodeInfo;
        if (RemoteNode_1.channelNodeInfoIsExpired(channelNodeInfo))
            return;
        if (!crypto_util_1.verifySignature(body, signature, core_1.nodeIdToPublicKey(channelNodeInfo.body.nodeId))) {
            throw Error(`Invalid signature for channelNodeInfo: : ${body.nodeId} ${body.channelConfigUrl}`);
        }
        if (body.nodeId === __classPrivateFieldGet(this, _node).nodeId()) {
            throw Error('Cannot set channel node info for self');
        }
        if (!__classPrivateFieldGet(this, _remoteNodes).has(body.nodeId)) {
            const remoteNodeOpts = {
                isBootstrap: false,
                isBootstrapMessageProxy: false,
                isBootstrapDataProxy: false,
                bootstrapAddress: null,
                bootstrapWebSocketAddress: null,
                bootstrapUdpSocketAddress: null
            };
            __classPrivateFieldGet(this, _remoteNodes).set(body.nodeId, new RemoteNode_1.default(__classPrivateFieldGet(this, _node), body.nodeId, remoteNodeOpts));
        }
        const n = __classPrivateFieldGet(this, _remoteNodes).get(body.nodeId);
        /* istanbul ignore next */
        if (!n)
            throw Error('Unexpected in setChannelNodeInfo');
        const newChannel = (!n.getJoinedChannelConfigUrls().includes(channelNodeInfo.body.channelConfigUrl));
        n.setChannelNodeInfoIfMoreRecent(channelNodeInfo.body.channelConfigUrl, channelNodeInfo);
        if (newChannel) {
            __classPrivateFieldGet(this, _onNodeChannelAddedCallbacks).forEach(cb => {
                cb(n.remoteNodeId(), channelNodeInfo.body.channelConfigUrl);
            });
        }
    }
    async setBootstrapNode(remoteNodeId, address, webSocketAddress, udpSocketAddress) {
        const n = __classPrivateFieldGet(this, _remoteNodes).get(remoteNodeId);
        if (n) {
            if ((!n.isBootstrap()) || (!core_1.jsonObjectsMatch(n.bootstrapAddress(), address))) {
                __classPrivateFieldGet(this, _remoteNodes).delete(remoteNodeId);
            }
        }
        if (!__classPrivateFieldGet(this, _remoteNodes).has(remoteNodeId)) {
            const remoteNode = new RemoteNode_1.default(__classPrivateFieldGet(this, _node), remoteNodeId, {
                isBootstrap: true,
                bootstrapAddress: address,
                bootstrapWebSocketAddress: webSocketAddress,
                bootstrapUdpSocketAddress: udpSocketAddress
            });
            __classPrivateFieldGet(this, _remoteNodes).set(remoteNodeId, remoteNode);
            __classPrivateFieldGet(this, _onBootstrapNodeAddedCallbacks).forEach(cb => {
                cb(remoteNode.remoteNodeId());
            });
        }
    }
    async getChannelInfo(channelConfigUrl) {
        const nodes = [];
        __classPrivateFieldGet(this, _remoteNodes).forEach(n => {
            const cni = n.getRemoteChannelNodeInfo(channelConfigUrl);
            if (cni !== null) {
                nodes.push(cni);
            }
        });
        if (__classPrivateFieldGet(this, _node).hasJoinedChannel(channelConfigUrl)) {
            nodes.push(await __classPrivateFieldGet(this, _node).getChannelNodeInfo(channelConfigUrl));
        }
        return {
            nodes
        };
    }
    canSendRequestToNode(nodeId, method) {
        const remoteNode = __classPrivateFieldGet(this, _remoteNodes).get(nodeId);
        if (!remoteNode) {
            return false;
        }
        return remoteNode.canSendRequest(method);
    }
    canReceiveMessagesFromNode(nodeId) {
        const n = this.getRemoteNode(nodeId);
        return n ? true : false;
    }
    async sendRequestToNode(nodeId, requestData, opts) {
        if (!this.canSendRequestToNode(nodeId, opts.method)) {
            throw Error(`Cannot send request to node: ${nodeId}`);
        }
        const remoteNode = __classPrivateFieldGet(this, _remoteNodes).get(nodeId);
        if (!remoteNode) {
            throw Error(`Cannot send request to node: node with ID ${nodeId} not found.`);
        }
        return await remoteNode.sendRequest(requestData, { timeoutMsec: opts.timeoutMsec, method: opts.method });
    }
    onNodeChannelAdded(callback) {
        __classPrivateFieldGet(this, _onNodeChannelAddedCallbacks).push(callback);
    }
    onBootstrapNodeAdded(callback) {
        __classPrivateFieldGet(this, _onBootstrapNodeAddedCallbacks).push(callback);
    }
    getRemoteNodesInChannel(channelConfigUrl, args) {
        this._pruneRemoteNodes();
        const ret = [];
        __classPrivateFieldGet(this, _remoteNodes).forEach((n, nodeId) => {
            if ((args.includeOffline) || (n.isOnline())) {
                if (n.getJoinedChannelConfigUrls().includes(channelConfigUrl)) {
                    ret.push(n);
                }
            }
        });
        return ret;
    }
    getBootstrapRemoteNodes(args) {
        this._pruneRemoteNodes();
        const ret = [];
        __classPrivateFieldGet(this, _remoteNodes).forEach((n, nodeId) => {
            if (n.isBootstrap() && (n.bootstrapAddress() !== null)) {
                if ((args.includeOffline) || (n.isOnline())) {
                    ret.push(n);
                }
            }
        });
        return ret;
    }
    getAllRemoteNodes(args) {
        this._pruneRemoteNodes();
        const ret = [];
        __classPrivateFieldGet(this, _remoteNodes).forEach((n, nodeId) => {
            if ((args.includeOffline) || (n.isOnline())) {
                ret.push(n);
            }
        });
        return ret;
    }
    getRemoteNode(remoteNodeId) {
        return __classPrivateFieldGet(this, _remoteNodes).get(remoteNodeId) || null;
    }
    sendRequestToAllNodes(requestData, opts) {
        let finished = false;
        let numComplete = 0;
        let numTotal = 0;
        const _onResponseCallbacks = [];
        const _onErrorResponseCallbacks = [];
        const _onFinishedCallbacks = [];
        __classPrivateFieldGet(this, _remoteNodes).forEach(n => {
            if (this.canSendRequestToNode(n.remoteNodeId(), 'default')) {
                numTotal++;
                (async () => {
                    let responseData;
                    try {
                        responseData = await this.sendRequestToNode(n.remoteNodeId(), requestData, { timeoutMsec: opts.timeoutMsec, method: 'default' });
                    }
                    catch (err) {
                        _onErrorResponseCallbacks.forEach(cb => {
                            cb(n.remoteNodeId(), err);
                        });
                        numComplete++;
                        _checkComplete();
                        return;
                    }
                    if (finished)
                        return;
                    _onResponseCallbacks.forEach(cb => {
                        cb(n.remoteNodeId(), responseData);
                    });
                    numComplete++;
                    _checkComplete();
                })();
            }
        });
        const _finalize = () => {
            if (!finished) {
                finished = true;
                _onFinishedCallbacks.forEach(cb => { cb(); });
            }
        };
        const _cancel = () => {
            // at some point in the future - cancel the pending requests
            _finalize();
        };
        const onResponse = (callback) => {
            _onResponseCallbacks.push(callback);
        };
        const onErrorResponse = (callback) => {
            _onErrorResponseCallbacks.push(callback);
        };
        const onFinished = (callback) => {
            _onFinishedCallbacks.push(callback);
        };
        setTimeout(() => {
            _cancel();
        }, core_1.durationMsecToNumber(opts.timeoutMsec));
        const _checkComplete = () => {
            if (numComplete === numTotal) {
                setTimeout(() => {
                    _finalize();
                }, 0);
            }
        };
        return {
            onResponse,
            onErrorResponse,
            onFinished,
            cancel: _cancel
        };
    }
    remoteNodeIsOnline(nodeId) {
        const rn = __classPrivateFieldGet(this, _remoteNodes).get(nodeId);
        return (rn && rn.isOnline());
    }
    _pruneRemoteNodes() {
        const remoteNodeIds = __classPrivateFieldGet(this, _remoteNodes).keys();
        for (let nodeId of remoteNodeIds) {
            const rn = __classPrivateFieldGet(this, _remoteNodes).get(nodeId);
            if (!rn)
                throw Error('Unexpected in _pruneRemoteNodes');
            rn.pruneChannelNodeInfos();
            if (!rn.isBootstrap()) {
                if (rn.getJoinedChannelConfigUrls().length === 0) {
                    __classPrivateFieldGet(this, _remoteNodes).delete(nodeId);
                }
            }
        }
    }
}
_node = new WeakMap(), _remoteNodes = new WeakMap(), _onNodeChannelAddedCallbacks = new WeakMap(), _onBootstrapNodeAddedCallbacks = new WeakMap(), _recentWarnings = new WeakMap();
// function cloneObject(obj) {
//     if (!obj) return obj;
//     return JSON.parse(JSON.stringify(obj));
// }
exports.default = RemoteNodeManager;
//# sourceMappingURL=RemoteNodeManager.js.map