"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var _cancelled, _onCancelledCallbacks, _lastUnorderedDataIndex, _unorderedDataChunksByIndex, _unorderedEndNumDataChunks, _producer, _completed, _finished, _started, _size, _bytesLoaded, _error, _pendingDataChunks, _onStartedCallbacks, _onDataCallbacks, _onFinishedCallbacks, _onCompleteCallbacks, _onErrorCallbacks, _onProgressCallbacks;
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("../interfaces/core");
const memoryLeakTest = [];
class DataStreamyProducer {
    constructor(dataStream) {
        this.dataStream = dataStream;
        _cancelled.set(this, false);
        _onCancelledCallbacks.set(this, []);
        _lastUnorderedDataIndex.set(this, -1);
        _unorderedDataChunksByIndex.set(this, new Map());
        _unorderedEndNumDataChunks.set(this, null);
    }
    onCancelled(cb) {
        if (__classPrivateFieldGet(this, _cancelled)) {
            cb();
        }
        __classPrivateFieldGet(this, _onCancelledCallbacks).push(cb);
    }
    isCancelled() {
        return __classPrivateFieldGet(this, _cancelled);
    }
    error(err) {
        if (__classPrivateFieldGet(this, _cancelled))
            return;
        this.dataStream._producer_error(err);
    }
    start(size) {
        if (__classPrivateFieldGet(this, _cancelled))
            return;
        this.dataStream._producer_start(size);
    }
    end() {
        if (__classPrivateFieldGet(this, _cancelled))
            return;
        this.dataStream._producer_end();
    }
    data(buf) {
        if (__classPrivateFieldGet(this, _cancelled))
            return;
        // memoryLeakTest.push(buf)
        this.dataStream._producer_data(buf);
    }
    unorderedData(index, buf) {
        __classPrivateFieldGet(this, _unorderedDataChunksByIndex).set(index, buf);
        while (__classPrivateFieldGet(this, _unorderedDataChunksByIndex).has(__classPrivateFieldGet(this, _lastUnorderedDataIndex) + 1)) {
            __classPrivateFieldSet(this, _lastUnorderedDataIndex, +__classPrivateFieldGet(this, _lastUnorderedDataIndex) + 1);
            const buf = __classPrivateFieldGet(this, _unorderedDataChunksByIndex).get(__classPrivateFieldGet(this, _lastUnorderedDataIndex));
            /* istanbul ignore next */
            if (!buf)
                throw Error('Unexpected no buf in unorderedData');
            __classPrivateFieldGet(this, _unorderedDataChunksByIndex).delete(__classPrivateFieldGet(this, _lastUnorderedDataIndex));
            this.data(buf);
            if (__classPrivateFieldGet(this, _unorderedEndNumDataChunks) !== null) {
                if (__classPrivateFieldGet(this, _lastUnorderedDataIndex) === __classPrivateFieldGet(this, _unorderedEndNumDataChunks) - 1) {
                    this.end();
                }
                else if (__classPrivateFieldGet(this, _lastUnorderedDataIndex) > __classPrivateFieldGet(this, _unorderedEndNumDataChunks) - 1) {
                    throw Error('Unexpected lastUnorderedDataIndex');
                }
            }
        }
    }
    unorderedEnd(numDataChunks) {
        if (__classPrivateFieldGet(this, _lastUnorderedDataIndex) >= numDataChunks - 1) {
            this.end();
        }
        else {
            __classPrivateFieldSet(this, _unorderedEndNumDataChunks, numDataChunks);
        }
    }
    incrementBytes(numBytes) {
        if (__classPrivateFieldGet(this, _cancelled))
            return;
        this.dataStream._producer_incrementBytes(numBytes);
    }
    reportBytesLoaded(numBytes) {
        if (__classPrivateFieldGet(this, _cancelled))
            return;
        this.dataStream._producer_reportBytesLoaded(numBytes);
    }
    setProgress(progress) {
        this.dataStream._producer_setProgress(progress);
    }
    _cancel() {
        if (__classPrivateFieldGet(this, _cancelled))
            return;
        __classPrivateFieldSet(this, _cancelled, true);
        __classPrivateFieldGet(this, _onCancelledCallbacks).forEach(cb => { cb(); });
        this.dataStream._producer_error(Error('Cancelled'));
    }
}
_cancelled = new WeakMap(), _onCancelledCallbacks = new WeakMap(), _lastUnorderedDataIndex = new WeakMap(), _unorderedDataChunksByIndex = new WeakMap(), _unorderedEndNumDataChunks = new WeakMap();
class DataStreamy {
    constructor() {
        _producer.set(this, void 0);
        // state
        _completed.set(this, false);
        _finished.set(this, false);
        _started.set(this, false);
        _size.set(this, null);
        _bytesLoaded.set(this, core_1.byteCount(0));
        _error.set(this, null);
        _pendingDataChunks.set(this, []
        // callbacks
        );
        // callbacks
        _onStartedCallbacks.set(this, []);
        _onDataCallbacks.set(this, []);
        _onFinishedCallbacks.set(this, []);
        _onCompleteCallbacks.set(this, []);
        _onErrorCallbacks.set(this, []);
        _onProgressCallbacks.set(this, []);
        __classPrivateFieldSet(this, _producer, new DataStreamyProducer(this));
    }
    onStarted(callback) {
        if (__classPrivateFieldGet(this, _started)) {
            callback(__classPrivateFieldGet(this, _size));
        }
        __classPrivateFieldGet(this, _onStartedCallbacks).push(callback);
    }
    onData(callback) {
        if ((__classPrivateFieldGet(this, _onDataCallbacks).length > 0) && (core_1.byteCountToNumber(__classPrivateFieldGet(this, _bytesLoaded)) > 0)) {
            throw Error('onData already called in DataStreamy, and we have already received data');
        }
        __classPrivateFieldGet(this, _pendingDataChunks).forEach((ch) => {
            callback(ch);
        });
        __classPrivateFieldSet(this, _pendingDataChunks, []);
        __classPrivateFieldGet(this, _onDataCallbacks).push(callback);
    }
    async allData() {
        return new Promise((resolve, reject) => {
            const buffers = [];
            this.onData(buf => buffers.push(buf));
            this.onFinished(() => {
                resolve(Buffer.concat(buffers));
            });
            this.onError((err) => {
                reject(err);
            });
        });
    }
    onFinished(callback) {
        if (__classPrivateFieldGet(this, _finished)) {
            // important to use setTimeout here because we want to get data before finished (if both are already available)
            setTimeout(() => {
                callback();
            }, 0);
        }
        __classPrivateFieldGet(this, _onFinishedCallbacks).push(callback);
    }
    onError(callback) {
        if (__classPrivateFieldGet(this, _error)) {
            // I think it is important to use setTimeout here
            setTimeout(() => {
                if (!__classPrivateFieldGet(this, _error))
                    throw Error('error');
                callback(__classPrivateFieldGet(this, _error));
            }, 0);
        }
        __classPrivateFieldGet(this, _onErrorCallbacks).push(callback);
    }
    onProgress(callback) {
        if ((core_1.byteCountToNumber(__classPrivateFieldGet(this, _bytesLoaded)) > 0) && (__classPrivateFieldGet(this, _size))) {
            callback({ bytesLoaded: __classPrivateFieldGet(this, _bytesLoaded), bytesTotal: __classPrivateFieldGet(this, _size) });
        }
        __classPrivateFieldGet(this, _onProgressCallbacks).push(callback);
    }
    bytesLoaded() {
        return __classPrivateFieldGet(this, _bytesLoaded);
    }
    cancel() {
        if (__classPrivateFieldGet(this, _completed))
            return;
        __classPrivateFieldGet(this, _producer)._cancel();
    }
    isComplete() {
        return __classPrivateFieldGet(this, _completed);
    }
    producer() {
        return __classPrivateFieldGet(this, _producer);
    }
    _producer_error(err) {
        if (__classPrivateFieldGet(this, _completed))
            return;
        this._handle_complete();
        __classPrivateFieldSet(this, _error, err);
        __classPrivateFieldGet(this, _onErrorCallbacks).forEach(cb => { cb(err); });
    }
    _producer_start(size) {
        if (__classPrivateFieldGet(this, _completed))
            return;
        if (__classPrivateFieldGet(this, _started))
            return;
        __classPrivateFieldSet(this, _started, true);
        __classPrivateFieldSet(this, _size, size);
        __classPrivateFieldGet(this, _onStartedCallbacks).forEach(cb => {
            cb(size);
        });
    }
    _producer_end() {
        if (__classPrivateFieldGet(this, _completed))
            return;
        this._handle_complete();
        __classPrivateFieldSet(this, _finished, true);
        __classPrivateFieldGet(this, _onFinishedCallbacks).forEach(cb => { cb(); });
    }
    _handle_complete() {
        __classPrivateFieldSet(this, _completed, true);
        __classPrivateFieldGet(this, _onCompleteCallbacks).forEach(cb => { cb(); });
        if (__classPrivateFieldGet(this, _pendingDataChunks).length > 0) {
            setTimeout(() => {
                __classPrivateFieldSet(this, _pendingDataChunks, []);
            }, 1000);
        }
    }
    _producer_data(buf) {
        if (__classPrivateFieldGet(this, _completed))
            return;
        if (!__classPrivateFieldGet(this, _started)) {
            __classPrivateFieldSet(this, _started, true);
            __classPrivateFieldGet(this, _onStartedCallbacks).forEach(cb => {
                cb(null);
            });
        }
        __classPrivateFieldGet(this, _onDataCallbacks).forEach(cb => {
            cb(buf);
        });
        this._producer_incrementBytes(core_1.byteCount(buf.length));
        if (__classPrivateFieldGet(this, _onDataCallbacks).length === 0) {
            __classPrivateFieldGet(this, _pendingDataChunks).push(buf);
        }
    }
    _producer_incrementBytes(numBytes) {
        this._producer_reportBytesLoaded(core_1.byteCount(core_1.byteCountToNumber(__classPrivateFieldGet(this, _bytesLoaded)) + core_1.byteCountToNumber(numBytes)));
    }
    _producer_reportBytesLoaded(numBytes) {
        __classPrivateFieldSet(this, _bytesLoaded, numBytes);
        const s = __classPrivateFieldGet(this, _size);
        if (s !== null) {
            __classPrivateFieldGet(this, _onProgressCallbacks).forEach(cb => {
                cb({ bytesLoaded: __classPrivateFieldGet(this, _bytesLoaded), bytesTotal: s });
            });
        }
    }
    _producer_setProgress(progress) {
        __classPrivateFieldSet(this, _bytesLoaded, progress.bytesLoaded);
        if (progress.bytesTotal) {
            __classPrivateFieldSet(this, _size, progress.bytesTotal);
        }
        const s = __classPrivateFieldGet(this, _size);
        if (s !== null) {
            __classPrivateFieldGet(this, _onProgressCallbacks).forEach(cb => {
                cb({ bytesLoaded: __classPrivateFieldGet(this, _bytesLoaded), bytesTotal: s });
            });
        }
    }
}
exports.default = DataStreamy;
_producer = new WeakMap(), _completed = new WeakMap(), _finished = new WeakMap(), _started = new WeakMap(), _size = new WeakMap(), _bytesLoaded = new WeakMap(), _error = new WeakMap(), _pendingDataChunks = new WeakMap(), _onStartedCallbacks = new WeakMap(), _onDataCallbacks = new WeakMap(), _onFinishedCallbacks = new WeakMap(), _onCompleteCallbacks = new WeakMap(), _onErrorCallbacks = new WeakMap(), _onProgressCallbacks = new WeakMap();
//# sourceMappingURL=DataStreamy.js.map