"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _map, _expirationTimeoutMsec, _lastCheckTimestamp;
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("../interfaces/core");
class GarbageMap {
    constructor(expirationTimeoutMSec) {
        _map.set(this, new Map());
        _expirationTimeoutMsec.set(this, void 0);
        _lastCheckTimestamp.set(this, core_1.nowTimestamp());
        __classPrivateFieldSet(this, _expirationTimeoutMsec, expirationTimeoutMSec);
    }
    get(key) {
        this._checkGarbageCollection();
        const x = __classPrivateFieldGet(this, _map).get(key);
        if (x === undefined) {
            return undefined;
        }
        else {
            return x.value;
        }
    }
    getWithDefault(key, defaultValue) {
        this._checkGarbageCollection();
        const x = __classPrivateFieldGet(this, _map).get(key);
        if (x === undefined) {
            return defaultValue;
        }
        else {
            return x.value;
        }
    }
    set(key, value) {
        __classPrivateFieldGet(this, _map).set(key, {
            value,
            timestamp: core_1.nowTimestamp()
        });
        this._checkGarbageCollection();
    }
    delete(key) {
        if (__classPrivateFieldGet(this, _map).has(key))
            __classPrivateFieldGet(this, _map).delete(key);
        this._checkGarbageCollection();
    }
    has(key) {
        return __classPrivateFieldGet(this, _map).has(key);
    }
    keys() {
        return Array.from(__classPrivateFieldGet(this, _map).keys());
    }
    values() {
        return Array.from(__classPrivateFieldGet(this, _map).values()).map(v => v.value);
    }
    async _checkGarbageCollection() {
        const elapsedSinceCheck = core_1.elapsedSince(__classPrivateFieldGet(this, _lastCheckTimestamp));
        if (__classPrivateFieldGet(this, _expirationTimeoutMsec) !== null) {
            if (elapsedSinceCheck > core_1.durationMsecToNumber(__classPrivateFieldGet(this, _expirationTimeoutMsec)) / 2) {
                __classPrivateFieldSet(this, _lastCheckTimestamp, core_1.nowTimestamp());
                let keysToDelete = [];
                __classPrivateFieldGet(this, _map).forEach((v, k) => {
                    const elapsed = core_1.elapsedSince(v.timestamp);
                    if (!__classPrivateFieldGet(this, _expirationTimeoutMsec))
                        throw Error('Unexpected in _checkGarbageCollection');
                    if (elapsed > core_1.durationMsecToNumber(__classPrivateFieldGet(this, _expirationTimeoutMsec))) {
                        keysToDelete.push(k);
                    }
                });
                keysToDelete.forEach(k => {
                    __classPrivateFieldGet(this, _map).delete(k);
                });
            }
        }
    }
}
exports.default = GarbageMap;
_map = new WeakMap(), _expirationTimeoutMsec = new WeakMap(), _lastCheckTimestamp = new WeakMap();
//# sourceMappingURL=GarbageMap.js.map