"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _jobs, _tasks, _providerNodes, _maxNumSimultaneousFileDownloads, _updateScheduled, _onReadyListeners;
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../common/constants");
const DataStreamy_1 = __importDefault(require("../common/DataStreamy"));
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const createDownloader_1 = __importDefault(require("./createDownloader"));
const DownloadOptimizerJob_1 = __importDefault(require("./DownloadOptimizerJob"));
const DownloadOptimizerProviderNode_1 = __importDefault(require("./DownloadOptimizerProviderNode"));
class DownloadOptimizer {
    constructor(node) {
        this.node = node;
        _jobs.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(60 * 1000 * 60)));
        _tasks.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(60 * 1000 * 60)));
        _providerNodes.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(60 * 1000 * 60)));
        _maxNumSimultaneousFileDownloads.set(this, 5);
        _updateScheduled.set(this, false);
        _onReadyListeners.set(this, new Map());
    }
    async waitForReady() {
        let numActiveFileDownloads = Array.from(__classPrivateFieldGet(this, _jobs).values()).filter(file => (file.isDownloading())).length;
        if (numActiveFileDownloads < __classPrivateFieldGet(this, _maxNumSimultaneousFileDownloads))
            return;
        return new Promise((resolve) => {
            const id = util_1.randomAlphaString(10);
            __classPrivateFieldGet(this, _onReadyListeners).set(id, () => {
                // self-destruct
                __classPrivateFieldGet(this, _onReadyListeners).delete(id);
                resolve();
            });
        });
    }
    createTask(fileKey, fileSize, label, opts) {
        const taskId = util_1.randomAlphaString(10);
        const fkh = core_1.fileKeyHash(fileKey);
        const t = {
            taskId,
            fileKey,
            fileSize,
            progressStream: new DataStreamy_1.default()
        };
        let j = __classPrivateFieldGet(this, _jobs).get(fkh);
        if (!j) {
            const findProviders = (onFound, onFinished) => {
                // todo: what happens if we create 2 tasks, with different opts?
                if (opts.fromNode) {
                    const pn = this._getProviderNode(opts.fromNode);
                    onFound(pn);
                }
                else {
                    const ff = this.node.findFile({ fileKey, timeoutMsec: constants_1.TIMEOUTS.loadFileFindFile });
                    ff.onFound(result => {
                        if (result.nodeId !== this.node.nodeId()) {
                            const pn = this._getProviderNode(result.nodeId);
                            onFound(pn);
                        }
                    });
                    ff.onFinished(() => {
                        onFinished();
                    });
                }
            };
            const createDownloader0 = (providerNode, fileSize, label) => {
                return createDownloader_1.default(this.node, fileKey, providerNode, fileSize, label);
            };
            j = new DownloadOptimizerJob_1.default(fileKey, fileSize, label, findProviders, createDownloader0, { numRetries: opts.numRetries });
            __classPrivateFieldGet(this, _jobs).set(fkh, j);
        }
        j.getProgressStream().onStarted((byteCount) => {
            t.progressStream.producer().start(byteCount);
        });
        j.getProgressStream().onProgress((progress) => {
            t.progressStream.producer().setProgress(progress);
        });
        j.getProgressStream().onError((err) => {
            t.progressStream.producer().error(err);
            this._deleteTask(fkh, taskId);
        });
        j.getProgressStream().onFinished(() => {
            t.progressStream.producer().end();
            this._deleteTask(fkh, taskId);
        });
        t.progressStream.producer().onCancelled(() => {
            this._deleteTask(fkh, taskId);
        });
        let tasks0 = __classPrivateFieldGet(this, _tasks).get(fkh);
        if (!tasks0) {
            tasks0 = new Map();
            __classPrivateFieldGet(this, _tasks).set(fkh, tasks0);
        }
        tasks0.set(t.taskId, t);
        this._scheduleUpdate();
        return t.progressStream;
    }
    _deleteTask(fileKeyHash, taskId) {
        const tasks = __classPrivateFieldGet(this, _tasks).get(fileKeyHash);
        if (!tasks)
            return;
        const t = tasks.get(taskId);
        if (!t)
            return;
        tasks.delete(taskId);
        if (tasks.size === 0) {
            const j = __classPrivateFieldGet(this, _jobs).get(fileKeyHash);
            if (j) {
                j.cancel();
                __classPrivateFieldGet(this, _jobs).delete(fileKeyHash);
            }
        }
        this._scheduleUpdate();
    }
    _getProviderNode(nodeId) {
        let p = __classPrivateFieldGet(this, _providerNodes).get(nodeId);
        if (p)
            return p;
        const pNew = new DownloadOptimizerProviderNode_1.default(nodeId);
        __classPrivateFieldGet(this, _providerNodes).set(nodeId, pNew);
        return pNew;
    }
    // _setProviderNodeForFile(fileKey: FileKey, nodeId: NodeId) {
    //     if (!this.#providerNodes.has(nodeId)) {
    //         const p = new DownloadOptimizerProviderNode(nodeId)
    //         this.#providerNodes.set(nodeId, p)
    //     }
    //     const fkh = fileKeyHash(fileKey)
    //     let s: Set<NodeId> | null = this.#providerNodesForFiles.get(fkh) || null
    //     if (s == null) {
    //         s = new Set<NodeId>()
    //         this.#providerNodesForFiles.set(fkh, s)
    //     }
    //     s.add(nodeId)
    //     this._scheduleUpdate()
    // }
    _scheduleUpdate() {
        if (__classPrivateFieldGet(this, _updateScheduled))
            return;
        __classPrivateFieldSet(this, _updateScheduled, true);
        setTimeout(() => {
            __classPrivateFieldSet(this, _updateScheduled, false);
            this._update().then(() => {
            }).catch((err) => {
                console.error(`Unexpected error updating download optimizer: ${err.message}`);
            });
        }, 1);
    }
    async _update() {
        // delete the complete jobs
        const keys = __classPrivateFieldGet(this, _jobs).keys();
        for (let key of keys) {
            const j = __classPrivateFieldGet(this, _jobs).get(key);
            if (!j)
                throw Error('Unexpected j is null in _update');
            if (j.isComplete()) {
                __classPrivateFieldGet(this, _jobs).delete(key);
            }
        }
        let numActiveFileDownloads = __classPrivateFieldGet(this, _jobs).values().filter(j => (j.isRunning())).length;
        for (let fkh of __classPrivateFieldGet(this, _jobs).keys()) {
            if (numActiveFileDownloads < __classPrivateFieldGet(this, _maxNumSimultaneousFileDownloads)) {
                const job = __classPrivateFieldGet(this, _jobs).get(fkh);
                /* istanbul ignore next */
                if (!job)
                    throw Error('Unexpected in _update');
                if ((!job.isRunning()) && (!job.isComplete())) {
                    job.start();
                    numActiveFileDownloads++;
                }
            }
        }
        if (numActiveFileDownloads < __classPrivateFieldGet(this, _maxNumSimultaneousFileDownloads)) {
            __classPrivateFieldGet(this, _onReadyListeners).forEach(listener => {
                // will self-destruct
                listener();
            });
        }
    }
}
exports.default = DownloadOptimizer;
_jobs = new WeakMap(), _tasks = new WeakMap(), _providerNodes = new WeakMap(), _maxNumSimultaneousFileDownloads = new WeakMap(), _updateScheduled = new WeakMap(), _onReadyListeners = new WeakMap();
const chooseFastestProviderNode = (providerNodeList) => {
    if (providerNodeList.length === 0) {
        return null;
    }
    const estimatedRates = providerNodeList.map(p => p.estimatedRateBps()).map(x => Number(x));
    const bestIndex = argMax(estimatedRates);
    return providerNodeList[bestIndex];
};
// thanks: https://gist.github.com/engelen/fbce4476c9e68c52ff7e5c2da5c24a28
function argMax(array) {
    return array.map((x, i) => [x, i]).reduce((r, a) => (a[0] > r[0] ? a : r))[1];
}
//# sourceMappingURL=DownloadOptimizer.js.map