"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var _currentEstimateTimestamp, _currentEstimate, _running;
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("../interfaces/core");
const UdpCongestionManager_1 = require("../udp/UdpCongestionManager");
const FILTER_TIME_CONSTANT_SEC = 10;
class RateEstimator {
    constructor() {
        _currentEstimateTimestamp.set(this, core_1.nowTimestamp());
        _currentEstimate.set(this, UdpCongestionManager_1.byteCountPerSec(1000 * 1000));
        _running.set(this, false);
    }
    estimatedRateBps() {
        if (__classPrivateFieldGet(this, _running)) {
            this.reportBytes(core_1.byteCount(0));
        }
        return __classPrivateFieldGet(this, _currentEstimate);
    }
    reportStart() {
        if (__classPrivateFieldGet(this, _running)) {
            /* istanbul ignore next */
            throw Error('Cannot start when already running');
        }
        __classPrivateFieldSet(this, _currentEstimateTimestamp, core_1.nowTimestamp());
        __classPrivateFieldSet(this, _running, true);
    }
    reportStop() {
        if (!__classPrivateFieldGet(this, _running)) {
            /* istanbul ignore next */
            throw Error('Cannot stop when not running');
        }
        this.reportBytes(core_1.byteCount(0));
        __classPrivateFieldSet(this, _running, false);
    }
    reportBytes(numBytes) {
        if (!__classPrivateFieldGet(this, _running)) {
            /* istanbul ignore next */
            throw Error('Cannot report bytes when not running');
        }
        const elapsedSecSinceLastEstimate = core_1.elapsedSince(__classPrivateFieldGet(this, _currentEstimateTimestamp)) / 1000;
        if (elapsedSecSinceLastEstimate > 0) {
            // update estimate based on numBytes, elapsedSinceLastEstimate
            const recentActivityEstimate = core_1.byteCountToNumber(numBytes) / elapsedSecSinceLastEstimate;
            const filterTimeConstant = FILTER_TIME_CONSTANT_SEC;
            const p = Math.exp(-elapsedSecSinceLastEstimate / filterTimeConstant);
            const newEstimate = recentActivityEstimate * (1 - p) + UdpCongestionManager_1.byteCountPerSecToNumber(__classPrivateFieldGet(this, _currentEstimate)) * p;
            __classPrivateFieldSet(this, _currentEstimate, UdpCongestionManager_1.byteCountPerSec(newEstimate));
        }
        __classPrivateFieldSet(this, _currentEstimateTimestamp, core_1.nowTimestamp());
    }
    isRunning() {
        return __classPrivateFieldGet(this, _running);
    }
}
exports.default = RateEstimator;
_currentEstimateTimestamp = new WeakMap(), _currentEstimate = new WeakMap(), _running = new WeakMap();
//# sourceMappingURL=RateEstimator.js.map