"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _mockFiles;
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
const DataStreamy_1 = __importDefault(require("../../common/DataStreamy"));
const core_1 = require("../../interfaces/core");
class MockKacheryStorageManager {
    constructor(getDefects) {
        this.getDefects = getDefects;
        _mockFiles.set(this, new Map());
    }
    async findFile(fileKey) {
        const content = __classPrivateFieldGet(this, _mockFiles).get(fileKey.sha1);
        if (content) {
            return {
                found: true,
                size: core_1.byteCount(content.length),
                localFilePath: core_1.localFilePath('mock-' + fileKey.sha1)
            };
        }
        else {
            return {
                found: false,
                size: core_1.byteCount(0),
                localFilePath: null
            };
        }
    }
    async hasLocalFile(fileKey) {
        return __classPrivateFieldGet(this, _mockFiles).has(fileKey.sha1);
    }
    async getFileReadStream(fileKey) {
        const content = __classPrivateFieldGet(this, _mockFiles).get(fileKey.sha1);
        if (content) {
            const ret = new DataStreamy_1.default();
            setTimeout(() => {
                ret.producer().start(core_1.byteCount(content.length));
                if (this.getDefects().fileReadDefect) {
                    ret.producer().error(Error('fileReadDefect'));
                    return;
                }
                ret.producer().data(content);
                ret.producer().end();
            }, 2);
            return ret;
        }
        else {
            throw Error(`File not found: ${fileKey.sha1}`);
        }
    }
    async storeFile(sha1, data) {
        const fileKey = this.addMockFile(data, { chunkSize: core_1.byteCount(data.length) });
        if (fileKey.sha1 !== sha1) {
            throw Error(`Unexpected hash for storing file: ${fileKey.sha1} <> ${sha1}`);
        }
    }
    async storeLocalFile(localFilePath) {
        throw Error('Not implemented in MockKacheryStorageManager');
    }
    async storeFileFromStream(stream, fileSize) {
        throw Error('Not implemented in MockKacheryStorageManager');
    }
    async concatenateChunksAndStoreResult(sha1Concat, chunkSha1s) {
        const chunks = [];
        chunkSha1s.forEach(sha1 => {
            const content = __classPrivateFieldGet(this, _mockFiles).get(sha1);
            if (content) {
                chunks.push(content);
            }
            else {
                throw Error('Unable to find chunk in mock kachery storage');
            }
        });
        const buf = Buffer.concat(chunks);
        this.storeFile(sha1Concat, buf);
    }
    addMockFile(content, opts) {
        const shasum = crypto_1.default.createHash('sha1');
        shasum.update(content);
        const sha1 = shasum.digest('hex');
        if (content.length > core_1.byteCountToNumber(opts.chunkSize)) {
            const manifest = this._createFileManifest(content, opts.chunkSize);
            const manifestContent = Buffer.from(JSON.stringify(manifest));
            const manifestKey = this.addMockFile(manifestContent, { chunkSize: core_1.byteCount(manifestContent.length) });
            return {
                sha1,
                manifestSha1: manifestKey.sha1
            };
        }
        else {
            __classPrivateFieldGet(this, _mockFiles).set(sha1, content);
            return {
                sha1
            };
        }
    }
    storageDir() {
        return core_1.localFilePath('<mock>');
    }
    _createFileManifest(content, chunkSize) {
        var shasum = crypto_1.default.createHash('sha1');
        shasum.update(content);
        const sha1 = shasum.digest('hex');
        const chunks = [];
        let i = 0;
        while (i < content.length) {
            const i2 = Math.min(content.length, i + core_1.byteCountToNumber(chunkSize));
            const chunkContent = content.slice(i, i2);
            const chunkSha1Sum = crypto_1.default.createHash('sha1');
            chunkSha1Sum.update(chunkContent);
            const chunkSha1 = chunkSha1Sum.digest('hex');
            this.storeFile(chunkSha1, chunkContent);
            chunks.push({
                start: core_1.byteCount(i),
                end: core_1.byteCount(i2),
                sha1: chunkSha1
            });
            i += core_1.byteCountToNumber(chunkSize);
        }
        return {
            sha1,
            size: core_1.byteCount(content.length),
            chunks
        };
    }
}
exports.default = MockKacheryStorageManager;
_mockFiles = new WeakMap();
//# sourceMappingURL=MockKacheryStorageManager.js.map