"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var _feeds, _feedIdsByFeedName, _feedId, _privateKey, _subfeeds, _signedSubfeedMessages, _subfeedAccessRules;
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_util_1 = require("../../common/crypto_util");
class MockLocalFeedManager {
    constructor() {
        _feeds.set(this, new Map());
        _feedIdsByFeedName.set(this, new Map());
    }
    async createFeed(feedName) {
        const f = new MockFeed(null);
        __classPrivateFieldGet(this, _feeds).set(f.feedId(), f);
        if (feedName !== null) {
            __classPrivateFieldGet(this, _feedIdsByFeedName).set(feedName, f.feedId());
        }
        return f.feedId();
    }
    async deleteFeed(feedId) {
        __classPrivateFieldGet(this, _feeds).delete(feedId);
    }
    async getFeedId(feedName) {
        return __classPrivateFieldGet(this, _feedIdsByFeedName).get(feedName) || null;
    }
    async hasWriteableFeed(feedId) {
        const f = __classPrivateFieldGet(this, _feeds).get(feedId);
        if (!f)
            return false;
        return f.isWriteable();
    }
    async getPrivateKeyForFeed(feedId) {
        const f = __classPrivateFieldGet(this, _feeds).get(feedId);
        if (!f)
            return null;
        const k = f.privateKey();
        if (!k)
            return null;
        return k;
    }
    async feedExistsLocally(feedId) {
        return __classPrivateFieldGet(this, _feeds).has(feedId);
    }
    async getSignedSubfeedMessages(feedId, subfeedHash) {
        const f = __classPrivateFieldGet(this, _feeds).get(feedId);
        if (!f)
            return [];
        return await f.getSignedSubfeedMessages(subfeedHash);
    }
    async getSubfeedAccessRules(feedId, subfeedHash) {
        const f = __classPrivateFieldGet(this, _feeds).get(feedId);
        if (!f)
            return null;
        return await f.getSubfeedAccessRules(subfeedHash);
    }
    async appendSignedMessagesToSubfeed(feedId, subfeedHash, messages) {
        let f = __classPrivateFieldGet(this, _feeds).get(feedId);
        if (!f) {
            f = new MockFeed(feedId);
            __classPrivateFieldGet(this, _feeds).set(feedId, f);
        }
        await f.appendSignedMessagesToSubfeed(subfeedHash, messages);
    }
    async setSubfeedAccessRules(feedId, subfeedHash, accessRules) {
        const f = __classPrivateFieldGet(this, _feeds).get(feedId);
        if (!f) {
            throw Error('No feed');
        }
        if (!f.isWriteable()) {
            throw Error('Not writeable');
        }
        await f.setSubfeedAccessRules(subfeedHash, accessRules);
    }
}
exports.default = MockLocalFeedManager;
_feeds = new WeakMap(), _feedIdsByFeedName = new WeakMap();
class MockFeed {
    constructor(feedId) {
        _feedId.set(this, void 0);
        _privateKey.set(this, null);
        _subfeeds.set(this, new Map());
        if (feedId !== null) {
            __classPrivateFieldSet(this, _feedId, feedId);
        }
        else {
            const { publicKey, privateKey } = crypto_util_1.createKeyPair();
            __classPrivateFieldSet(this, _privateKey, privateKey);
            __classPrivateFieldSet(this, _feedId, crypto_util_1.publicKeyHexToFeedId(crypto_util_1.publicKeyToHex(publicKey)));
        }
    }
    feedId() {
        return __classPrivateFieldGet(this, _feedId);
    }
    isWriteable() {
        return __classPrivateFieldGet(this, _privateKey) !== null;
    }
    privateKey() {
        return __classPrivateFieldGet(this, _privateKey);
    }
    async getSignedSubfeedMessages(subfeedHash) {
        const s = __classPrivateFieldGet(this, _subfeeds).get(subfeedHash);
        if (!s)
            return [];
        return await s.getSignedSubfeedMessages();
    }
    async getSubfeedAccessRules(subfeedHash) {
        const s = __classPrivateFieldGet(this, _subfeeds).get(subfeedHash);
        if (!s)
            return null;
        return await s.getSubfeedAccessRules();
    }
    async appendSignedMessagesToSubfeed(subfeedHash, messages) {
        let s = __classPrivateFieldGet(this, _subfeeds).get(subfeedHash);
        if (!s) {
            s = new MockSubfeed();
            __classPrivateFieldGet(this, _subfeeds).set(subfeedHash, s);
        }
        await s.appendSignedMessagesToSubfeed(messages);
    }
    async setSubfeedAccessRules(subfeedHash, accessRules) {
        let s = __classPrivateFieldGet(this, _subfeeds).get(subfeedHash);
        if (!s) {
            s = new MockSubfeed();
            __classPrivateFieldGet(this, _subfeeds).set(subfeedHash, s);
        }
        await s.setSubfeedAccessRules(accessRules);
    }
}
_feedId = new WeakMap(), _privateKey = new WeakMap(), _subfeeds = new WeakMap();
class MockSubfeed {
    constructor() {
        _signedSubfeedMessages.set(this, []);
        _subfeedAccessRules.set(this, null);
    }
    async getSignedSubfeedMessages() {
        return __classPrivateFieldGet(this, _signedSubfeedMessages);
    }
    async getSubfeedAccessRules() {
        return __classPrivateFieldGet(this, _subfeedAccessRules);
    }
    async appendSignedMessagesToSubfeed(messages) {
        __classPrivateFieldSet(this, _signedSubfeedMessages, [...__classPrivateFieldGet(this, _signedSubfeedMessages), ...messages]);
    }
    async setSubfeedAccessRules(accessRules) {
        __classPrivateFieldSet(this, _subfeedAccessRules, accessRules);
    }
}
_signedSubfeedMessages = new WeakMap(), _subfeedAccessRules = new WeakMap();
//# sourceMappingURL=MockLocalFeedManager.js.map