"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _daemonGroup, _d, _defects, _daemons;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MockNodeDaemonGroup = void 0;
const util_1 = require("../../common/util");
const core_1 = require("../../interfaces/core");
const startDaemon_1 = __importDefault(require("../../startDaemon"));
const mockExternalInterface_1 = __importDefault(require("./mockExternalInterface"));
class MockNodeDaemon {
    constructor(daemonGroup, opts) {
        this.opts = opts;
        _daemonGroup.set(this, void 0);
        _d.set(this, null);
        _defects.set(this, {});
        __classPrivateFieldSet(this, _daemonGroup, daemonGroup);
    }
    address() {
        return {
            hostName: core_1.hostName(this.nodeId().toString()),
            port: core_1.toPort(20)
        };
    }
    defects() {
        return __classPrivateFieldGet(this, _defects);
    }
    setDefects(defects) {
        __classPrivateFieldSet(this, _defects, defects);
    }
    clearDefects() {
        this.setDefects({});
    }
    async initialize() {
        const externalInterface = mockExternalInterface_1.default(__classPrivateFieldGet(this, _daemonGroup), () => (this.defects()));
        __classPrivateFieldSet(this, _d, await startDaemon_1.default({
            configDir: null,
            verbose: 0,
            hostName: null,
            publicUrl: null,
            daemonApiPort: null,
            httpListenPort: null,
            label: core_1.nodeLabel('mock-daemon-' + util_1.randomAlphaString(5)),
            externalInterface,
            getDefects: () => (__classPrivateFieldGet(this, _defects)),
            opts: this.opts
        }));
    }
    stop() {
        if (__classPrivateFieldGet(this, _d)) {
            __classPrivateFieldGet(this, _d).stop();
        }
    }
    node() {
        if (!__classPrivateFieldGet(this, _d)) {
            /* istanbul ignore next */
            throw Error('mock daemon not yet initialized');
        }
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _d).node)
            throw Error('Unexpected');
        return __classPrivateFieldGet(this, _d).node;
    }
    remoteNodeManager() {
        return this.node().remoteNodeManager();
    }
    nodeId() {
        return this.node().nodeId();
    }
    mockKacheryStorageManager() {
        return this.node().kacheryStorageManager();
    }
    // feedManager() {
    //     return this.node().feedManager()
    // }
    async mockPublicApiPost(path, data) {
        if (!__classPrivateFieldGet(this, _d)) {
            /* istanbul ignore next */
            throw Error('mock daemon not yet initialized');
        }
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _d).publicApiServer)
            throw Error('unexpected');
        return await __classPrivateFieldGet(this, _d).publicApiServer.mockPostJson(path, data);
    }
    async mockPublicApiGetDownload(path) {
        if (!__classPrivateFieldGet(this, _d)) {
            /* istanbul ignore next */
            throw Error('mock daemon not yet initialized');
        }
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _d).publicApiServer)
            throw Error('unexpected');
        return await __classPrivateFieldGet(this, _d).publicApiServer.mockGetDownload(path);
    }
    async mockDaemonApiPost(path, data) {
        if (!__classPrivateFieldGet(this, _d)) {
            /* istanbul ignore next */
            throw Error('mock daemon not yet initialized');
        }
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _d).daemonApiServer)
            throw Error('unexpected');
        return await __classPrivateFieldGet(this, _d).daemonApiServer.mockPostJson(path, data);
    }
    mockDaemonApiServer() {
        if (!__classPrivateFieldGet(this, _d)) {
            /* istanbul ignore next */
            throw Error('mock daemon not yet initialized');
        }
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _d).daemonApiServer)
            throw Error('unexpected');
        return __classPrivateFieldGet(this, _d).daemonApiServer;
    }
    async mockDaemonPostFindFile(reqData) {
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _d))
            throw Error('Unexpected. Mock daemon not yet initialized');
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _d).daemonApiServer)
            throw Error('unexpected');
        return await __classPrivateFieldGet(this, _d).daemonApiServer.mockPostFindFile(reqData);
    }
}
exports.default = MockNodeDaemon;
_daemonGroup = new WeakMap(), _d = new WeakMap(), _defects = new WeakMap();
class MockNodeDaemonGroup {
    constructor() {
        _daemons.set(this, new Map());
    }
    stop() {
        __classPrivateFieldGet(this, _daemons).forEach((daemon, nodeId) => {
            daemon.stop();
        });
        // this.#daemons.clear()
    }
    async createDaemon(opts, initialDefects) {
        const daemon = new MockNodeDaemon(this, opts);
        if (initialDefects) {
            daemon.setDefects(initialDefects);
        }
        await daemon.initialize();
        __classPrivateFieldGet(this, _daemons).set(daemon.nodeId(), daemon);
        await util_1.sleepMsec(core_1.scaledDurationMsec(1000));
        return daemon;
    }
    getDaemon(nodeId) {
        return __classPrivateFieldGet(this, _daemons).get(nodeId) || null;
    }
    async mockHttpPostJson(address, path, data, opts) {
        if (!address.hostName)
            throw Error('Unexpected in mockHttpPostJson');
        const nodeId = address.hostName.toString();
        if (core_1.isNodeId(nodeId)) {
            const daemon = this.getDaemon(nodeId);
            if (!daemon) {
                /* istanbul ignore next */
                throw Error(`No daemon: ${nodeId}`);
            }
            const responseMessage = await daemon.mockPublicApiPost(path.toString(), data);
            return responseMessage;
        }
        /* istanbul ignore next */
        throw Error('mock - unable to process http post json');
    }
    async mockHttpGetDownload(address, path, stats, opts) {
        if (!address.hostName)
            throw Error('Unexpected in mockHttpGetDownload');
        const nodeId = address.hostName.toString();
        if (core_1.isNodeId(nodeId)) {
            const daemon = this.getDaemon(nodeId);
            if (!daemon) {
                /* istanbul ignore next */
                throw Error(`No daemon: ${nodeId}`);
            }
            const ds = await daemon.mockPublicApiGetDownload(path.toString());
            ds.onData(d => {
                stats.reportBytesReceived('http', opts.fromNodeId, core_1.byteCount(d.length));
            });
            return ds;
        }
        else {
            /* istanbul ignore next */
            throw Error('mock - unable to process http get download');
        }
    }
}
exports.MockNodeDaemonGroup = MockNodeDaemonGroup;
_daemons = new WeakMap();
//# sourceMappingURL=MockNodeDaemon.js.map