"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _incomingConnections, _onListeningCallbacks, _onConnectionCallbacks, _listening, _id_1, _onOpenCallbacks, _onCloseCallbacks, _onErrorCallbacks, _onMessageCallbacks, _open, _companion, _webSockets, _webSocketServers;
Object.defineProperty(exports, "__esModule", { value: true });
exports.mockCreateWebSocket = exports.mockStartWebSocketServer = void 0;
const util_1 = require("../../common/util");
const core_1 = require("../../interfaces/core");
const mockWebSocketUrl = (x) => {
    return x;
};
const createMockWebSocketId = () => {
    return util_1.randomAlphaString(10);
};
class MockWebSocketServer {
    constructor(port, nodeId) {
        this.port = port;
        this.nodeId = nodeId;
        _incomingConnections.set(this, new Map());
        _onListeningCallbacks.set(this, []);
        _onConnectionCallbacks.set(this, []);
        _listening.set(this, false);
        mockWebSocketManager.startServer(port, nodeId, this);
        __classPrivateFieldSet(this, _listening, true);
    }
    onListening(callback) {
        if (__classPrivateFieldGet(this, _listening)) {
            setTimeout(() => {
                callback();
            }, 1);
        }
        __classPrivateFieldGet(this, _onListeningCallbacks).push(callback);
    }
    onConnection(callback) {
        __classPrivateFieldGet(this, _incomingConnections).forEach(ws => {
            setTimeout(() => {
                callback(ws);
            }, 1);
        });
        __classPrivateFieldGet(this, _onConnectionCallbacks).push(callback);
    }
    close() {
        __classPrivateFieldGet(this, _incomingConnections).forEach(ws => {
            ws.close();
        });
        mockWebSocketManager.stopServer(this.port, this.nodeId);
    }
    _addConnection(ws) {
        const incomingWs = new MockWebSocket(null, { timeoutMsec: core_1.scaledDurationMsec(0) });
        incomingWs._setCompanion(ws);
        __classPrivateFieldGet(this, _incomingConnections).set(ws._id(), ws);
        __classPrivateFieldGet(this, _onConnectionCallbacks).forEach(cb => {
            cb(incomingWs);
        });
    }
}
_incomingConnections = new WeakMap(), _onListeningCallbacks = new WeakMap(), _onConnectionCallbacks = new WeakMap(), _listening = new WeakMap();
class MockWebSocket {
    constructor(url, opts) {
        _id_1.set(this, createMockWebSocketId());
        _onOpenCallbacks.set(this, []);
        _onCloseCallbacks.set(this, []);
        _onErrorCallbacks.set(this, []);
        _onMessageCallbacks.set(this, []);
        _open.set(this, false);
        _companion.set(this, null);
        if (url) {
            this._connectToServer(url, opts);
        }
    }
    onOpen(callback) {
        if (__classPrivateFieldGet(this, _open)) {
            callback();
        }
        __classPrivateFieldGet(this, _onOpenCallbacks).push(callback);
    }
    onClose(callback) {
        __classPrivateFieldGet(this, _onCloseCallbacks).push(callback);
    }
    onError(callback) {
        __classPrivateFieldGet(this, _onErrorCallbacks).push(callback);
    }
    onMessage(callback) {
        __classPrivateFieldGet(this, _onMessageCallbacks).push(callback);
    }
    close() {
        if (__classPrivateFieldGet(this, _open)) {
            __classPrivateFieldSet(this, _open, false);
            __classPrivateFieldGet(this, _onCloseCallbacks).forEach(cb => { cb(0, ''); });
            if (__classPrivateFieldGet(this, _companion)) {
                __classPrivateFieldGet(this, _companion).close();
            }
            __classPrivateFieldSet(this, _companion, null);
        }
    }
    send(buf) {
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _open)) {
            this._handleError(Error('Cannot send, websocket is closed'));
            return;
        }
        /* istanbul ignore next */
        if (!__classPrivateFieldGet(this, _companion)) {
            this._handleError(Error('Cannot send, no websocket companion'));
            return;
        }
        const companion = __classPrivateFieldGet(this, _companion);
        setTimeout(() => {
            companion._handleIncomingMessage(buf);
        }, 2);
    }
    _id() {
        return __classPrivateFieldGet(this, _id_1);
    }
    _connectToServer(url, opts) {
        (async () => {
            const timer = core_1.nowTimestamp();
            while (true) {
                const S = mockWebSocketManager.findServer(url);
                if (S) {
                    S._addConnection(this);
                    return;
                }
                else {
                    if (core_1.elapsedSince(timer) > core_1.durationMsecToNumber(opts.timeoutMsec)) {
                        this._handleError(Error(`Unable to find websocket server: ${url}`));
                        return;
                    }
                    else {
                        await util_1.sleepMsecNum(1);
                    }
                }
            }
        })();
    }
    _handleError(err) {
        __classPrivateFieldGet(this, _onErrorCallbacks).forEach(cb => {
            cb(err);
        });
        if (__classPrivateFieldGet(this, _open)) {
            this.close();
        }
    }
    _handleIncomingMessage(buf) {
        __classPrivateFieldGet(this, _onMessageCallbacks).forEach(cb => {
            cb(buf);
        });
    }
    _setCompanion(c) {
        if (!__classPrivateFieldGet(this, _open)) {
            __classPrivateFieldSet(this, _open, true);
            __classPrivateFieldSet(this, _companion, c);
            c._setCompanion(this);
            __classPrivateFieldGet(this, _onOpenCallbacks).forEach(cb => { cb(); });
        }
    }
}
exports.default = MockWebSocket;
_id_1 = new WeakMap(), _onOpenCallbacks = new WeakMap(), _onCloseCallbacks = new WeakMap(), _onErrorCallbacks = new WeakMap(), _onMessageCallbacks = new WeakMap(), _open = new WeakMap(), _companion = new WeakMap();
class MockWebSocketManager {
    constructor() {
        _webSockets.set(this, new Map());
        _webSocketServers.set(this, new Map());
    }
    startServer(port, nodeId, server) {
        const url = mockWebSocketUrl(`ws://${nodeId.toString()}:${port}`);
        __classPrivateFieldGet(this, _webSocketServers).set(url, server);
    }
    stopServer(port, nodeId) {
        const url = mockWebSocketUrl(`ws://${nodeId.toString()}:${port}`);
        __classPrivateFieldGet(this, _webSocketServers).delete(url);
    }
    findServer(url) {
        return __classPrivateFieldGet(this, _webSocketServers).get(url) || null;
    }
}
_webSockets = new WeakMap(), _webSocketServers = new WeakMap();
const mockWebSocketManager = new MockWebSocketManager();
const mockStartWebSocketServer = async (port, nodeId) => {
    const wss = new MockWebSocketServer(port, nodeId);
    return new Promise((resolve, reject) => {
        wss.onListening(() => {
            resolve(wss);
        });
    });
};
exports.mockStartWebSocketServer = mockStartWebSocketServer;
const mockCreateWebSocket = (url, opts) => {
    return new MockWebSocket(url, opts);
};
exports.mockCreateWebSocket = mockCreateWebSocket;
//# sourceMappingURL=MockWebSocket.js.map