"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var _boundSockets, _id, _type, _reuseAddr, _hostName, _bound, _port, _onListeningCallbacks, _onMessageCallbacks, _memberships, _firewalledAddresses, _outgoingAddresses;
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../../common/util");
const core_1 = require("../../interfaces/core");
class MockDgramSocketManager {
    constructor() {
        _boundSockets.set(this, []);
    }
    bind(id, port, socket) {
        __classPrivateFieldGet(this, _boundSockets).push({
            id,
            socket,
            port
        });
    }
    unbind(id) {
        __classPrivateFieldSet(this, _boundSockets, __classPrivateFieldGet(this, _boundSockets).filter(x => (x.id !== id)));
    }
    socketsBelongingToAddress(address, port, fromAddress) {
        return __classPrivateFieldGet(this, _boundSockets).filter(x => {
            if (x.port !== port)
                return false;
            if (x.socket.hasMembership(address)) {
                return true;
            }
            if (x.socket.hasFirewalledAddress(address)) {
                if (x.socket.hasOutgoingAddress(fromAddress)) {
                    return true;
                }
                else {
                    return false;
                }
            }
        }).map(x => (x.socket));
    }
}
_boundSockets = new WeakMap();
const dgramSocketManager = new MockDgramSocketManager();
class MockDgramSocket {
    constructor(args) {
        _id.set(this, util_1.randomAlphaString(10));
        _type.set(this, void 0);
        _reuseAddr.set(this, void 0);
        _hostName.set(this, void 0);
        _bound.set(this, false);
        _port.set(this, core_1.toPort(1));
        _onListeningCallbacks.set(this, []);
        _onMessageCallbacks.set(this, []);
        _memberships.set(this, new Set());
        _firewalledAddresses.set(this, new Set());
        _outgoingAddresses.set(this, new Set());
        __classPrivateFieldSet(this, _type, args.type);
        __classPrivateFieldSet(this, _reuseAddr, args.reuseAddr);
        __classPrivateFieldSet(this, _hostName, args.hostName);
    }
    bind(p) {
        dgramSocketManager.bind(__classPrivateFieldGet(this, _id), core_1.toPort(p), this);
        __classPrivateFieldSet(this, _bound, true);
        __classPrivateFieldSet(this, _port, core_1.toPort(p));
        __classPrivateFieldGet(this, _onListeningCallbacks).forEach(cb => { cb(); });
    }
    hasMembership(a) {
        return __classPrivateFieldGet(this, _memberships).has(a);
    }
    hasFirewalledAddress(a) {
        return __classPrivateFieldGet(this, _firewalledAddresses).has(a);
    }
    close() {
        dgramSocketManager.unbind(__classPrivateFieldGet(this, _id));
    }
    on(eventName, callback) {
        if (eventName === 'listening') {
            __classPrivateFieldGet(this, _onListeningCallbacks).push(callback);
            if (__classPrivateFieldGet(this, _bound)) {
                __classPrivateFieldGet(this, _onListeningCallbacks).forEach(cb => { cb(); });
            }
        }
        else if (eventName === 'message') {
            __classPrivateFieldGet(this, _onMessageCallbacks).push(callback);
        }
    }
    addMembership(address) {
        __classPrivateFieldGet(this, _memberships).add(address);
    }
    addFirewalledAddress(address) {
        __classPrivateFieldGet(this, _firewalledAddresses).add(address);
    }
    send(message, offset, length, port, address, callback) {
        __classPrivateFieldGet(this, _outgoingAddresses).add(address + ':' + port);
        const sockets = dgramSocketManager.socketsBelongingToAddress(address, core_1.toPort(port), __classPrivateFieldGet(this, _hostName) + ':' + __classPrivateFieldGet(this, _port));
        sockets.forEach(socket => {
            const rinfoAddress = address.startsWith('local-') ? 'local-' + __classPrivateFieldGet(this, _hostName).toString() : __classPrivateFieldGet(this, _hostName).toString();
            const rinfo = {
                address: rinfoAddress,
                family: 'IPv4',
                port: __classPrivateFieldGet(this, _port),
                size: 0
            };
            socket._handleMessage(message, rinfo);
        });
        callback && callback(null, message.length);
    }
    hasOutgoingAddress(x) {
        return __classPrivateFieldGet(this, _outgoingAddresses).has(x);
    }
    _handleMessage(message, rinfo) {
        __classPrivateFieldGet(this, _onMessageCallbacks).forEach(cb => {
            cb(message, rinfo);
        });
    }
}
_id = new WeakMap(), _type = new WeakMap(), _reuseAddr = new WeakMap(), _hostName = new WeakMap(), _bound = new WeakMap(), _port = new WeakMap(), _onListeningCallbacks = new WeakMap(), _onMessageCallbacks = new WeakMap(), _memberships = new WeakMap(), _firewalledAddresses = new WeakMap(), _outgoingAddresses = new WeakMap();
const mockDgramCreateSocket = (args) => {
    const s = new MockDgramSocket({ type: args.type, reuseAddr: args.reuseAddr, hostName: core_1.hostName(args.nodeId.toString()) });
    if (args.firewalled) {
        s.addFirewalledAddress(args.nodeId.toString());
    }
    else {
        s.addMembership(args.nodeId.toString());
    }
    s.addMembership('local-' + args.nodeId.toString());
    return s;
};
exports.default = mockDgramCreateSocket;
//# sourceMappingURL=mockDgramCreateSocket.js.map