"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _feedsConfigMemCache, _feedIdsByNameMemCache, _feedsConfigManager, _localFeedsDatabase;
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const crypto_util_1 = require("../../common/crypto_util");
const GarbageMap_1 = __importDefault(require("../../common/GarbageMap"));
const util_1 = require("../../common/util");
const core_1 = require("../../interfaces/core");
const LocalFeedsDatabase_1 = __importDefault(require("./LocalFeedsDatabase"));
const isFeedConfig = (x) => {
    return core_1._validateObject(x, {
        feedId: core_1.isFeedId,
        privateKey: core_1.isPrivateKeyHex
    });
};
class OldFeedsConfigManager {
    constructor(configDir, opts) {
        this.configDir = configDir;
        this.opts = opts;
        _feedsConfigMemCache.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(60 * 60 * 1000)));
        _feedIdsByNameMemCache.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(60 * 60 * 1000)));
    }
    async addFeed(feedId, privateKey) {
        const c = await this.getFeedConfig(feedId);
        if (c) {
            throw Error('Cannot add feed. Feed with this ID already exists');
        }
        const dirPath = await this._feedConfigDirectory(feedId, { create: true });
        const config = {
            feedId,
            privateKey
        };
        await writeJsonFile(`${dirPath}/config.json`, config, fs_1.default.constants.S_IRUSR | fs_1.default.constants.S_IWUSR);
        __classPrivateFieldGet(this, _feedsConfigMemCache).set(feedId, { feedId, privateKey });
    }
    async getFeedConfig(feedId) {
        const c = __classPrivateFieldGet(this, _feedsConfigMemCache).get(feedId);
        if ((c) && (this.opts.useMemoryCache)) {
            return c;
        }
        const dirPath = await this._feedConfigDirectory(feedId, { create: false });
        if (fs_1.default.existsSync(dirPath.toString())) {
            const configPath = `${dirPath}/config.json`;
            if (util_1.isReadableByOthers(configPath)) {
                fs_1.default.chmodSync(configPath, fs_1.default.constants.S_IRUSR | fs_1.default.constants.S_IWUSR);
                if (util_1.isReadableByOthers(configPath)) {
                    throw Error(`Feed config file is readable by others even after setting permissions: ${configPath}`);
                }
            }
            const config = await readJsonFile(configPath, {});
            if (isFeedConfig(config)) {
                __classPrivateFieldGet(this, _feedsConfigMemCache).set(feedId, config);
                return config;
            }
            else {
                throw Error(`Problem with feed config file: ${dirPath}/config.json`);
            }
        }
        return null;
    }
    async deleteFeed(feedId) {
        if (__classPrivateFieldGet(this, _feedsConfigMemCache).has(feedId)) {
            __classPrivateFieldGet(this, _feedsConfigMemCache).delete(feedId);
        }
        const dirPath = await this._feedConfigDirectory(feedId, { create: false });
        if (fs_1.default.existsSync(dirPath.toString())) {
            await fs_1.default.promises.rmdir(dirPath.toString(), { recursive: true });
        }
    }
    async setFeedIdForName(feedName, feedId) {
        const path = await this._feedIdByNamePath(feedName, { create: true });
        await writeJsonFile(path.toString(), { feedName, feedId });
        __classPrivateFieldGet(this, _feedIdsByNameMemCache).set(feedName, feedId);
    }
    async getFeedIdForName(feedName) {
        const f = __classPrivateFieldGet(this, _feedIdsByNameMemCache).get(feedName);
        if ((f) && (this.opts.useMemoryCache))
            return f;
        const path = await this._feedIdByNamePath(feedName, { create: false });
        if (!fs_1.default.existsSync(path.toString()))
            return null;
        const a = await readJsonFile(path.toString(), {});
        if (!core_1.isJSONObject(a))
            return null;
        const feedId = a.feedId;
        if (!feedId)
            return null;
        if (core_1.isFeedId(feedId)) {
            __classPrivateFieldGet(this, _feedIdsByNameMemCache).set(feedName, feedId);
            return feedId;
        }
        else
            return null;
    }
    async _feedConfigDirectory(feedId, opts) {
        const f = feedId;
        const ret = `${this.configDir}/feeds/${f[0]}${f[1]}/${f[2]}${f[3]}/${f[4]}${f[5]}/${f}`;
        if (opts.create) {
            if (!fs_1.default.existsSync(ret)) {
                await fs_1.default.promises.mkdir(ret, { recursive: true });
            }
        }
        return core_1.localFilePath(ret);
    }
    async _feedIdByNamePath(feedName, opts) {
        const f = feedName;
        const p = `${this.configDir}/feeds/byName`;
        if (opts.create) {
            if (!fs_1.default.existsSync(p)) {
                await fs_1.default.promises.mkdir(p, { recursive: true });
            }
        }
        return core_1.localFilePath(`${p}/${feedName}.json`);
    }
    _migration_getAllFeedIds() {
        const checkDirectory = (path) => {
            if (!fs_1.default.existsSync(path))
                return [];
            const ret = [];
            for (let fname of fs_1.default.readdirSync(path)) {
                const path2 = `${path}/${fname}`;
                if (fname === 'config.json') {
                    const config = JSON.parse(fs_1.default.readFileSync(path2, { encoding: 'utf-8' }));
                    ret.push(config.feedId);
                }
                if (fs_1.default.statSync(path2).isDirectory()) {
                    const ret2 = checkDirectory(path2);
                    for (let x of ret2)
                        ret.push(x);
                }
            }
            return ret;
        };
        return checkDirectory(`${this.configDir}/feeds`);
    }
    _migration_getAllFeedNames() {
        const p = `${this.configDir}/feeds/byName`;
        if (!fs_1.default.existsSync(p))
            return [];
        const ret = [];
        for (let fname of fs_1.default.readdirSync(p)) {
            if (fname.endsWith('.json')) {
                const path2 = `${p}/${fname}`;
                const a = JSON.parse(fs_1.default.readFileSync(path2, { encoding: 'utf-8' }));
                // const feedId = a.feedId
                const feedName = a.feedName;
                ret.push(feedName);
            }
        }
        return ret;
    }
    _migration_deleteFeedIdForName(feedName) {
        const p = `${this.configDir}/feeds/byName`;
        if (!fs_1.default.existsSync(p))
            throw Error(`Unable to delete feedId for name: byName directory does not exist: ${feedName} ${p}`);
        const ret = [];
        for (let fname of fs_1.default.readdirSync(p)) {
            if (fname.endsWith('.json')) {
                const path2 = `${p}/${fname}`;
                const a = JSON.parse(fs_1.default.readFileSync(path2, { encoding: 'utf-8' }));
                // const feedId = a.feedId
                if (a.feedName === feedName.toString()) {
                    fs_1.default.unlinkSync(path2);
                    return;
                }
            }
        }
        throw Error(`Unable to delete feedId for name: ${feedName}`);
    }
}
_feedsConfigMemCache = new WeakMap(), _feedIdsByNameMemCache = new WeakMap();
class FeedsConfigManager {
    constructor(mutableManager, configDir) {
        this.mutableManager = mutableManager;
        this.configDir = configDir;
        if (fs_1.default.existsSync(`${configDir}/feeds`)) {
            const M = new OldFeedsConfigManager(configDir, { useMemoryCache: false });
            const allFeedIds = M._migration_getAllFeedIds();
            for (let feedId of allFeedIds) {
                console.info(`Migrating feed to new system: ${feedId}`);
                M.getFeedConfig(feedId).then(feedConfig => {
                    if (feedConfig) {
                        this.addFeed(feedId, feedConfig.privateKey).then(() => {
                            M.deleteFeed(feedId).catch((err) => {
                                console.warn(`Problem deleting feed config during migration: ${feedId} (${err.message})`);
                            });
                        }).catch((err) => {
                            console.warn(`Problem adding feed config during migration: ${feedId} (${err.message})`);
                        });
                    }
                }).catch((err) => {
                    console.warn(`Problem getting feed config during migration: ${feedId} (${err.message})`);
                });
            }
            const allFeedNames = M._migration_getAllFeedNames();
            for (let feedName of allFeedNames) {
                console.info(`Migrating feed name to new system: ${feedName}`);
                M.getFeedIdForName(feedName).then(feedId => {
                    if (feedId) {
                        this.setFeedIdForName(feedName, feedId).then(() => {
                            try {
                                M._migration_deleteFeedIdForName(feedName);
                            }
                            catch (err) {
                                console.warn(`Problem deleting feed id for name during migration: ${feedName} ${feedId} (${err.message})`);
                            }
                        }).catch((err) => {
                            console.warn(`Problem setting feed for name during migration: ${feedName} ${feedId} (${err.message})`);
                        });
                    }
                }).catch((err) => {
                    console.warn(`Problem getting feed for name during migration: ${feedName} (${err.message})`);
                });
            }
        }
    }
    async addFeed(feedId, privateKey) {
        const c = await this.getFeedConfig(feedId);
        if (c) {
            throw Error('Cannot add feed. Feed with this ID already exists');
        }
        const config = {
            feedId, privateKey
        };
        await this.mutableManager.set({ type: 'feedConfig', feedId: feedId.toString() }, config);
    }
    async getFeedConfig(feedId) {
        const r = await this.mutableManager.get({ type: 'feedConfig', feedId: feedId.toString() });
        if (!r)
            return null;
        const config = r.value;
        if (!isFeedConfig(config)) {
            throw Error(`Not a valid feed config for feed: ${feedId}`);
        }
        return config;
    }
    async deleteFeed(feedId) {
        await this.mutableManager.delete({ type: 'feedConfig', feedId: feedId.toString() });
    }
    async setFeedIdForName(feedName, feedId) {
        await this.mutableManager.set({ type: 'feedIdForName', feedName: feedName.toString() }, feedId.toString());
    }
    async getFeedIdForName(feedName) {
        const x = await this.mutableManager.get({ type: 'feedIdForName', feedName: feedName.toString() });
        if (!x)
            return null;
        const feedId = x.value;
        if (!core_1.isFeedId(feedId)) {
            throw Error(`Not a valid feed ID for name: ${feedName} ${feedId}`);
        }
        return feedId;
    }
}
class LocalFeedManager {
    // #localFeedsDatabaseOld: LocalFeedsDatabaseOld
    constructor(storageDir, mutableManager, configDir) {
        _feedsConfigManager.set(this, void 0);
        _localFeedsDatabase.set(this, void 0);
        if (!fs_1.default.existsSync(storageDir.toString())) {
            throw Error(`Storage directory does not exist: ${storageDir}`);
        }
        __classPrivateFieldSet(this, _feedsConfigManager, new FeedsConfigManager(mutableManager, configDir));
        __classPrivateFieldSet(this, _localFeedsDatabase, new LocalFeedsDatabase_1.default(core_1.localFilePath(storageDir + '/feeds.db'))
        // this.#localFeedsDatabaseOld = new LocalFeedsDatabaseOld(storageDir)
        );
        // this.#localFeedsDatabaseOld = new LocalFeedsDatabaseOld(storageDir)
    }
    async createFeed(feedName) {
        // Create a new writeable feed on this node and return the ID of the new feed
        // Generate the crypto keypair. The publicKey determines the new feed ID
        const { publicKey, privateKey } = crypto_util_1.createKeyPair();
        const feedId = crypto_util_1.publicKeyHexToFeedId(crypto_util_1.publicKeyToHex(publicKey));
        await __classPrivateFieldGet(this, _feedsConfigManager).addFeed(feedId, crypto_util_1.privateKeyToHex(privateKey));
        if (feedName !== null) {
            await __classPrivateFieldGet(this, _feedsConfigManager).setFeedIdForName(feedName, feedId);
        }
        await __classPrivateFieldGet(this, _localFeedsDatabase).addFeed(feedId);
        // Return the feed ID
        return feedId;
    }
    async deleteFeed(feedId) {
        await __classPrivateFieldGet(this, _localFeedsDatabase).deleteFeed(feedId);
        await __classPrivateFieldGet(this, _feedsConfigManager).deleteFeed(feedId);
    }
    async getFeedId(feedName) {
        // assert(typeof(feedName) === 'string');
        // Look up the feed ID for a particular feed name by consulting the config file
        const feedId = await __classPrivateFieldGet(this, _feedsConfigManager).getFeedIdForName(feedName);
        if (feedId) {
            const existsLocally = await this.feedExistsLocally(feedId);
            if (!existsLocally)
                return null;
        }
        return feedId;
    }
    async hasWriteableFeed(feedId) {
        // Check whether this node has a writeable feed.
        // We do this by ensuring that we have the associated private key and that the feed directory exists
        const privateKey = await this.getPrivateKeyForFeed(feedId);
        if (!privateKey)
            return false;
        // Returns true if we have the writeable feed
        return await this.feedExistsLocally(feedId);
    }
    async getPrivateKeyForFeed(feedId) {
        // Consult the config to get the private key associated with a particular feed ID
        const c = await __classPrivateFieldGet(this, _feedsConfigManager).getFeedConfig(feedId);
        return c ? crypto_util_1.hexToPrivateKey(c.privateKey) : null;
    }
    async feedExistsLocally(feedId) {
        return await __classPrivateFieldGet(this, _localFeedsDatabase).hasFeed(feedId);
    }
    async getSignedSubfeedMessages(feedId, subfeedHash) {
        // Read the messages file
        return await __classPrivateFieldGet(this, _localFeedsDatabase).getSignedSubfeedMessages(feedId, subfeedHash);
    }
    async getSubfeedAccessRules(feedId, subfeedHash) {
        return await __classPrivateFieldGet(this, _localFeedsDatabase).getSubfeedAccessRules(feedId, subfeedHash);
    }
    async appendSignedMessagesToSubfeed(feedId, subfeedHash, messages) {
        await __classPrivateFieldGet(this, _localFeedsDatabase).appendSignedMessagesToSubfeed(feedId, subfeedHash, messages);
    }
    async setSubfeedAccessRules(feedId, subfeedHash, accessRules) {
        await __classPrivateFieldGet(this, _localFeedsDatabase).setSubfeedAccessRules(feedId, subfeedHash, accessRules);
    }
}
exports.default = LocalFeedManager;
_feedsConfigManager = new WeakMap(), _localFeedsDatabase = new WeakMap();
const readJsonFile = async (path, defaultVal) => {
    try {
        const txt = await fs_1.default.promises.readFile(path, 'utf-8');
        return JSON.parse(txt);
    }
    catch (err) {
        return defaultVal;
    }
};
const writeJsonFile = async (path, obj, mode) => {
    const txt = crypto_util_1.JSONStringifyDeterministic(obj, 4);
    await fs_1.default.promises.writeFile(path, txt, { mode });
};
//# sourceMappingURL=LocalFeedManager.js.map