"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.httpGetDownload = exports.httpPostJson = exports.HttpPostJsonError = exports._tests = void 0;
const axios_1 = __importDefault(require("axios"));
const DataStreamy_1 = __importDefault(require("../../common/DataStreamy"));
const core_1 = require("../../interfaces/core");
exports._tests = {};
class HttpPostJsonError extends Error {
    constructor(errorString) {
        super(errorString);
    }
}
exports.HttpPostJsonError = HttpPostJsonError;
const formUrl = (address, path) => {
    let url;
    if (address.url) {
        url = core_1.urlString(address.url.toString() + path);
    }
    else if ((address.hostName) && (address.port)) {
        url = core_1.urlString('http://' + address.hostName + ':' + address.port + path);
    }
    else {
        throw Error(`Unexpected address in formUrl: ${address}`);
    }
    return url;
};
const httpPostJson = async (address, path, data, opts) => {
    const url = formUrl(address, path);
    let res;
    try {
        res = await axios_1.default.post(url.toString(), data, { timeout: core_1.durationMsecToNumber(opts.timeoutMsec), responseType: 'json' });
    }
    catch (err) {
        throw new HttpPostJsonError(err.message);
    }
    return res.data;
};
exports.httpPostJson = httpPostJson;
const httpGetDownload = async (address, path, stats, opts) => {
    const url = formUrl(address, path);
    const res = await axios_1.default.get(url.toString(), { responseType: 'stream' });
    const stream = res.data;
    const socket = stream.socket;
    const req = stream.req;
    const size = res.headers['Content-Length'];
    const ret = new DataStreamy_1.default();
    let complete = false;
    ret.producer().start(size);
    ret.producer().onCancelled(() => {
        if (complete)
            return;
        // todo: is this the right way to close it?
        req.abort();
    });
    stream.on('data', (data) => {
        if (complete)
            return;
        stats.reportBytesReceived('http', opts.fromNodeId, core_1.byteCount(data.length));
        ret.producer().data(data);
    });
    stream.on('error', (err) => {
        if (complete)
            return;
        complete = true;
        ret.producer().error(err);
    });
    stream.on('end', () => {
        if (complete)
            return;
        complete = true;
        ret.producer().end();
    });
    socket.on('close', () => {
        if (complete)
            return;
        complete = true;
        ret.producer().error(Error('Socket closed.'));
    });
    return ret;
};
exports.httpGetDownload = httpGetDownload;
//# sourceMappingURL=httpRequests.js.map