"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const core_1 = require("../../interfaces/core");
const startHttpServer = async (app, listenPort) => {
    // convenient for starting either as http or https depending on the port
    let protocol;
    let server;
    if (process.env.SSL != null ? process.env.SSL : core_1.portToNumber(listenPort) % 1000 == 443) {
        // The port number ends with 443, so we are using https
        // app.USING_HTTPS = true;
        protocol = 'https';
        // Look for the credentials inside the encryption directory
        // You can generate these for free using the tools of letsencrypt.org
        const options = {
            key: fs_1.default.readFileSync(__dirname + '/encryption/privkey.pem'),
            cert: fs_1.default.readFileSync(__dirname + '/encryption/fullchain.pem'),
            ca: fs_1.default.readFileSync(__dirname + '/encryption/chain.pem')
        };
        // Create the https server
        server = https_1.default.createServer(options, app);
    }
    else {
        protocol = 'http';
        // Create the http server and start listening
        server = http_1.default.createServer(app);
    }
    server.listen(core_1.portToNumber(listenPort));
    await new Promise((resolve, reject) => {
        server.on('listening', () => {
            resolve();
        });
    });
    return server;
};
exports.default = startHttpServer;
//# sourceMappingURL=startHttpServer.js.map