"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createWebSocket = exports.startWebSocketServer = void 0;
const ws_1 = __importDefault(require("ws"));
const core_1 = require("../../interfaces/core");
const webSocketInterfaceFromWebSocket = (ws) => {
    const onMessage = (cb) => {
        ws.on('message', (buf) => {
            /* istanbul ignore next */
            if (!core_1.isBuffer(buf)) {
                console.warn('Incoming message is not a Buffer');
                ws.close();
                return;
            }
            cb(buf);
        });
    };
    const onError = (cb) => {
        ws.on('error', (err) => {
            cb(err);
        });
    };
    const onClose = (cb) => {
        ws.on('close', (code, reason) => {
            cb(code, reason);
        });
    };
    const onOpen = (cb) => {
        ws.on('open', () => {
            cb();
        });
    };
    return {
        onOpen,
        onMessage,
        onError,
        onClose,
        close: () => { ws.close(); },
        send: (buf) => { ws.send(buf); }
    };
};
const webSocketServerInterfaceFromWebSocketServer = (S) => {
    const onConnection = (cb) => {
        S.on('connection', (ws) => {
            cb(webSocketInterfaceFromWebSocket(ws));
        });
    };
    return {
        onConnection,
        close: () => { S.close(); }
    };
};
const startWebSocketServer = async (port, nodeId) => {
    const S = new ws_1.default.Server({ port: port });
    const wsi = webSocketServerInterfaceFromWebSocketServer(S);
    return new Promise((resolve, reject) => {
        S.on('listening', () => {
            resolve(wsi);
        });
        S.on('error', err => {
            /* istanbul ignore next */
            reject(err);
        });
    });
};
exports.startWebSocketServer = startWebSocketServer;
const createWebSocket = (url, opts) => {
    const ws = new ws_1.default(url, { timeout: core_1.durationMsecToNumber(opts.timeoutMsec) });
    return webSocketInterfaceFromWebSocket(ws);
};
exports.createWebSocket = createWebSocket;
//# sourceMappingURL=webSocket.js.map