"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var _signedMessages, _appending;
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_util_1 = require("../common/crypto_util");
const core_1 = require("../interfaces/core");
class LocalSubfeedSignedMessagesManager {
    constructor(localFeedManager, feedId, subfeedHash, publicKey) {
        this.localFeedManager = localFeedManager;
        this.feedId = feedId;
        this.subfeedHash = subfeedHash;
        this.publicKey = publicKey;
        _signedMessages.set(this, null); // in-memory cache
        _appending.set(this, false);
        this.isInitialized = () => {
            return __classPrivateFieldGet(this, _signedMessages) !== null;
        };
    }
    async initializeFromLocal() {
        const messages = await this.localFeedManager.getSignedSubfeedMessages(this.feedId, this.subfeedHash);
        // Verify the integrity of the messages
        // The first message has a previousSignature of null
        let previousSignature = null;
        let previousMessageNumber = -1;
        for (let msg of messages) {
            if (!crypto_util_1.verifySignatureJson(msg.body, msg.signature, this.publicKey)) {
                /* istanbul ignore next */
                throw Error(`Unable to verify signature of message in feed: ${msg.signature}`);
            }
            if (previousSignature !== (msg.body.previousSignature || null)) {
                /* istanbul ignore next */
                throw Error(`Inconsistent previousSignature of message in feed when reading messages from file: ${previousSignature} ${msg.body.previousSignature}`);
            }
            if (previousMessageNumber + 1 !== msg.body.messageNumber) {
                /* istanbul ignore next */
                throw Error(`Incorrect message number for message in feed when reading messages from file: ${previousMessageNumber + 1} ${msg.body.messageNumber}`);
            }
            previousSignature = msg.signature;
            previousMessageNumber = msg.body.messageNumber;
        }
        // store in memory
        __classPrivateFieldSet(this, _signedMessages, messages);
    }
    initializeEmptyMessageList() {
        __classPrivateFieldSet(this, _signedMessages, []);
    }
    getNumMessages() {
        if (__classPrivateFieldGet(this, _signedMessages) === null) {
            /* istanbul ignore next */
            throw Error('#signedMessages is null. Perhaps getNumMessages was called before subfeed was initialized.');
        }
        return core_1.messageCount(__classPrivateFieldGet(this, _signedMessages).length);
    }
    getSignedMessage(i) {
        if (__classPrivateFieldGet(this, _signedMessages) === null) {
            /* istanbul ignore next */
            throw Error('#signedMessages is null. Perhaps getSignedMessage was called before subfeed was initialized.');
        }
        return __classPrivateFieldGet(this, _signedMessages)[i];
    }
    async appendSignedMessages(signedMessagesToAppend) {
        if (__classPrivateFieldGet(this, _signedMessages) === null) {
            /* istanbul ignore next */
            throw Error('#signedMessages is null. Perhaps appendSignedMessages was called before subfeed was initialized.');
        }
        const firstAppendMessageNumber = signedMessagesToAppend.length === 0 ? null : signedMessagesToAppend[0].body.messageNumber;
        const lastExistingMessageNumber = __classPrivateFieldGet(this, _signedMessages).length === 0 ? null : __classPrivateFieldGet(this, _signedMessages)[__classPrivateFieldGet(this, _signedMessages).length - 1].body.messageNumber;
        if (firstAppendMessageNumber !== null) {
            if (lastExistingMessageNumber === null) {
                if (firstAppendMessageNumber !== 0)
                    throw Error('Unexpected in appendSignedMessages: first message number to append does not equal zero');
            }
            else {
                if (firstAppendMessageNumber !== lastExistingMessageNumber + 1)
                    throw Error(`Unexpected in appendSignedMessages: unexpcted first message number for appending ${firstAppendMessageNumber} <> ${lastExistingMessageNumber + 1}`);
            }
        }
        if (__classPrivateFieldGet(this, _appending))
            throw Error('Cannot append messages while messages are being appended.');
        __classPrivateFieldSet(this, _appending, true
        // CHAIN:append_messages:step(6)
        );
        // CHAIN:append_messages:step(6)
        await this.localFeedManager.appendSignedMessagesToSubfeed(this.feedId, this.subfeedHash, signedMessagesToAppend);
        for (let sm of signedMessagesToAppend) {
            __classPrivateFieldGet(this, _signedMessages).push(sm);
        }
        __classPrivateFieldSet(this, _appending, false);
    }
}
_signedMessages = new WeakMap(), _appending = new WeakMap();
exports.default = LocalSubfeedSignedMessagesManager;
//# sourceMappingURL=LocalSubfeedSignedMessagesManager.js.map