"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _incomingSubscriptions, _subscriptionCodesBySubfeedCode, _lastRenewTimestamp;
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../common/constants");
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
class NewIncomingSubfeedSubscriptionManager {
    constructor(node) {
        this.node = node;
        _incomingSubscriptions.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(300 * 60 * 1000)));
        _subscriptionCodesBySubfeedCode.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(300 * 60 * 1000)));
    }
    createOrRenewIncomingSubscription(remoteNodeId, feedId, subfeedHash) {
        const subscriptionCode = makeSubscriptionCode(remoteNodeId, feedId, subfeedHash);
        const subfeedCode = makeSubfeedCode(feedId, subfeedHash);
        __classPrivateFieldGet(this, _subscriptionCodesBySubfeedCode).set(subfeedCode, { ...__classPrivateFieldGet(this, _subscriptionCodesBySubfeedCode).get(subfeedCode) || {}, [subscriptionCode]: true });
        let S = __classPrivateFieldGet(this, _incomingSubscriptions).get(subfeedCode);
        if (!S) {
            // CHAIN:get_remote_messages:step(12)
            S = new IncomingSubfeedSubscription(this.node, remoteNodeId, feedId, subfeedHash);
            __classPrivateFieldGet(this, _incomingSubscriptions).set(subscriptionCode, S);
        }
        S.renew();
        setTimeout(() => {
            this._checkRemove(remoteNodeId, feedId, subfeedHash);
        }, core_1.durationMsecToNumber(core_1.scaledDurationMsec(60000)));
    }
    reportMessagesAdded(feedId, subfeedHash, numMessages) {
        const subfeedCode = makeSubfeedCode(feedId, subfeedHash);
        // CHAIN:append_messages:step(11)
        const x = __classPrivateFieldGet(this, _subscriptionCodesBySubfeedCode).get(subfeedCode) || {};
        for (let subscriptionCode in x) {
            const s = __classPrivateFieldGet(this, _incomingSubscriptions).get(subscriptionCode);
            if (s) {
                // CHAIN:get_remote_messages:step(13)
                this._reportNewSubfeedMessages(s.remoteNodeId, s.feedId, s.subfeedHash, numMessages).then(() => { }).catch((err) => {
                    console.warn(`Problem reporting new subfeed messages to remote node: ${err.message}`);
                });
            }
        }
    }
    async _reportNewSubfeedMessages(toNodeId, feedId, subfeedHash, numMessages) {
        // CHAIN:get_remote_messages:step(14)
        const requestData = {
            requestType: 'reportNewSubfeedMessages',
            feedId,
            subfeedHash,
            numMessages
        };
        const responseData = await this.node.remoteNodeManager().sendRequestToNode(toNodeId, requestData, { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, method: 'default' });
        if (!NodeToNodeRequest_1.isReportNewSubfeedMessagesResponseData(responseData)) {
            throw Error('Unexpected response to reportNewSubfeedMessages request');
        }
        if (!responseData.success) {
            throw Error(`Error in reportNewSubfeedMessages: ${responseData.errorMessage}`);
        }
    }
    _checkRemove(remoteNodeId, feedId, subfeedHash) {
        const subfeedCode = makeSubscriptionCode(remoteNodeId, feedId, subfeedHash);
        const S = __classPrivateFieldGet(this, _incomingSubscriptions).get(subfeedCode);
        if (!S)
            return;
        const elapsedMsec = S.elapsedMsecSinceLastRenew();
        if (elapsedMsec > core_1.durationMsecToNumber(S.durationMsec())) {
            __classPrivateFieldGet(this, _incomingSubscriptions).delete(subfeedCode);
        }
    }
}
_incomingSubscriptions = new WeakMap(), _subscriptionCodesBySubfeedCode = new WeakMap();
const makeSubscriptionCode = (remoteNodeId, feedId, subfeedHash) => {
    return remoteNodeId + ':' + feedId.toString() + ':' + subfeedHash.toString();
};
const makeSubfeedCode = (feedId, subfeedHash) => {
    return feedId.toString() + ':' + subfeedHash.toString();
};
class IncomingSubfeedSubscription {
    constructor(node, remoteNodeId, feedId, subfeedHash) {
        this.node = node;
        this.remoteNodeId = remoteNodeId;
        this.feedId = feedId;
        this.subfeedHash = subfeedHash;
        _lastRenewTimestamp.set(this, core_1.zeroTimestamp());
    }
    renew() {
        __classPrivateFieldSet(this, _lastRenewTimestamp, core_1.nowTimestamp());
    }
    durationMsec() {
        return core_1.scaledDurationMsec(60 * 1000);
    }
    elapsedMsecSinceLastRenew() {
        return core_1.elapsedSince(__classPrivateFieldGet(this, _lastRenewTimestamp));
    }
}
_lastRenewTimestamp = new WeakMap();
exports.default = NewIncomingSubfeedSubscriptionManager;
//# sourceMappingURL=NewIncomingSubfeedSubscriptionManager.js.map