"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _outgoingSubscriptions, _lastRenewTimestamp, _lastRenewDurationMsec, _initialMessageSent;
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../common/constants");
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
class NewOutgoingSubfeedSubscriptionManager {
    constructor(node) {
        this.node = node;
        _outgoingSubscriptions.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(300 * 60 * 1000)));
    }
    async createOrRenewOutgoingSubscription(remoteNodeId, feedId, subfeedHash) {
        const subfeedCode = makeSubscriptionCode(remoteNodeId, feedId, subfeedHash);
        let S = __classPrivateFieldGet(this, _outgoingSubscriptions).get(subfeedCode);
        if (!S) {
            S = new OutgoingSubfeedSubscription(this.node, remoteNodeId, feedId, subfeedHash);
            __classPrivateFieldGet(this, _outgoingSubscriptions).set(subfeedCode, S);
        }
        // CHAIN:get_remote_messages:step(6)
        await S.renew();
        setTimeout(() => {
            this._checkRemove(remoteNodeId, feedId, subfeedHash);
        }, core_1.durationMsecToNumber(S.durationMsec()) + core_1.durationMsecToNumber(core_1.scaledDurationMsec(5000)));
    }
    _checkRemove(remoteNodeId, feedId, subfeedHash) {
        const subfeedCode = makeSubscriptionCode(remoteNodeId, feedId, subfeedHash);
        const S = __classPrivateFieldGet(this, _outgoingSubscriptions).get(subfeedCode);
        if (!S)
            return;
        const elapsedMsec = S.elapsedMsecSinceLastRenew();
        if (elapsedMsec > core_1.durationMsecToNumber(S.durationMsec())) {
            __classPrivateFieldGet(this, _outgoingSubscriptions).delete(subfeedCode);
        }
    }
}
_outgoingSubscriptions = new WeakMap();
const makeSubscriptionCode = (remoteNodeId, feedId, subfeedHash) => {
    return remoteNodeId + ':' + feedId.toString() + ':' + subfeedHash.toString();
};
class OutgoingSubfeedSubscription {
    constructor(node, remoteNodeId, feedId, subfeedHash) {
        this.node = node;
        this.remoteNodeId = remoteNodeId;
        this.feedId = feedId;
        this.subfeedHash = subfeedHash;
        _lastRenewTimestamp.set(this, core_1.zeroTimestamp());
        _lastRenewDurationMsec.set(this, core_1.scaledDurationMsec(1000 * 60));
        _initialMessageSent.set(this, false);
    }
    async renew() {
        if (core_1.elapsedSince(__classPrivateFieldGet(this, _lastRenewTimestamp)) < core_1.durationMsecToNumber(this.durationMsec()) / 2) {
            return;
        }
        __classPrivateFieldSet(this, _lastRenewTimestamp, core_1.nowTimestamp()
        // CHAIN:get_remote_messages:step(7)
        );
        // CHAIN:get_remote_messages:step(7)
        const requestData = {
            requestType: 'subscribeToSubfeed',
            feedId: this.feedId,
            subfeedHash: this.subfeedHash
        };
        const responseData = await this.node.remoteNodeManager().sendRequestToNode(this.remoteNodeId, requestData, {
            timeoutMsec: constants_1.TIMEOUTS.defaultRequest,
            method: 'default'
        });
        if (!NodeToNodeRequest_1.isSubscribeToSubfeedResponseData(responseData))
            throw Error('Unexpected response to subscribeToSubfeed');
        if (!responseData.success)
            throw Error(`Error in response to subscribeToSubfeed: ${responseData.errorMessage}`);
        const numRemoteMessages = responseData.numMessages;
        if (numRemoteMessages !== null) {
            this.node.feedManager().reportNumRemoteMessages(this.remoteNodeId, this.feedId, this.subfeedHash, numRemoteMessages);
        }
    }
    elapsedMsecSinceLastRenew() {
        return core_1.elapsedSince(__classPrivateFieldGet(this, _lastRenewTimestamp));
    }
    durationMsec() {
        return __classPrivateFieldGet(this, _lastRenewDurationMsec);
    }
}
_lastRenewTimestamp = new WeakMap(), _lastRenewDurationMsec = new WeakMap(), _initialMessageSent = new WeakMap();
exports.default = NewOutgoingSubfeedSubscriptionManager;
//# sourceMappingURL=NewOutgoingSubfeedSubscriptionManager.js.map