"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _remoteNodeId, _numRemoteMessages, _activeDownload;
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../common/constants");
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
class RemoteSubfeedMessageDownloader {
    constructor(node, subfeed) {
        this.node = node;
        this.subfeed = subfeed;
        _remoteNodeId.set(this, null);
        _numRemoteMessages.set(this, null);
        _activeDownload.set(this, false);
    }
    reportNumRemoteMessages(remoteNodeId, numRemoteMessages) {
        __classPrivateFieldSet(this, _remoteNodeId, remoteNodeId);
        __classPrivateFieldSet(this, _numRemoteMessages, numRemoteMessages);
        if (this.subfeed.getNumLocalMessages() < __classPrivateFieldGet(this, _numRemoteMessages)) {
            if (!__classPrivateFieldGet(this, _activeDownload))
                this._startDownload();
        }
    }
    async _startDownload() {
        if (__classPrivateFieldGet(this, _activeDownload))
            return;
        __classPrivateFieldSet(this, _activeDownload, true);
        while (true) {
            const numLocalMessages = this.subfeed.getNumLocalMessages();
            const numRemoteMessages = __classPrivateFieldGet(this, _numRemoteMessages);
            if (numRemoteMessages === null)
                break;
            const remoteNodeId = __classPrivateFieldGet(this, _remoteNodeId);
            if (remoteNodeId === null)
                break;
            const numMessagesToDownload = core_1.messageCountToNumber(numRemoteMessages) - core_1.messageCountToNumber(numLocalMessages);
            if (numMessagesToDownload <= 0)
                break;
            let signedMessages;
            try {
                signedMessages = await this._doDownloadRemoteMessages(remoteNodeId, core_1.subfeedPosition(core_1.messageCountToNumber(numLocalMessages)), core_1.messageCount(numMessagesToDownload));
            }
            catch (err) {
                console.warn('Error downloading remote subfeed messages', err);
                break;
            }
            if (signedMessages.length !== numMessagesToDownload) {
                console.warn('Unexpected problem downloading remote subfeed messages. Got unexpected number of messages.');
                break;
            }
            // maybe somehow the number of local messages has changed (shouldn't happen though)
            const signedMessagesToAppend = signedMessages.slice(core_1.messageCountToNumber(this.subfeed.getNumLocalMessages()) - core_1.messageCountToNumber(numLocalMessages));
            await this.subfeed.appendSignedMessages(signedMessagesToAppend);
        }
        __classPrivateFieldSet(this, _activeDownload, false);
    }
    async _doDownloadRemoteMessages(remoteNodeId, position, numMessages) {
        const n = this.node.remoteNodeManager().getRemoteNode(remoteNodeId);
        /* istanbul ignore next */
        if (!n) {
            throw Error('Unexpected. Remote node not found.');
        }
        const requestData = {
            requestType: 'downloadSubfeedMessages',
            feedId: this.subfeed.getFeedId(),
            subfeedHash: this.subfeed.getSubfeedHash(),
            position,
            numMessages
        };
        const responseData = await n.sendRequest(requestData, { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, method: 'default' });
        if (!NodeToNodeRequest_1.isDownloadSubfeedMessagesResponseData(responseData)) {
            throw Error('Unexpected response data for downloadSubfeedMessages');
        }
        if (!responseData.success) {
            throw Error(`Unable to download remote messages: ${responseData.errorMessage}`);
        }
        if (!responseData.streamId) {
            throw Error('Unexpected: no stream ID');
        }
        const o = await n.downloadFileData(responseData.streamId, { method: 'default' });
        return new Promise((resolve, reject) => {
            let completed = false;
            const _data = [];
            o.dataStream.onError(err => {
                if (completed)
                    return;
                completed = true;
                reject(err);
            });
            const _err = (err) => {
                if (completed)
                    return;
                completed = true;
                reject();
            };
            o.dataStream.onFinished(() => {
                if (completed)
                    return;
                let json;
                try {
                    const data = Buffer.concat(_data);
                    json = data.toString('utf-8');
                }
                catch (err) {
                    _err(new Error('Problem assembling downloaded signed messages'));
                    return;
                }
                let signedMessages;
                try {
                    signedMessages = JSON.parse(json);
                }
                catch (err) {
                    _err(new Error('Problem parsing downloaded signed messages'));
                    return;
                }
                if (!core_1.isArrayOf(core_1.isSignedSubfeedMessage)(signedMessages)) {
                    _err(new Error('Invalid array of downloaded signed messages'));
                    return;
                }
                resolve(signedMessages);
            });
            o.dataStream.onData((buf) => {
                if (completed)
                    return;
                _data.push(buf);
            });
        });
    }
}
_remoteNodeId = new WeakMap(), _numRemoteMessages = new WeakMap(), _activeDownload = new WeakMap();
exports.default = RemoteSubfeedMessageDownloader;
//# sourceMappingURL=RemoteSubfeedMessageDownloader.js.map