"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _publicKey, _privateKey, _localSubfeedSignedMessagesManager, _accessRules, _isWriteable, _initialized, _initializing, _onInitializedCallbacks, _onInitializeErrorCallbacks, _newMessageListeners, _onMessagesAddedCallbacks, _mutex, _remoteSubfeedMessageDownloader;
Object.defineProperty(exports, "__esModule", { value: true });
const async_mutex_1 = require("async-mutex");
const console_1 = require("console");
const process_1 = require("process");
const crypto_util_1 = require("../common/crypto_util");
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const LocalSubfeedSignedMessagesManager_1 = __importDefault(require("./LocalSubfeedSignedMessagesManager"));
const RemoteSubfeedMessageDownloader_1 = __importDefault(require("./RemoteSubfeedMessageDownloader"));
// import NewOutgoingSubfeedSubscriptionManager from './NewOutgoingSubfeedSubscriptionManager';
class Subfeed {
    constructor(node, feedId, subfeedHash, localFeedManager, remoteFeedManager) {
        this.node = node;
        this.feedId = feedId;
        this.subfeedHash = subfeedHash;
        this.localFeedManager = localFeedManager;
        this.remoteFeedManager = remoteFeedManager;
        // Represents a subfeed, which may or may not be writeable on this node
        _publicKey.set(this, void 0); // The public key of the feed (which is determined by the feed ID)
        _privateKey.set(this, void 0); // The private key (or null if this is not writeable on the local node) -- set below
        _localSubfeedSignedMessagesManager.set(this, void 0); // The signed messages loaded from the messages file (in-memory cache)
        _accessRules.set(this, null); // Access rules for this subfeed -- like which nodes on the p2p network have permission to submit messages
        _isWriteable.set(this, null
        // #outgoingSubfeedSubscriptionManager: NewOutgoingSubfeedSubscriptionManager
        );
        // #outgoingSubfeedSubscriptionManager: NewOutgoingSubfeedSubscriptionManager
        _initialized.set(this, false);
        _initializing.set(this, false);
        _onInitializedCallbacks.set(this, []);
        _onInitializeErrorCallbacks.set(this, []);
        _newMessageListeners.set(this, new Map());
        _onMessagesAddedCallbacks.set(this, []);
        _mutex.set(this, new async_mutex_1.Mutex());
        _remoteSubfeedMessageDownloader.set(this, void 0);
        __classPrivateFieldSet(this, _publicKey, crypto_util_1.hexToPublicKey(core_1.feedIdToPublicKeyHex(feedId))); // The public key of the feed (which is determined by the feed ID)
        __classPrivateFieldSet(// The public key of the feed (which is determined by the feed ID)
        this, _localSubfeedSignedMessagesManager, new LocalSubfeedSignedMessagesManager_1.default(localFeedManager, feedId, subfeedHash, __classPrivateFieldGet(this, _publicKey)));
        __classPrivateFieldSet(this, _remoteSubfeedMessageDownloader, new RemoteSubfeedMessageDownloader_1.default(node, this));
    }
    async acquireLock() {
        return await __classPrivateFieldGet(this, _mutex).acquire();
    }
    async initialize(privateKey) {
        __classPrivateFieldSet(this, _privateKey, privateKey);
        if (__classPrivateFieldGet(this, _initialized))
            return;
        if (__classPrivateFieldGet(this, _initializing)) {
            await this.waitUntilInitialized();
            return;
        }
        try {
            __classPrivateFieldSet(this, _initializing, true
            // Check whether we have the feed locally (may or may not be locally writeable)
            );
            // Check whether we have the feed locally (may or may not be locally writeable)
            const existsLocally = await this.localFeedManager.feedExistsLocally(this.feedId);
            if (existsLocally) {
                await __classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).initializeFromLocal();
                // If this is a writeable feed, we also load the access rules into memory
                __classPrivateFieldSet(this, _isWriteable, await this.localFeedManager.hasWriteableFeed(this.feedId));
                if (__classPrivateFieldGet(this, _isWriteable)) {
                    const accessRules = await this.localFeedManager.getSubfeedAccessRules(this.feedId, this.subfeedHash);
                    if (accessRules) {
                        __classPrivateFieldSet(this, _accessRules, accessRules);
                    }
                    else {
                        __classPrivateFieldSet(this, _accessRules, null);
                    }
                }
            }
            else {
                __classPrivateFieldSet(this, _isWriteable, false
                // Otherwise, we don't have it locally -- so let's just initialize things
                );
                // Otherwise, we don't have it locally -- so let's just initialize things
                __classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).initializeEmptyMessageList();
                const messages = await this.localFeedManager.getSignedSubfeedMessages(this.feedId, this.subfeedHash);
                console_1.assert(messages.length === 0);
                __classPrivateFieldSet(this, _accessRules, null
                // don't do this
                // // Let's try to load messages from remote nodes on the p2p network
                // if (!opts.localOnly) {
                //     await this.getSignedMessages({position: subfeedPosition(0), maxNumMessages: messageCount(10), waitMsec: scaledDurationMsec(1)})
                // }
                );
                // don't do this
                // // Let's try to load messages from remote nodes on the p2p network
                // if (!opts.localOnly) {
                //     await this.getSignedMessages({position: subfeedPosition(0), maxNumMessages: messageCount(10), waitMsec: scaledDurationMsec(1)})
                // }
            }
        }
        catch (err) {
            __classPrivateFieldGet(this, _onInitializeErrorCallbacks).forEach(cb => { cb(err); });
            throw err;
        }
        __classPrivateFieldSet(this, _initializing, false);
        __classPrivateFieldSet(this, _initialized, true);
        __classPrivateFieldGet(this, _onInitializedCallbacks).forEach(cb => { cb(); });
    }
    async waitUntilInitialized() {
        if (__classPrivateFieldGet(this, _initialized))
            return;
        return new Promise((resolve, reject) => {
            __classPrivateFieldGet(this, _onInitializeErrorCallbacks).push((err) => {
                reject(err);
            });
            __classPrivateFieldGet(this, _onInitializedCallbacks).push(() => {
                resolve();
            });
        });
    }
    getFeedId() {
        return this.feedId;
    }
    getSubfeedHash() {
        return this.subfeedHash;
    }
    getNumLocalMessages() {
        // Return the number of messages that are currently loaded into memory
        return __classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages();
    }
    isWriteable() {
        // Whether this subfeed is writeable. That depends on whether we have a private key
        if (__classPrivateFieldGet(this, _isWriteable) === null) {
            /* istanbul ignore next */
            throw Error('#isWriteable is null. Perhaps isWriteable was called before subfeed was initialized.');
        }
        return __classPrivateFieldGet(this, _isWriteable);
    }
    async waitForSignedMessages({ position, maxNumMessages, waitMsec }) {
        const check = () => {
            if (core_1.subfeedPositionToNumber(position) < core_1.messageCountToNumber(this.getNumLocalMessages())) {
                let numMessages = core_1.messageCount(core_1.messageCountToNumber(this.getNumLocalMessages()) - core_1.subfeedPositionToNumber(position));
                if (core_1.messageCountToNumber(maxNumMessages) > 0) {
                    numMessages = core_1.messageCount(Math.min(core_1.messageCountToNumber(maxNumMessages), core_1.messageCountToNumber(numMessages)));
                }
                return this.getLocalSignedMessages({ position, numMessages });
            }
            else
                return [];
        };
        const messages = check();
        if (messages.length > 0)
            return messages;
        if (core_1.durationMsecToNumber(waitMsec) > 0) {
            const success = this.remoteFeedManager.subscribeToRemoteSubfeed(this.feedId, this.subfeedHash);
            if (!success) {
                return [];
            }
            return new Promise((resolve, reject) => {
                const listenerId = createListenerId();
                let completed = false;
                __classPrivateFieldGet(this, _newMessageListeners).set(listenerId, () => {
                    if (completed)
                        return;
                    const msgs = check();
                    if (msgs.length > 0) {
                        completed = true;
                        __classPrivateFieldGet(this, _newMessageListeners).delete(listenerId);
                        resolve(msgs);
                    }
                });
                setTimeout(() => {
                    if (completed)
                        return;
                    completed = true;
                    __classPrivateFieldGet(this, _newMessageListeners).delete(listenerId);
                    resolve([]);
                }, core_1.durationMsecToNumber(waitMsec));
            });
        }
        else {
            return [];
        }
    }
    getLocalSignedMessages({ position, numMessages }) {
        // Get some signed messages starting at position
        if (!__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).isInitialized()) {
            /* istanbul ignore next */
            throw Error('signed messages not initialized. Perhaps getLocalSignedMessages was called before subfeed was initialized.');
        }
        if (core_1.subfeedPositionToNumber(position) + core_1.messageCountToNumber(numMessages) <= Number(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages())) {
            // If we have some messages loaded into memory, let's return those!
            let signedMessages = [];
            for (let i = core_1.subfeedPositionToNumber(position); i < core_1.subfeedPositionToNumber(position) + core_1.messageCountToNumber(numMessages); i++) {
                signedMessages.push(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getSignedMessage(i));
            }
            return signedMessages;
        }
        else {
            throw Error(`Cannot get local signed messages (position=${position}, numMessages=${numMessages}, getNumMessages=${__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages()})`);
        }
    }
    async appendMessages(messages, { metaData }) {
        if (!__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).isInitialized()) {
            /* istanbul ignore next */
            throw Error('signed messages not initialized. Perhaps appendMessages was called before subfeed was initialized.');
        }
        if (messages.length === 0)
            return;
        if (!__classPrivateFieldGet(this, _privateKey)) {
            /* istanbul ignore next */
            throw Error(`Cannot write to feed without private key: ${__classPrivateFieldGet(this, _privateKey)}`);
        }
        const signedMessagesToAppend = [];
        let previousSignature;
        if (Number(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages()) > 0) {
            previousSignature = __classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getSignedMessage(Number(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages()) - 1).signature;
        }
        let messageNumber = Number(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages());
        for (let msg of messages) {
            let body = {
                message: msg,
                previousSignature,
                messageNumber,
                timestamp: core_1.nowTimestamp(),
                metaData: metaData ? metaData : undefined
            };
            const signedMessage = {
                body,
                signature: crypto_util_1.getSignatureJson(body, { publicKey: __classPrivateFieldGet(this, _publicKey), privateKey: __classPrivateFieldGet(this, _privateKey) })
            };
            if (!crypto_util_1.verifySignatureJson(body, crypto_util_1.getSignatureJson(body, { publicKey: __classPrivateFieldGet(this, _publicKey), privateKey: __classPrivateFieldGet(this, _privateKey) }), __classPrivateFieldGet(this, _publicKey))) {
                throw Error('Error verifying signature');
            }
            signedMessagesToAppend.push(signedMessage);
            previousSignature = signedMessage.signature;
            messageNumber++;
        }
        // CHAIN:append_messages:step(4)
        await this.appendSignedMessages(signedMessagesToAppend);
    }
    async appendSignedMessages(signedMessages) {
        if (!__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).isInitialized()) {
            /* istanbul ignore next */
            throw Error('signed messages not initialized. Perhaps appendSignedMessages was called before subfeed was initialized.');
        }
        if (signedMessages.length === 0)
            return;
        // it's possible that we have already appended some of these messages. Let's check
        if (signedMessages[0].body.messageNumber < core_1.messageCountToNumber(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages())) {
            signedMessages = signedMessages.slice(core_1.messageCountToNumber(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages()) - signedMessages[0].body.messageNumber);
        }
        if (signedMessages.length === 0)
            return;
        const signedMessagesToAppend = [];
        let previousSignature;
        if (Number(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages()) > 0) {
            previousSignature = __classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getSignedMessage(Number(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages()) - 1).signature;
        }
        let messageNumber = Number(__classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).getNumMessages());
        for (let signedMessage of signedMessages) {
            const body = signedMessage.body;
            const signature = signedMessage.signature;
            if (!crypto_util_1.verifySignatureJson(body, signature, __classPrivateFieldGet(this, _publicKey))) {
                throw Error(`Error verifying signature when appending signed message for: ${this.feedId} ${this.subfeedHash} ${signature}`);
            }
            if ((body.previousSignature || null) !== (previousSignature || null)) {
                throw Error(`Error in previousSignature when appending signed message for: ${this.feedId} ${this.subfeedHash} ${body.previousSignature} <> ${previousSignature}`);
            }
            if (body.messageNumber !== messageNumber) {
                // problem here
                throw Error(`Error in messageNumber when appending signed message for: ${this.feedId} ${this.subfeedHash} ${body.messageNumber} <> ${messageNumber}`);
            }
            previousSignature = signedMessage.signature;
            messageNumber++;
            signedMessagesToAppend.push(signedMessage);
        }
        // CHAIN:append_messages:step(5)
        await __classPrivateFieldGet(this, _localSubfeedSignedMessagesManager).appendSignedMessages(signedMessagesToAppend);
        process_1.nextTick(() => {
            // CHAIN:get_remote_messages:step(17)
            __classPrivateFieldGet(this, _newMessageListeners).forEach((listener) => {
                listener();
            });
            __classPrivateFieldGet(this, _onMessagesAddedCallbacks).forEach(cb => {
                // CHAIN:append_messages:step(9)
                cb();
            });
        });
    }
    async getAccessRules() {
        return __classPrivateFieldGet(this, _accessRules);
    }
    async setAccessRules(accessRules) {
        if (!this.isWriteable()) {
            /* istanbul ignore next */
            throw Error(`Cannot set access rules for not writeable subfeed.`);
        }
        await this.localFeedManager.setSubfeedAccessRules(this.feedId, this.subfeedHash, accessRules);
        __classPrivateFieldSet(this, _accessRules, accessRules);
    }
    onMessagesAdded(callback) {
        __classPrivateFieldGet(this, _onMessagesAddedCallbacks).push(callback);
    }
    reportNumRemoteMessages(remoteNodeId, numRemoteMessages) {
        __classPrivateFieldGet(this, _remoteSubfeedMessageDownloader).reportNumRemoteMessages(remoteNodeId, numRemoteMessages);
    }
}
_publicKey = new WeakMap(), _privateKey = new WeakMap(), _localSubfeedSignedMessagesManager = new WeakMap(), _accessRules = new WeakMap(), _isWriteable = new WeakMap(), _initialized = new WeakMap(), _initializing = new WeakMap(), _onInitializedCallbacks = new WeakMap(), _onInitializeErrorCallbacks = new WeakMap(), _newMessageListeners = new WeakMap(), _onMessagesAddedCallbacks = new WeakMap(), _mutex = new WeakMap(), _remoteSubfeedMessageDownloader = new WeakMap();
const createListenerId = () => {
    return util_1.randomAlphaString(10);
};
exports.default = Subfeed;
//# sourceMappingURL=Subfeed.js.map