"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getStats = exports.isGetStatsOpts = void 0;
const core_1 = require("./interfaces/core");
const isGetStatsOpts = (x) => {
    return core_1._validateObject(x, {
        format: core_1.optional(core_1.isOneOf([core_1.isEqualTo('json'), core_1.isEqualTo('html')]))
    });
};
exports.isGetStatsOpts = isGetStatsOpts;
const getStats = (node, o) => {
    const s = node.publicUdpSocketServer();
    const ret = {
        nodeId: node.nodeId(),
        joinedChannels: node.joinedChannels(),
        totalBytesSent: node.stats().totalBytesSent(),
        totalBytesReceived: node.stats().totalBytesReceived(),
        remoteNodes: []
    };
    node.remoteNodeManager().getAllRemoteNodes({ includeOffline: true }).forEach(rn => {
        ret.remoteNodes.push(rn.getStats());
    });
    const format = o.format || 'json';
    if (format === 'json') {
        return ret;
    }
    else if (format === 'html') {
        return {
            ...ret,
            html: createJsonViewHtml(ret)
        };
    }
    else {
        /* istanbul ignore next */
        throw Error('Unexpected in getStats');
    }
};
exports.getStats = getStats;
const createJsonViewHtml = (x) => {
    return `
<!DOCTYPE HTML>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>kachery-p2p node stats</title>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/jsonview@1.2.0/dist/jquery.jsonview.css" />
    <script type="text/javascript" src="http://code.jquery.com/jquery.min.js"></script>
    <script type="text/javascript" src="https://cdn.jsdelivr.net/npm/jsonview@1.2.0/dist/jquery.jsonview.js"></script>
    <script type="text/javascript">
    var x = \`${JSON.stringify(x)}\`
    $(function() {
        $("#json").JSONView(x, {collapsed: true, nl2br: false});
    });
    console.info(JSON.parse(x))
    </script>
</head>
<body>
    <h2>Kachery-p2p node stats</h2>
    <div id="json"></div>
</body>
</html>
`;
};
//# sourceMappingURL=getStats.js.map