"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFallbackUdpPacketResponseData = exports.isFallbackUdpPacketRequestData = exports.isStopStreamViaUdpResponseData = exports.isStopStreamViaUdpRequestData = exports.isStartStreamViaUdpResponseData = exports.isStartStreamViaUdpRequestData = exports.isDownloadSubfeedMessagesResponseData = exports.isDownloadSubfeedMessagesRequestData = exports.isDownloadFileDataResponseData = exports.isDownloadFileDataRequestData = exports.isSubmitMessageToLiveFeedResponseData = exports.isSubmitMessageToLiveFeedRequestData = exports.isReportNewSubfeedMessagesResponseData = exports.isReportNewSubfeedMessagesRequestData = exports.isSubscribeToSubfeedResponseData = exports.isSubscribeToSubfeedRequestData = exports.isCheckForLiveFeedResponseData = exports.isCheckForLiveFeedRequestData = exports.isCheckForFileResponseData = exports.isCheckForFileRequestData = exports.isAnnounceResponseData = exports.isAnnounceRequestData = exports.isGetChannelInfoResponseData = exports.isGetChannelInfoRequestData = exports.isCheckAliveResponseData = exports.isCheckAliveRequestData = exports.isNodeToNodeResponseData = exports.isNodeToNodeRequestData = exports.isNodeToNodeResponse = exports.isNodeToNodeResponseBody = exports.isNodeToNodeRequest = exports.isNodeToNodeRequestBody = exports.createStreamId = exports.isStreamId = exports._tests = void 0;
const assert_1 = __importDefault(require("assert"));
const util_1 = require("../common/util");
const protocolVersion_1 = require("../protocolVersion");
const UdpPacketSender_1 = require("../udp/UdpPacketSender");
const core_1 = require("./core");
exports._tests = {};
const isStreamId = (x) => {
    if (!core_1.isString(x))
        return false;
    return (/^[A-Za-z]{10}$/.test(x));
};
exports.isStreamId = isStreamId;
const createStreamId = () => {
    return util_1.randomAlphaString(10);
};
exports.createStreamId = createStreamId;
exports._tests.StreamId = () => { assert_1.default(exports.isStreamId(exports.createStreamId())); };
const isNodeToNodeRequestBody = (x) => {
    return core_1._validateObject(x, {
        protocolVersion: core_1.isEqualTo(protocolVersion_1.protocolVersion()),
        requestId: core_1.isRequestId,
        fromNodeId: core_1.isNodeId,
        toNodeId: core_1.isNodeId,
        timestamp: core_1.isTimestamp,
        requestData: exports.isNodeToNodeRequestData,
        timeoutMsec: core_1.isDurationMsec
    });
};
exports.isNodeToNodeRequestBody = isNodeToNodeRequestBody;
const isNodeToNodeRequest = (x) => {
    return core_1._validateObject(x, {
        body: exports.isNodeToNodeRequestBody,
        signature: core_1.isSignature
    });
};
exports.isNodeToNodeRequest = isNodeToNodeRequest;
const isNodeToNodeResponseBody = (x) => {
    return core_1._validateObject(x, {
        protocolVersion: core_1.isEqualTo(protocolVersion_1.protocolVersion()),
        requestId: core_1.isRequestId,
        fromNodeId: core_1.isNodeId,
        toNodeId: core_1.isNodeId,
        timestamp: core_1.isTimestamp,
        responseData: exports.isNodeToNodeResponseData
    });
};
exports.isNodeToNodeResponseBody = isNodeToNodeResponseBody;
const isNodeToNodeResponse = (x) => {
    return core_1._validateObject(x, {
        body: exports.isNodeToNodeResponseBody,
        signature: core_1.isSignature
    });
};
exports.isNodeToNodeResponse = isNodeToNodeResponse;
const isNodeToNodeRequestData = (x) => {
    return core_1.isOneOf([
        exports.isCheckAliveRequestData,
        exports.isGetChannelInfoRequestData,
        exports.isAnnounceRequestData,
        exports.isCheckForFileRequestData,
        exports.isCheckForLiveFeedRequestData,
        exports.isSubscribeToSubfeedResponseData,
        exports.isReportNewSubfeedMessagesResponseData,
        exports.isSubmitMessageToLiveFeedRequestData,
        exports.isDownloadFileDataRequestData,
        exports.isDownloadSubfeedMessagesRequestData,
        exports.isStartStreamViaUdpRequestData,
        exports.isStopStreamViaUdpRequestData,
        exports.isFallbackUdpPacketRequestData
    ]) ? true : false;
};
exports.isNodeToNodeRequestData = isNodeToNodeRequestData;
const isNodeToNodeResponseData = (x) => {
    return core_1.isOneOf([
        exports.isCheckAliveResponseData,
        exports.isGetChannelInfoResponseData,
        exports.isAnnounceResponseData,
        exports.isCheckForFileResponseData,
        exports.isCheckForLiveFeedResponseData,
        exports.isSubscribeToSubfeedResponseData,
        exports.isReportNewSubfeedMessagesResponseData,
        exports.isSubmitMessageToLiveFeedResponseData,
        exports.isDownloadFileDataResponseData,
        exports.isDownloadSubfeedMessagesResponseData,
        exports.isStartStreamViaUdpResponseData,
        exports.isStopStreamViaUdpResponseData,
        exports.isFallbackUdpPacketResponseData
    ]) ? true : false;
};
exports.isNodeToNodeResponseData = isNodeToNodeResponseData;
const isCheckAliveRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('checkAlive')
    });
};
exports.isCheckAliveRequestData = isCheckAliveRequestData;
const isCheckAliveResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('checkAlive'),
        alive: core_1.isBoolean
    });
};
exports.isCheckAliveResponseData = isCheckAliveResponseData;
const isGetChannelInfoRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('getChannelInfo'),
        channelConfigUrl: core_1.isChannelConfigUrl
    });
};
exports.isGetChannelInfoRequestData = isGetChannelInfoRequestData;
const isGetChannelInfoResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('getChannelInfo'),
        channelInfo: core_1.isChannelInfo
    });
};
exports.isGetChannelInfoResponseData = isGetChannelInfoResponseData;
const isAnnounceRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('announce'),
        channelNodeInfo: core_1.isChannelNodeInfo
    });
};
exports.isAnnounceRequestData = isAnnounceRequestData;
const isAnnounceResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('announce'),
        success: core_1.isBoolean,
        errorMessage: core_1.isOneOf([core_1.isNull, core_1.isErrorMessage])
    });
};
exports.isAnnounceResponseData = isAnnounceResponseData;
const isCheckForFileRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('checkForFile'),
        fileKey: core_1.isFileKey
    });
};
exports.isCheckForFileRequestData = isCheckForFileRequestData;
const isCheckForFileResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('checkForFile'),
        found: core_1.isBoolean,
        size: core_1.isOneOf([core_1.isNull, core_1.isByteCount])
    });
};
exports.isCheckForFileResponseData = isCheckForFileResponseData;
const isCheckForLiveFeedRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('checkForLiveFeed'),
        feedId: core_1.isFeedId
    });
};
exports.isCheckForLiveFeedRequestData = isCheckForLiveFeedRequestData;
const isCheckForLiveFeedResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('checkForLiveFeed'),
        found: core_1.isBoolean
    });
};
exports.isCheckForLiveFeedResponseData = isCheckForLiveFeedResponseData;
const isSubscribeToSubfeedRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('subscribeToSubfeed'),
        feedId: core_1.isFeedId,
        subfeedHash: core_1.isSubfeedHash
    });
};
exports.isSubscribeToSubfeedRequestData = isSubscribeToSubfeedRequestData;
const isSubscribeToSubfeedResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('subscribeToSubfeed'),
        success: core_1.isBoolean,
        numMessages: core_1.isOneOf([core_1.isMessageCount, core_1.isNull]),
        errorMessage: core_1.isOneOf([core_1.isErrorMessage, core_1.isNull])
    });
};
exports.isSubscribeToSubfeedResponseData = isSubscribeToSubfeedResponseData;
const isReportNewSubfeedMessagesRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('reportNewSubfeedMessages'),
        feedId: core_1.isFeedId,
        subfeedHash: core_1.isSubfeedHash,
        numMessages: core_1.isMessageCount
    });
};
exports.isReportNewSubfeedMessagesRequestData = isReportNewSubfeedMessagesRequestData;
const isReportNewSubfeedMessagesResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('reportNewSubfeedMessages'),
        success: core_1.isBoolean,
        errorMessage: core_1.isOneOf([core_1.isErrorMessage, core_1.isNull])
    });
};
exports.isReportNewSubfeedMessagesResponseData = isReportNewSubfeedMessagesResponseData;
const isSubmitMessageToLiveFeedRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('submitMessageToLiveFeed'),
        feedId: core_1.isFeedId,
        subfeedHash: core_1.isSubfeedHash,
        message: core_1.isSubmittedSubfeedMessage
    });
};
exports.isSubmitMessageToLiveFeedRequestData = isSubmitMessageToLiveFeedRequestData;
const isSubmitMessageToLiveFeedResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('submitMessageToLiveFeed'),
        success: core_1.isBoolean,
        errorMessage: core_1.isOneOf([core_1.isNull, core_1.isErrorMessage])
    });
};
exports.isSubmitMessageToLiveFeedResponseData = isSubmitMessageToLiveFeedResponseData;
const isDownloadFileDataRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('downloadFileData'),
        fileKey: core_1.isFileKey,
        startByte: core_1.isByteCount,
        endByte: core_1.isOneOf([core_1.isNull, core_1.isByteCount])
    });
};
exports.isDownloadFileDataRequestData = isDownloadFileDataRequestData;
const isDownloadFileDataResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('downloadFileData'),
        fileKey: core_1.isFileKey,
        startByte: core_1.isByteCount,
        endByte: core_1.isByteCount,
        success: core_1.isBoolean,
        streamId: core_1.isOneOf([core_1.isNull, exports.isStreamId]),
        errorMessage: core_1.isOneOf([core_1.isNull, core_1.isErrorMessage])
    });
};
exports.isDownloadFileDataResponseData = isDownloadFileDataResponseData;
const isDownloadSubfeedMessagesRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('downloadSubfeedMessages'),
        feedId: core_1.isFeedId,
        subfeedHash: core_1.isSubfeedHash,
        position: core_1.isSubfeedPosition,
        numMessages: core_1.isMessageCount
    });
};
exports.isDownloadSubfeedMessagesRequestData = isDownloadSubfeedMessagesRequestData;
const isDownloadSubfeedMessagesResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('downloadSubfeedMessages'),
        success: core_1.isBoolean,
        streamId: core_1.isOneOf([core_1.isNull, exports.isStreamId]),
        errorMessage: core_1.isOneOf([core_1.isNull, core_1.isErrorMessage])
    });
};
exports.isDownloadSubfeedMessagesResponseData = isDownloadSubfeedMessagesResponseData;
const isStartStreamViaUdpRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('startStreamViaUdp'),
        streamId: exports.isStreamId
    });
};
exports.isStartStreamViaUdpRequestData = isStartStreamViaUdpRequestData;
const isStartStreamViaUdpResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('startStreamViaUdp'),
        success: core_1.isBoolean,
        errorMessage: core_1.isOneOf([core_1.isNull, core_1.isErrorMessage])
    });
};
exports.isStartStreamViaUdpResponseData = isStartStreamViaUdpResponseData;
const isStopStreamViaUdpRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('stopStreamViaUdp'),
        streamId: exports.isStreamId
    });
};
exports.isStopStreamViaUdpRequestData = isStopStreamViaUdpRequestData;
const isStopStreamViaUdpResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('stopStreamViaUdp'),
        success: core_1.isBoolean,
        errorMessage: core_1.isOneOf([core_1.isNull, core_1.isErrorMessage])
    });
};
exports.isStopStreamViaUdpResponseData = isStopStreamViaUdpResponseData;
const isFallbackUdpPacketRequestData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('fallbackUdpPacket'),
        dataBase64: core_1.isString,
        packetId: UdpPacketSender_1.isPacketId
    });
};
exports.isFallbackUdpPacketRequestData = isFallbackUdpPacketRequestData;
const isFallbackUdpPacketResponseData = (x) => {
    return core_1._validateObject(x, {
        requestType: core_1.isEqualTo('fallbackUdpPacket'),
        success: core_1.isBoolean,
        errorMessage: core_1.isOneOf([core_1.isNull, core_1.isErrorMessage])
    });
};
exports.isFallbackUdpPacketResponseData = isFallbackUdpPacketResponseData;
//# sourceMappingURL=NodeToNodeRequest.js.map