"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isUdpHeader = exports.udpMessageMetaData = exports.isUdpMessageMetaData = exports.udpMessagePartId = exports.partIndex = exports.partIndexToNumber = exports.isPartIndex = exports.examplePartIndex = exports.numParts = exports.numPartsToNumber = exports.isNumParts = exports.createUdpMessageId = exports.isUdpMessageId = exports.isUdpMessageType = exports.UDP_PACKET_SIZE = exports.UDP_MESSAGE_HEADER_SIZE = void 0;
const crypto_util_1 = require("../common/crypto_util");
const util_1 = require("../common/util");
const protocolVersion_1 = require("../protocolVersion");
const core_1 = require("./core");
exports.UDP_MESSAGE_HEADER_SIZE = 1000;
exports.UDP_PACKET_SIZE = 20000;
const exampleUdpMessageType = "NodeToNodeRequest";
const isUdpMessageType = (x) => {
    if (!core_1.isString(x))
        return false;
    const possible = [
        "NodeToNodeRequest",
        "NodeToNodeResponse",
        "KeepAlive",
        "streamDataChunk",
        "streamDataError",
        "streamDataEnd"
    ];
    return possible.includes(x);
};
exports.isUdpMessageType = isUdpMessageType;
const exampleUdpMessageId = "messageIdA";
const isUdpMessageId = (x) => {
    if (!core_1.isString(x))
        return false;
    return (/^[A-Za-z]{10}$/.test(x));
};
exports.isUdpMessageId = isUdpMessageId;
const createUdpMessageId = () => {
    return util_1.randomAlphaString(10);
};
exports.createUdpMessageId = createUdpMessageId;
const exampleNumParts = 1;
const isNumParts = (x) => {
    if (!core_1.isNumber(x))
        return false;
    if (x < 1)
        return false;
    return true;
};
exports.isNumParts = isNumParts;
const numPartsToNumber = (x) => {
    return x;
};
exports.numPartsToNumber = numPartsToNumber;
const numParts = (i) => {
    return i;
};
exports.numParts = numParts;
exports.examplePartIndex = 0;
const isPartIndex = (x) => {
    if (!core_1.isNumber(x))
        return false;
    if (x < 0)
        return false;
    return true;
};
exports.isPartIndex = isPartIndex;
const partIndexToNumber = (x) => {
    return x;
};
exports.partIndexToNumber = partIndexToNumber;
const partIndex = (i) => {
    return i;
};
exports.partIndex = partIndex;
const udpMessagePartId = (udpMessageId, partIndex, numParts) => {
    return `${udpMessageId}:${partIndex}:${numParts}`;
};
exports.udpMessagePartId = udpMessagePartId;
const isUdpMessageMetaData = (x) => {
    if (!core_1.isJSONObject(x))
        return false;
    if (crypto_util_1.JSONStringifyDeterministic(x).length > 100) {
        return false;
    }
    return true;
};
exports.isUdpMessageMetaData = isUdpMessageMetaData;
const udpMessageMetaData = (x) => {
    if (crypto_util_1.JSONStringifyDeterministic(x).length > 100) {
        throw Error('UDP message meta data is too large');
    }
    return x;
};
exports.udpMessageMetaData = udpMessageMetaData;
const isUdpHeader = (x) => {
    return core_1._validateObject(x, {
        body: {
            udpMessageId: exports.isUdpMessageId,
            protocolVersion: core_1.isEqualTo(protocolVersion_1.protocolVersion()),
            fromNodeId: core_1.isNodeId,
            toAddress: core_1.isOneOf([core_1.isNull, core_1.isAddress]),
            udpMessageType: exports.isUdpMessageType,
            metaData: exports.isUdpMessageMetaData,
            partIndex: exports.isPartIndex,
            numParts: exports.isNumParts,
            payloadIsJson: core_1.isBoolean
        },
        signature: core_1.isSignature
    });
};
exports.isUdpHeader = isUdpHeader;
//# sourceMappingURL=UdpMessage.js.map