"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _methodHealths, _health, _requestRecords;
Object.defineProperty(exports, "__esModule", { value: true });
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const core_1 = require("../interfaces/core");
class SendMessageMethodOptimizer {
    constructor(node, remoteNode) {
        this.node = node;
        this.remoteNode = remoteNode;
        _methodHealths.set(this, {
            'udp': new MethodHealth(),
            'http': new MethodHealth(),
            'http-proxy': new MethodHealth(),
            'websocket': new MethodHealth()
        });
    }
    reportSendRequestStart(requestId, method, timeoutMsec) {
        if ((method === 'udp') || (method === 'http') || (method === 'http-proxy') || (method === 'websocket')) {
            __classPrivateFieldGet(this, _methodHealths)[method].reportSendRequestStart(requestId, timeoutMsec);
        }
        else {
            /* istanbul ignore next */
            throw Error(`Unexpected method ${method} in reportSendRequestStart`);
        }
    }
    reportSendRequestEnd(requestId, method) {
        if ((method === 'udp') || (method === 'http') || (method === 'http-proxy') || (method === 'websocket')) {
            __classPrivateFieldGet(this, _methodHealths)[method].reportSendRequestEnd(requestId);
        }
        else {
            /* istanbul ignore next */
            throw Error('Unexpected method ${method} in reportSendRequestEnd');
        }
    }
    determineSendRequestMethod(method) {
        const websocketAvailable = (this.node.getProxyConnectionToClient(this.remoteNode.remoteNodeId()) || this.node.getProxyConnectionToServer(this.remoteNode.remoteNodeId())) ? true : false;
        const availableMethods = {
            'http': this.remoteNode.getRemoteNodeHttpAddress() ? true : false,
            'http-proxy': this.remoteNode.getRemoteNodeMessageProxyNode() ? true : false,
            'udp': ((this.node.publicUdpSocketServer()) && (this.remoteNode.getUdpAddressForRemoteNode())),
            'websocket': websocketAvailable
        };
        if (method === 'udp-fallback') {
            if (availableMethods['http']) {
                return 'http';
            }
            else if (availableMethods['websocket']) {
                return 'websocket';
            }
            else if (availableMethods['http-proxy']) {
                return 'http-proxy';
            }
            else {
                return null;
            }
        }
        else if (method === 'default') {
            return this.determineSendRequestMethod('prefer-http');
        }
        else if (method === 'prefer-http') {
            if (availableMethods['http']) {
                return 'http';
            }
            else if (availableMethods['websocket']) {
                return 'websocket';
            }
            else {
                if ((availableMethods['udp']) && (availableMethods['http-proxy'])) {
                    // use 'udp' or 'http-proxy', whichever has better health
                    if (__classPrivateFieldGet(this, _methodHealths)['udp'].health() >= __classPrivateFieldGet(this, _methodHealths)['http-proxy'].health()) {
                        return 'udp';
                    }
                    else {
                        return 'http-proxy';
                    }
                }
                else if (availableMethods['udp']) {
                    return 'udp';
                }
                else if (availableMethods['http-proxy']) {
                    return 'http-proxy';
                }
                else {
                    return null;
                }
            }
        }
        else if (method === 'prefer-udp') {
            if (availableMethods['udp']) {
                return 'udp';
            }
            else {
                return this.determineSendRequestMethod('prefer-http');
            }
        }
        else if ((method === 'http') || (method === 'websocket') || (method === 'udp') || (method === 'http-proxy')) {
            if (availableMethods[method])
                return method;
            else
                return null;
        }
        else {
            return null;
        }
    }
}
exports.default = SendMessageMethodOptimizer;
_methodHealths = new WeakMap();
class MethodHealth {
    constructor() {
        _health.set(this, 0);
        _requestRecords.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(2 * 60 * 1000)));
    }
    health() {
        return 0;
    }
    reportSendRequestStart(requestId, timeoutMsec) {
        __classPrivateFieldGet(this, _requestRecords).set(requestId, {
            timestamp: core_1.nowTimestamp(),
            timeoutMsec
        });
        setTimeout(() => {
            if (__classPrivateFieldGet(this, _requestRecords).has(requestId)) {
                // we still haven't gotten the response - give it a score of -1
                this._reportScore(requestId, -1);
            }
        }, core_1.durationMsecToNumber(timeoutMsec) * 2);
    }
    reportSendRequestEnd(requestId) {
        const r = __classPrivateFieldGet(this, _requestRecords).get(requestId);
        if (!r)
            return;
        const elapsed = core_1.elapsedSince(r.timestamp);
        if (elapsed <= core_1.durationMsecToNumber(r.timeoutMsec) * 0.5) {
            // it came back fast, give it a score of 1
            this._reportScore(requestId, 1);
        }
        else if (elapsed <= core_1.durationMsecToNumber(r.timeoutMsec) * 1) {
            // it came back reasonable, give it a score of 0.5
            this._reportScore(requestId, 0.5);
        }
        else if (elapsed <= core_1.durationMsecToNumber(r.timeoutMsec) * 1.5) {
            // at least it came back (sort of unexpected)
            this._reportScore(requestId, 0);
        }
        else {
            // it took too long (sort of unexpected)
            this._reportScore(requestId, -1);
        }
    }
    _reportScore(requestId, score) {
        const r = __classPrivateFieldGet(this, _requestRecords).get(requestId);
        if (!r)
            return;
        const alpha = 0.2;
        __classPrivateFieldSet(this, _health, __classPrivateFieldGet(this, _health) * (1 - alpha) + score * alpha);
        __classPrivateFieldGet(this, _requestRecords).delete(requestId);
    }
}
_health = new WeakMap(), _requestRecords = new WeakMap();
//# sourceMappingURL=SendMessageMethodOptimizer.js.map