"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _memoryCache, _mutableDatabase;
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_util_1 = require("../common/crypto_util");
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const core_1 = require("../interfaces/core");
const MutableDatabase_1 = __importDefault(require("./MutableDatabase"));
class MutableManager {
    constructor(storageDir) {
        // Manages the mutables
        _memoryCache.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(1000 * 60 * 10)));
        _mutableDatabase.set(this, void 0);
        __classPrivateFieldSet(this, _mutableDatabase, new MutableDatabase_1.default(core_1.localFilePath(storageDir + '/mutables.db')));
    }
    async set(key, value) {
        const sha1 = core_1.sha1OfString(crypto_util_1.JSONStringifyDeterministic(key));
        const rec = { key, value };
        await __classPrivateFieldGet(this, _mutableDatabase).set(sha1, rec);
        __classPrivateFieldGet(this, _memoryCache).set(sha1, rec);
    }
    async get(key) {
        const sha1 = core_1.sha1OfString(crypto_util_1.JSONStringifyDeterministic(key));
        const rec = __classPrivateFieldGet(this, _memoryCache).get(sha1);
        if (rec !== undefined) {
            return rec;
        }
        const rec2 = await __classPrivateFieldGet(this, _mutableDatabase).get(sha1);
        return rec2;
    }
    async delete(key) {
        const sha1 = core_1.sha1OfString(crypto_util_1.JSONStringifyDeterministic(key));
        await __classPrivateFieldGet(this, _mutableDatabase).delete(sha1);
        __classPrivateFieldGet(this, _memoryCache).delete(sha1);
    }
}
exports.default = MutableManager;
_memoryCache = new WeakMap(), _mutableDatabase = new WeakMap();
//# sourceMappingURL=MutableManager.js.map