"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleDownloadFileDataRequest = void 0;
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
const handleDownloadFileDataRequest = async (node, fromNodeId, requestData) => {
    let { fileKey, startByte, endByte } = requestData;
    if ((core_1.byteCountToNumber(startByte) < 0) || ((endByte !== null) && (core_1.byteCountToNumber(startByte) >= core_1.byteCountToNumber(endByte)))) {
        return {
            requestType: 'downloadFileData',
            fileKey,
            startByte,
            endByte: endByte === null ? core_1.byteCount(0) : endByte,
            success: false,
            streamId: null,
            errorMessage: core_1.errorMessage(`Invalid start/end bytes: ${startByte}/${endByte}`)
        };
    }
    const { found, size } = await node.kacheryStorageManager().findFile(fileKey);
    if (!found) {
        return {
            requestType: 'downloadFileData',
            fileKey,
            startByte,
            endByte: endByte === null ? core_1.byteCount(0) : endByte,
            success: false,
            streamId: null,
            errorMessage: core_1.errorMessage('Unable to find file')
        };
    }
    /* istanbul ignore next */
    if (size === null)
        throw Error('Unexpected null size in handleDownloadFileDataRequest');
    if (endByte === null) {
        endByte = size;
    }
    if (endByte > size) {
        return {
            requestType: 'downloadFileData',
            fileKey,
            startByte,
            endByte: endByte === null ? core_1.byteCount(0) : endByte,
            success: false,
            streamId: null,
            errorMessage: core_1.errorMessage('Start/end bytes out of range')
        };
    }
    const streamId = NodeToNodeRequest_1.createStreamId();
    node.downloadStreamManager().set(streamId, {
        ...requestData,
        endByte
    });
    return {
        requestType: 'downloadFileData',
        fileKey,
        startByte,
        endByte,
        success: true,
        streamId,
        errorMessage: null
    };
};
exports.handleDownloadFileDataRequest = handleDownloadFileDataRequest;
//# sourceMappingURL=handleDownloadFileDataRequest.js.map