"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleStartStreamViaUdpRequest = void 0;
const core_1 = require("../interfaces/core");
const handleStartStreamViaUdpRequest = async (node, fromNodeId, requestData) => {
    let { streamId } = requestData;
    const rn = node.remoteNodeManager().getRemoteNode(fromNodeId);
    if (!rn) {
        /* istanbul ignore next */
        return {
            requestType: 'startStreamViaUdp',
            success: false,
            errorMessage: core_1.errorMessage('Unable to find remote node')
        };
    }
    if (!rn.getUdpAddressForRemoteNode()) {
        /* istanbul ignore next */
        return {
            requestType: 'startStreamViaUdp',
            success: false,
            errorMessage: core_1.errorMessage('Cannot stream via udp when we do not have a udp address for the remote node')
        };
    }
    try {
        await rn.startStreamViaUdpToRemoteNode(streamId);
    }
    catch (err) {
        /* istanbul ignore next */
        return {
            requestType: 'startStreamViaUdp',
            success: false,
            errorMessage: core_1.errorMessage(err.message)
        };
    }
    return {
        requestType: 'startStreamViaUdp',
        success: true,
        errorMessage: null
    };
};
exports.handleStartStreamViaUdpRequest = handleStartStreamViaUdpRequest;
//# sourceMappingURL=handleStartStreamViaUdpRequest.js.map