"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleSubmitMessageToLiveFeedRequest = void 0;
const core_1 = require("../interfaces/core");
const handleSubmitMessageToLiveFeedRequest = async (node, fromNodeId, requestData) => {
    const { feedId, subfeedHash, message } = requestData;
    const hasWriteable = await node.feedManager().hasWriteableFeed(feedId);
    if (!hasWriteable) {
        return {
            requestType: 'submitMessageToLiveFeed',
            success: false,
            errorMessage: core_1.errorMessage('Node does not have writeable feed with that ID')
        };
    }
    const rules = await node.feedManager().getAccessRules({ feedId, subfeedHash });
    if ((rules === null) || (rules.rules.filter(r => ((r.nodeId === fromNodeId) && (r.write))).length === 0)) {
        return {
            requestType: 'submitMessageToLiveFeed',
            success: false,
            errorMessage: core_1.errorMessage('Permission denied')
        };
    }
    try {
        await node.feedManager().appendMessages({ feedId, subfeedHash, messages: [core_1.submittedSubfeedMessageToSubfeedMessage(message)] });
    }
    catch (err) {
        /* istanbul ignore next */
        return {
            requestType: 'submitMessageToLiveFeed',
            success: false,
            errorMessage: core_1.errorMessage('Error appending message')
        };
    }
    return {
        requestType: 'submitMessageToLiveFeed',
        success: true,
        errorMessage: null
    };
};
exports.handleSubmitMessageToLiveFeedRequest = handleSubmitMessageToLiveFeedRequest;
//# sourceMappingURL=handleSubmitMessageToLiveFeedRequest.js.map