"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleSubscribeToSubfeed = void 0;
const core_1 = require("../interfaces/core");
const handleSubscribeToSubfeed = async (node, fromNodeId, requestData) => {
    // CHAIN:get_remote_messages:step(8)
    const { feedId, subfeedHash } = requestData;
    if (!node.feedManager().hasWriteableFeed(feedId)) {
        return {
            requestType: 'subscribeToSubfeed',
            success: false,
            numMessages: null,
            errorMessage: core_1.errorMessage(`Does not have writeable feed: ${feedId.slice(0, 10)}`)
        };
    }
    try {
        // CHAIN:get_remote_messages:step(9)
        const numMessages = await node.feedManager().renewIncomingSubfeedSubscription(fromNodeId, feedId, subfeedHash);
        return {
            requestType: 'subscribeToSubfeed',
            success: true,
            numMessages,
            errorMessage: null
        };
    }
    catch (err) {
        return {
            requestType: 'subscribeToSubfeed',
            success: false,
            numMessages: null,
            errorMessage: core_1.errorMessage(`Error renewing subfeed subscription: ${err.message}`)
        };
    }
};
exports.handleSubscribeToSubfeed = handleSubscribeToSubfeed;
//# sourceMappingURL=handleSubscribeToSubfeed.js.map