"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _remoteNodeId, _ws, _initialized, _closed, _onClosedCallbacks, _onInitializedCallbacks, _proxyStreamFileDataCancelCallbacks, _responseListeners, _proxyStreamFileDataResponseMessageListeners;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProxyWebsocketConnection = exports.isProxyStreamFileDataRequestId = exports.isMessageFromServer = exports.isMessageFromClient = exports.isInitialMessageFromServer = exports.isInitialMessageFromServerBody = exports.isProxyStreamFileDataCancelRequest = exports.isProxyStreamFileDataRequest = exports.isInitialMessageFromClient = exports.isInitialMessageFromClientBody = void 0;
const action_1 = require("../common/action");
const crypto_util_1 = require("../common/crypto_util");
const DataStreamy_1 = __importDefault(require("../common/DataStreamy"));
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
const isInitialMessageFromClientBody = (x) => {
    return core_1._validateObject(x, {
        type: core_1.isEqualTo('proxyConnectionInitialMessageFromClient'),
        fromNodeId: core_1.isNodeId,
        toNodeId: core_1.isNodeId,
        timestamp: core_1.isTimestamp
    });
};
exports.isInitialMessageFromClientBody = isInitialMessageFromClientBody;
const isInitialMessageFromClient = (x) => {
    return core_1._validateObject(x, {
        body: exports.isInitialMessageFromClientBody,
        signature: core_1.isSignature
    });
};
exports.isInitialMessageFromClient = isInitialMessageFromClient;
const isProxyStreamFileDataRequest = (x) => {
    return core_1._validateObject(x, {
        messageType: core_1.isEqualTo('proxyStreamFileDataRequest'),
        proxyStreamFileDataRequestId: exports.isProxyStreamFileDataRequestId,
        streamId: NodeToNodeRequest_1.isStreamId
    });
};
exports.isProxyStreamFileDataRequest = isProxyStreamFileDataRequest;
const isProxyStreamFileDataCancelRequest = (x) => {
    return core_1._validateObject(x, {
        messageType: core_1.isEqualTo('proxyStreamFileDataCancelRequest'),
        proxyStreamFileDataRequestId: exports.isProxyStreamFileDataRequestId
    });
};
exports.isProxyStreamFileDataCancelRequest = isProxyStreamFileDataCancelRequest;
const isProxyStreamFileDataResponseMessageType = (x) => {
    if (!core_1.isString(x))
        return false;
    return [
        'started',
        'data',
        'finished',
        'error'
    ].includes(x);
};
const isProxyStreamFileDataResponseStartedMessage = (x) => {
    return core_1._validateObject(x, {
        proxyStreamFileDataRequestId: exports.isProxyStreamFileDataRequestId,
        messageType: core_1.isEqualTo('started'),
        size: core_1.isByteCount
    });
};
const isProxyStreamFileDataResponseDataMessage = (x) => {
    return core_1._validateObject(x, {
        proxyStreamFileDataRequestId: exports.isProxyStreamFileDataRequestId,
        messageType: isProxyStreamFileDataResponseMessageType,
        data: core_1.isBuffer
    });
};
const isProxyStreamFileDataResponseFinishedMessage = (x) => {
    return core_1._validateObject(x, {
        proxyStreamFileDataRequestId: exports.isProxyStreamFileDataRequestId,
        messageType: isProxyStreamFileDataResponseMessageType
    });
};
const isProxyStreamFileDataResponseErrorMessage = (x) => {
    return core_1._validateObject(x, {
        proxyStreamFileDataRequestId: exports.isProxyStreamFileDataRequestId,
        messageType: isProxyStreamFileDataResponseMessageType,
        errorMessage: core_1.isErrorMessage
    });
};
const isProxyStreamFileDataResponseMessage = (x) => {
    return (isProxyStreamFileDataResponseStartedMessage(x) ||
        isProxyStreamFileDataResponseDataMessage(x) ||
        isProxyStreamFileDataResponseFinishedMessage(x) ||
        isProxyStreamFileDataResponseErrorMessage(x));
};
const isInitialMessageFromServerBody = (x) => {
    return core_1._validateObject(x, {
        type: core_1.isEqualTo('proxyConnectionInitialMessageFromServer'),
        fromNodeId: core_1.isNodeId,
        toNodeId: core_1.isNodeId,
        timestamp: core_1.isTimestamp
    });
};
exports.isInitialMessageFromServerBody = isInitialMessageFromServerBody;
const isInitialMessageFromServer = (x) => {
    return core_1._validateObject(x, {
        body: exports.isInitialMessageFromServerBody,
        signature: core_1.isSignature
    });
};
exports.isInitialMessageFromServer = isInitialMessageFromServer;
const isMessageFromClient = (x) => {
    return NodeToNodeRequest_1.isNodeToNodeRequest(x) || NodeToNodeRequest_1.isNodeToNodeResponse(x) || isProxyStreamFileDataResponseMessage(x);
};
exports.isMessageFromClient = isMessageFromClient;
const isMessageFromServer = (x) => {
    return NodeToNodeRequest_1.isNodeToNodeRequest(x) || NodeToNodeRequest_1.isNodeToNodeResponse(x) || exports.isProxyStreamFileDataRequest(x) || exports.isProxyStreamFileDataCancelRequest(x);
};
exports.isMessageFromServer = isMessageFromServer;
const isProxyStreamFileDataRequestId = (x) => {
    if (!core_1.isString(x))
        return false;
    return (/^[A-Za-z]{10}$/.test(x));
};
exports.isProxyStreamFileDataRequestId = isProxyStreamFileDataRequestId;
const createProxyStreamFileDataRequestId = () => {
    return util_1.randomAlphaString(10);
};
class ProxyWebsocketConnection {
    constructor(node, opts) {
        this.opts = opts;
        _node.set(this, void 0);
        _remoteNodeId.set(this, null);
        _ws.set(this, void 0);
        _initialized.set(this, false);
        _closed.set(this, false);
        _onClosedCallbacks.set(this, []);
        _onInitializedCallbacks.set(this, []);
        _proxyStreamFileDataCancelCallbacks.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
        _responseListeners.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(5 * 60 * 1000)));
        _proxyStreamFileDataResponseMessageListeners.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
        __classPrivateFieldSet(this, _node, node);
    }
    async initializeConnectionToServer(remoteNodeId, address, opts) {
        if (this.opts.connectionType !== 'connectionToServer') {
            throw Error('Unexpected in initializeConnectionToServer');
        }
        __classPrivateFieldSet(this, _remoteNodeId, remoteNodeId);
        if (!address.hostName)
            throw Error('Unexpected in initializeConnectionToServer');
        return new Promise((resolve, reject) => {
            const url = `ws://${address.hostName}:${address.port}`;
            __classPrivateFieldSet(this, _ws, __classPrivateFieldGet(this, _node).externalInterface().createWebSocket(url, { timeoutMsec: opts.timeoutMsec }));
            __classPrivateFieldGet(this, _ws).onClose((code, reason) => {
                __classPrivateFieldSet(this, _closed, true);
                __classPrivateFieldGet(this, _onClosedCallbacks).forEach(cb => cb(reason));
            });
            __classPrivateFieldGet(this, _ws).onError(err => {
                /* istanbul ignore next */
                console.warn(err);
                // this is important so we don't throw an exception
                // question: do we need to do something here? will 'close' be called also?
            });
            this.onInitialized(() => {
                resolve();
            });
            this.onClosed((reason) => {
                if (!__classPrivateFieldGet(this, _initialized)) {
                    reject(reason);
                }
            });
            const msgBody = {
                type: 'proxyConnectionInitialMessageFromClient',
                fromNodeId: __classPrivateFieldGet(this, _node).nodeId(),
                toNodeId: remoteNodeId,
                timestamp: core_1.nowTimestamp()
            };
            const msg = {
                body: msgBody,
                signature: crypto_util_1.getSignature(msgBody, __classPrivateFieldGet(this, _node).keyPair())
            };
            __classPrivateFieldGet(this, _ws).onOpen(() => {
                const messageSerialized = util_1.kacheryP2PSerialize(msg);
                __classPrivateFieldGet(this, _node).stats().reportBytesSent('webSocket', __classPrivateFieldGet(this, _remoteNodeId), core_1.byteCount(messageSerialized.length));
                __classPrivateFieldGet(this, _ws).send(messageSerialized);
            });
            __classPrivateFieldGet(this, _ws).onMessage(messageBuffer => {
                if (__classPrivateFieldGet(this, _closed))
                    return;
                if (!core_1.isBuffer(messageBuffer))
                    throw Error('Unexpected message buffer in proxyConnectionToServerMessage');
                __classPrivateFieldGet(this, _node).stats().reportBytesReceived('webSocket', __classPrivateFieldGet(this, _remoteNodeId), core_1.byteCount(messageBuffer.length));
                /////////////////////////////////////////////////////////////////////////
                action_1.action('proxyConnectionToServerMessage', { context: "ProxyConnectionToServer", remoteNodeId: __classPrivateFieldGet(this, _remoteNodeId) }, async () => {
                    let messageParsed;
                    try {
                        messageParsed = util_1.kacheryP2PDeserialize(messageBuffer);
                    }
                    catch (err) {
                        __classPrivateFieldGet(this, _ws).close();
                        return;
                    }
                    if (!__classPrivateFieldGet(this, _initialized)) {
                        if (!exports.isInitialMessageFromServer(messageParsed)) {
                            /* istanbul ignore next */
                            console.warn(`Invalid initial websocket message from server. Closing.`);
                            /* istanbul ignore next */
                            __classPrivateFieldGet(this, _ws).close();
                            /* istanbul ignore next */
                            return;
                        }
                        if (messageParsed.body.toNodeId !== __classPrivateFieldGet(this, _node).nodeId()) {
                            /* istanbul ignore next */
                            console.warn(`Invalid initial websocket message from server (wrong toNodeId). Closing.`);
                            /* istanbul ignore next */
                            __classPrivateFieldGet(this, _ws).close();
                            /* istanbul ignore next */
                            return;
                        }
                        if (messageParsed.body.fromNodeId === __classPrivateFieldGet(this, _node).nodeId()) {
                            /* istanbul ignore next */
                            console.warn(`Invalid initial websocket message from server (invalid fromNodeId). Closing.`);
                            /* istanbul ignore next */
                            __classPrivateFieldGet(this, _ws).close();
                            /* istanbul ignore next */
                            return;
                        }
                        if (!crypto_util_1.verifySignature(messageParsed.body, messageParsed.signature, core_1.nodeIdToPublicKey(messageParsed.body.fromNodeId))) {
                            /* istanbul ignore next */
                            console.warn(`Invalid initial websocket message from server (invalid signature). Closing.`);
                            /* istanbul ignore next */
                            __classPrivateFieldGet(this, _ws).close();
                            /* istanbul ignore next */
                            return;
                        }
                        __classPrivateFieldSet(this, _initialized, true);
                        __classPrivateFieldGet(this, _onInitializedCallbacks).forEach(cb => { cb(); });
                    }
                    else {
                        if (!__classPrivateFieldGet(this, _remoteNodeId)) {
                            /* istanbul ignore next */
                            throw Error('Unexpected.');
                        }
                        if (!exports.isMessageFromServer(messageParsed)) {
                            /* istanbul ignore next */
                            console.warn(`Invalid websocket message from server. Closing.`);
                            /* istanbul ignore next */
                            __classPrivateFieldGet(this, _ws).close();
                            /* istanbul ignore next */
                            return;
                        }
                        this._handleMessageFromServer(messageParsed);
                    }
                }, async () => {
                    //
                });
                /////////////////////////////////////////////////////////////////////////
            });
        });
    }
    async initializeConnectionToClient(ws) {
        if (this.opts.connectionType !== 'connectionToClient') {
            throw Error('Unexpected type in initializeConnectionToClient');
        }
        return new Promise((resolve, reject) => {
            __classPrivateFieldSet(this, _ws, ws);
            __classPrivateFieldGet(this, _ws).onClose((code, reason) => {
                /////////////////////////////////////////////////////////////////////////
                action_1.action('proxyConnectionToClientClosed', { context: "ProxyConnectionToClient", remoteNodeId: __classPrivateFieldGet(this, _remoteNodeId) }, async () => {
                    __classPrivateFieldSet(this, _closed, true);
                    __classPrivateFieldGet(this, _onClosedCallbacks).forEach(cb => cb(reason));
                }, null);
                /////////////////////////////////////////////////////////////////////////
            });
            __classPrivateFieldGet(this, _ws).onError((err) => {
                // this is important so we don't throw an exception
                // question: do we need to do something here? will 'close' be called also?
                /* istanbul ignore next */
                console.warn(err);
            });
            this.onInitialized(() => {
                resolve();
            });
            this.onClosed((reason) => {
                if (!__classPrivateFieldGet(this, _initialized)) {
                    reject(reason);
                }
            });
            ws.onMessage(messageBuffer => {
                if (__classPrivateFieldGet(this, _closed))
                    return;
                if (!core_1.isBuffer(messageBuffer))
                    throw Error('Unexpected message buffer in proxyConnectionToClientMessage');
                __classPrivateFieldGet(this, _node).stats().reportBytesReceived('webSocket', __classPrivateFieldGet(this, _remoteNodeId), core_1.byteCount(messageBuffer.length));
                /////////////////////////////////////////////////////////////////////////
                action_1.action('proxyConnectionToClientMessage', { context: "ProxyConnectionToClient", remoteNodeId: __classPrivateFieldGet(this, _remoteNodeId) }, async () => {
                    /* istanbul ignore next */
                    let messageParsed;
                    try {
                        messageParsed = util_1.kacheryP2PDeserialize(messageBuffer);
                    }
                    catch (err) {
                        __classPrivateFieldGet(this, _ws).close();
                        return;
                    }
                    if (!__classPrivateFieldGet(this, _initialized)) {
                        if (!exports.isInitialMessageFromClient(messageParsed)) {
                            /* istanbul ignore next */
                            {
                                console.warn(`Invalid initial websocket message from client. Closing.`);
                                __classPrivateFieldGet(this, _ws).close();
                                return;
                            }
                        }
                        if (messageParsed.body.toNodeId !== __classPrivateFieldGet(this, _node).nodeId()) {
                            /* istanbul ignore next */
                            {
                                console.warn(`Invalid initial websocket message from client (wrong toNodeId) (${messageParsed.body.toNodeId} <> ${__classPrivateFieldGet(this, _node).nodeId()}). Closing.`);
                                __classPrivateFieldGet(this, _ws).close();
                                return;
                            }
                        }
                        if (messageParsed.body.fromNodeId === __classPrivateFieldGet(this, _node).nodeId()) {
                            /* istanbul ignore next */
                            {
                                console.warn(`Invalid initial websocket message from client (invalid fromNodeId). Closing.`);
                                __classPrivateFieldGet(this, _ws).close();
                                return;
                            }
                        }
                        if (!crypto_util_1.verifySignature(messageParsed.body, messageParsed.signature, core_1.nodeIdToPublicKey(messageParsed.body.fromNodeId))) {
                            /* istanbul ignore next */
                            {
                                console.warn(`Invalid initial websocket message from client (invalid signature). Closing.`);
                                __classPrivateFieldGet(this, _ws).close();
                                return;
                            }
                        }
                        __classPrivateFieldSet(this, _initialized, true);
                        __classPrivateFieldSet(this, _remoteNodeId, messageParsed.body.fromNodeId);
                        const msgBody = {
                            type: 'proxyConnectionInitialMessageFromServer',
                            fromNodeId: __classPrivateFieldGet(this, _node).nodeId(),
                            toNodeId: __classPrivateFieldGet(this, _remoteNodeId),
                            timestamp: core_1.nowTimestamp()
                        };
                        const msg = {
                            body: msgBody,
                            signature: crypto_util_1.getSignature(msgBody, __classPrivateFieldGet(this, _node).keyPair())
                        };
                        const messageSerialized = util_1.kacheryP2PSerialize(msg);
                        __classPrivateFieldGet(this, _node).stats().reportBytesSent('webSocket', __classPrivateFieldGet(this, _remoteNodeId), core_1.byteCount(messageSerialized.length));
                        __classPrivateFieldGet(this, _ws).send(messageSerialized);
                        __classPrivateFieldGet(this, _onInitializedCallbacks).forEach(cb => { cb(); });
                    }
                    else {
                        if (!__classPrivateFieldGet(this, _remoteNodeId))
                            throw Error('Unexpected.');
                        if (!exports.isMessageFromClient(messageParsed)) {
                            /* istanbul ignore next */
                            {
                                console.warn(`Invalid websocket message from client. Closing.`);
                                __classPrivateFieldGet(this, _ws).close();
                                return;
                            }
                        }
                        this._handleMessageFromClient(messageParsed);
                    }
                }, null);
                /////////////////////////////////////////////////////////////////////////
            });
        });
    }
    streamDataForStreamId(streamId) {
        if (this.opts.connectionType !== 'connectionToClient') {
            throw Error('Unexpected type in streamDataForStreamId');
        }
        const ret = new DataStreamy_1.default();
        const _handleResponseMessageFromServer = (msg) => {
            if (isProxyStreamFileDataResponseStartedMessage(msg)) {
                ret.producer().start(msg.size);
            }
            else if (isProxyStreamFileDataResponseDataMessage(msg)) {
                ret.producer().data(msg.data);
            }
            else if (isProxyStreamFileDataResponseFinishedMessage(msg)) {
                ret.producer().end();
            }
            else if (isProxyStreamFileDataResponseErrorMessage(msg)) {
                ret.producer().error(Error(msg.errorMessage.toString()));
            }
            else {
                /* istanbul ignore next */
                throw Error('Unexpected in _handleResponseMessageFromServer');
            }
        };
        const proxyStreamFileDataRequestId = createProxyStreamFileDataRequestId();
        const request = {
            messageType: 'proxyStreamFileDataRequest',
            proxyStreamFileDataRequestId,
            streamId
        };
        __classPrivateFieldGet(this, _proxyStreamFileDataResponseMessageListeners).set(proxyStreamFileDataRequestId, (msg) => {
            _handleResponseMessageFromServer(msg);
        });
        this._sendMessageToRemote(request);
        ret.producer().onCancelled(() => {
            const cancelRequest = {
                messageType: 'proxyStreamFileDataCancelRequest',
                proxyStreamFileDataRequestId
            };
            this._sendMessageToRemote(cancelRequest);
        });
        return ret;
    }
    onInitialized(callback) {
        __classPrivateFieldGet(this, _onInitializedCallbacks).push(callback);
    }
    close() {
        __classPrivateFieldGet(this, _ws).close();
    }
    onClosed(callback) {
        __classPrivateFieldGet(this, _onClosedCallbacks).push(callback);
    }
    async sendRequest(request, opts) {
        this._sendMessageToRemote(request);
        return await this._waitForResponse(request.body.requestId, { timeoutMsec: opts.timeoutMsec, requestType: request.body.requestData.requestType });
    }
    remoteNodeId() {
        if (!__classPrivateFieldGet(this, _remoteNodeId)) {
            /* istanbul ignore next */
            throw Error('Unexpected. remoteNodeId() called before initialized.');
        }
        return __classPrivateFieldGet(this, _remoteNodeId);
    }
    async _handleMessageFromServer(message) {
        if (this.opts.connectionType !== 'connectionToServer')
            throw Error('Unexpected type in _handleMessageFromServer');
        if (__classPrivateFieldGet(this, _closed))
            return;
        if (NodeToNodeRequest_1.isNodeToNodeRequest(message)) {
            const response = await __classPrivateFieldGet(this, _node).handleNodeToNodeRequest(message);
            this._sendMessageToRemote(response);
        }
        else if (NodeToNodeRequest_1.isNodeToNodeResponse(message)) {
            const callback = __classPrivateFieldGet(this, _responseListeners).get(message.body.requestId);
            if (!callback) {
                // must have timed out
                return;
            }
            callback(message);
        }
        else if (exports.isProxyStreamFileDataRequest(message)) {
            await this._handleProxyStreamFileDataRequest(message);
        }
        else if (exports.isProxyStreamFileDataCancelRequest(message)) {
            await this._handleProxyStreamFileDataCancelRequest(message);
        }
        else {
            throw Error('Unexpected message from server');
        }
    }
    async _handleMessageFromClient(message) {
        if (this.opts.connectionType !== 'connectionToClient')
            throw Error('Unexpected type in _handleMessageFromClient');
        if (__classPrivateFieldGet(this, _closed))
            return;
        if (NodeToNodeRequest_1.isNodeToNodeRequest(message)) {
            const response = await __classPrivateFieldGet(this, _node).handleNodeToNodeRequest(message);
            this._sendMessageToRemote(response);
        }
        else if (NodeToNodeRequest_1.isNodeToNodeResponse(message)) {
            const callback = __classPrivateFieldGet(this, _responseListeners).get(message.body.requestId);
            if (!callback) {
                // must have timed out
                return;
            }
            callback(message);
        }
        else if (isProxyStreamFileDataResponseMessage(message)) {
            const a = __classPrivateFieldGet(this, _proxyStreamFileDataResponseMessageListeners).get(message.proxyStreamFileDataRequestId);
            if (a) {
                a(message);
            }
        }
        else {
            /* istanbul ignore next */
            throw Error('Unexpected message from client');
        }
    }
    async _handleProxyStreamFileDataRequest(request) {
        if (this.opts.connectionType !== 'connectionToServer')
            throw Error('Unexpected type in _handleProxyStreamFileDataRequest');
        // the server is requesting to stream file data up from the client
        const { streamId } = request;
        const s = await __classPrivateFieldGet(this, _node).streamDataForStreamId(__classPrivateFieldGet(this, _node).nodeId(), streamId);
        s.onStarted(size => {
            if (size === null) {
                /* istanbul ignore next */
                throw Error('unexpected.');
            }
            const response = {
                messageType: 'started',
                proxyStreamFileDataRequestId: request.proxyStreamFileDataRequestId,
                size
            };
            this._sendMessageToRemote(response);
        });
        s.onData(data => {
            const response = {
                messageType: 'data',
                proxyStreamFileDataRequestId: request.proxyStreamFileDataRequestId,
                data
            };
            this._sendMessageToRemote(response);
        });
        s.onFinished(() => {
            const response = {
                messageType: 'finished',
                proxyStreamFileDataRequestId: request.proxyStreamFileDataRequestId
            };
            this._sendMessageToRemote(response);
        });
        s.onError((err) => {
            const response = {
                messageType: 'error',
                proxyStreamFileDataRequestId: request.proxyStreamFileDataRequestId,
                errorMessage: core_1.errorMessage(err.message)
            };
            this._sendMessageToRemote(response);
        });
        __classPrivateFieldGet(this, _proxyStreamFileDataCancelCallbacks).set(request.proxyStreamFileDataRequestId, () => { s.cancel(); });
    }
    async _handleProxyStreamFileDataCancelRequest(request) {
        if (this.opts.connectionType !== 'connectionToServer')
            throw Error('Unexpected type in _handleProxyStreamFileDataCancelRequest');
        const cb = __classPrivateFieldGet(this, _proxyStreamFileDataCancelCallbacks).get(request.proxyStreamFileDataRequestId);
        if (cb)
            cb();
    }
    _sendMessageToRemote(msg) {
        if (!__classPrivateFieldGet(this, _initialized)) {
            /* istanbul ignore next */
            throw Error('Cannot send message over websocket before initialized.');
        }
        if (__classPrivateFieldGet(this, _closed))
            return;
        const messageSerialized = util_1.kacheryP2PSerialize(msg);
        __classPrivateFieldGet(this, _node).stats().reportBytesSent('webSocket', __classPrivateFieldGet(this, _remoteNodeId), core_1.byteCount(messageSerialized.length));
        __classPrivateFieldGet(this, _ws).send(messageSerialized);
    }
    async _waitForResponse(requestId, { timeoutMsec, requestType }) {
        return new Promise((resolve, reject) => {
            let completed = false;
            __classPrivateFieldGet(this, _responseListeners).set(requestId, (response) => {
                if (!completed) {
                    completed = true;
                    __classPrivateFieldGet(this, _responseListeners).delete(requestId);
                    resolve(response);
                }
            });
            (async () => {
                const timer = core_1.nowTimestamp();
                while (true) {
                    if (completed)
                        return;
                    if (__classPrivateFieldGet(this, _closed)) {
                        /* istanbul ignore next */
                        {
                            reject(Error('Websocket closed while waiting for response'));
                            return;
                        }
                    }
                    const elapsed = core_1.elapsedSince(timer);
                    if (elapsed >= core_1.durationMsecToNumber(timeoutMsec)) {
                        reject(Error(`Timeout while waiting for ${requestType} response in proxy connection to client.`));
                        return;
                    }
                    await util_1.sleepMsec(core_1.minDuration(core_1.scaledDurationMsec(800), timeoutMsec));
                }
            })();
        });
    }
}
exports.ProxyWebsocketConnection = ProxyWebsocketConnection;
_node = new WeakMap(), _remoteNodeId = new WeakMap(), _ws = new WeakMap(), _initialized = new WeakMap(), _closed = new WeakMap(), _onClosedCallbacks = new WeakMap(), _onInitializedCallbacks = new WeakMap(), _proxyStreamFileDataCancelCallbacks = new WeakMap(), _responseListeners = new WeakMap(), _proxyStreamFileDataResponseMessageListeners = new WeakMap();
//# sourceMappingURL=ProxyWebsocketConnection.js.map