"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _remoteNodeManager, _halted, _announceHistoryTimestamps;
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = require("../common/action");
const constants_1 = require("../common/constants");
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const util_1 = require("../common/util");
const httpRequests_1 = require("../external/real/httpRequests");
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
class AnnounceService {
    constructor(node, opts) {
        this.opts = opts;
        _node.set(this, void 0);
        _remoteNodeManager.set(this, void 0);
        _halted.set(this, false);
        _announceHistoryTimestamps.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
        __classPrivateFieldSet(this, _node, node);
        __classPrivateFieldSet(this, _remoteNodeManager, node.remoteNodeManager()
        // announce self when a new node-channel has been added
        );
        // announce self when a new node-channel has been added
        __classPrivateFieldGet(this, _remoteNodeManager).onNodeChannelAdded(async (remoteNodeId, channelConfigUrl) => {
            if (__classPrivateFieldGet(this, _halted))
                return;
            // check if we can send message to node, if not, delay a bit
            let numPasses = 0;
            while (!__classPrivateFieldGet(this, _remoteNodeManager).canSendRequestToNode(remoteNodeId, 'default')) {
                if (numPasses > 3)
                    return;
                numPasses++;
                await util_1.sleepMsec(core_1.scaledDurationMsec(1000));
            }
            if (__classPrivateFieldGet(this, _node).hasJoinedChannel(channelConfigUrl)) { // only if we belong to this channel
                const rn = __classPrivateFieldGet(this, _remoteNodeManager).getRemoteNode(remoteNodeId);
                if ((!rn) || (!rn.isAuthorizedToCommunicate()))
                    return;
                /////////////////////////////////////////////////////////////////////////
                action_1.action('announceToNewNode', { context: 'AnnounceService', remoteNodeId }, async () => {
                    await this._announceToNode(remoteNodeId, channelConfigUrl);
                }, null);
                /////////////////////////////////////////////////////////////////////////
            }
        });
        __classPrivateFieldGet(this, _remoteNodeManager).onBootstrapNodeAdded((bootstrapNodeId) => {
            if (__classPrivateFieldGet(this, _halted))
                return;
            const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
            for (let channelConfigUrl of channelConfigUrls) {
                /////////////////////////////////////////////////////////////////////////
                action_1.action('announceToNewBootstrap', { context: 'AnnounceService', bootstrapNodeId, channelConfigUrl }, async () => {
                    await this._announceToNode(bootstrapNodeId, channelConfigUrl);
                }, null);
                /////////////////////////////////////////////////////////////////////////
            }
        });
        __classPrivateFieldGet(this, _node).onProxyConnectionToServer(() => {
            if (__classPrivateFieldGet(this, _halted))
                return;
            this._announceToAllBootstrapNodes();
        });
        this._start();
    }
    stop() {
        __classPrivateFieldSet(this, _halted, true);
    }
    async _announceToNode(remoteNodeId, channelConfigUrl) {
        let numPasses = 0;
        while (!__classPrivateFieldGet(this, _remoteNodeManager).canSendRequestToNode(remoteNodeId, 'default')) {
            numPasses++;
            if (numPasses > 3)
                return;
            await util_1.sleepMsec(core_1.scaledDurationMsec(1500));
        }
        const requestData = {
            requestType: 'announce',
            channelNodeInfo: await __classPrivateFieldGet(this, _node).getChannelNodeInfo(channelConfigUrl)
        };
        let method = 'prefer-udp'; // we prefer to send via udp so that we can discover our own public udp address when we get the response
        let responseData;
        try {
            responseData = await __classPrivateFieldGet(this, _remoteNodeManager).sendRequestToNode(remoteNodeId, requestData, { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, method });
        }
        catch (err) {
            if ((err instanceof httpRequests_1.HttpPostJsonError) || (err instanceof util_1.RequestTimeoutError)) {
                // the node is probably not connected
                return;
            }
            else {
                throw err;
            }
        }
        if (!NodeToNodeRequest_1.isAnnounceResponseData(responseData)) {
            throw Error('Unexpected.');
        }
        if (!responseData.success) {
            // what should we do here? remove the node?
            console.warn(`Response error for announce: ${responseData.errorMessage}`);
        }
    }
    async _announceToAllBootstrapNodes() {
        const bootstrapNodes = __classPrivateFieldGet(this, _remoteNodeManager).getBootstrapRemoteNodes({ includeOffline: false });
        const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
        for (let bootstrapNode of bootstrapNodes) {
            for (let channelConfigUrl of channelConfigUrls) {
                if (bootstrapNode.isOnline()) {
                    /////////////////////////////////////////////////////////////////////////
                    await action_1.action('announceToNode', { context: 'AnnounceService', bootstrapNodeId: bootstrapNode.remoteNodeId(), channelConfigUrl }, async () => {
                        await this._announceToNode(bootstrapNode.remoteNodeId(), channelConfigUrl);
                    }, async (err) => {
                        console.warn(`Problem announcing to bootstrap node ${bootstrapNode.remoteNodeId().slice(0, 6)} (${err.message})`);
                    });
                    /////////////////////////////////////////////////////////////////////////
                }
            }
        }
    }
    async _start() {
        const timestampStarted = core_1.nowTimestamp();
        await util_1.sleepMsecNum(2); // important for tests
        // Announce self other nodes in our channels and to bootstrap nodes
        let lastBootstrapAnnounceTimestamp = core_1.zeroTimestamp();
        let lastIndividualNodeAnnounceTimestamp = core_1.zeroTimestamp();
        while (true) {
            if (__classPrivateFieldGet(this, _halted))
                return;
            const startingUp = core_1.elapsedSince(timestampStarted) < core_1.durationMsecToNumber(core_1.scaledDurationMsec(15000));
            const announceBootstrapIntervalMsec = startingUp ? core_1.scaledDurationMsec(2000) : this.opts.announceBootstrapIntervalMsec;
            const announceToIndividualNodeIntervalMsec = startingUp ? core_1.scaledDurationMsec(500) : this.opts.announceToIndividualNodeIntervalMsec;
            // periodically announce to bootstrap nodes
            const elapsedSinceLastBootstrapAnnounce = core_1.elapsedSince(lastBootstrapAnnounceTimestamp);
            if (elapsedSinceLastBootstrapAnnounce > core_1.durationMsecToNumber(announceBootstrapIntervalMsec)) {
                await this._announceToAllBootstrapNodes();
                lastBootstrapAnnounceTimestamp = core_1.nowTimestamp();
            }
            const elapsedSinceLastIndividualNodeAnnounce = core_1.elapsedSince(lastIndividualNodeAnnounceTimestamp);
            if (elapsedSinceLastIndividualNodeAnnounce > core_1.durationMsecToNumber(announceToIndividualNodeIntervalMsec)) {
                // for each channel, choose a node and announce to that node
                const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
                for (let channelConfigUrl of channelConfigUrls) {
                    let nodes = __classPrivateFieldGet(this, _remoteNodeManager).getRemoteNodesInChannel(channelConfigUrl, { includeOffline: false });
                    if (nodes.length > 0) {
                        var individualNode = selectNode(nodes, channelConfigUrl, __classPrivateFieldGet(this, _announceHistoryTimestamps));
                        __classPrivateFieldGet(this, _announceHistoryTimestamps).set(getCode(individualNode.remoteNodeId(), channelConfigUrl), core_1.nowTimestamp());
                        /////////////////////////////////////////////////////////////////////////
                        await action_1.action('announceToIndividualNode', { context: 'AnnounceService', remoteNodeId: individualNode.remoteNodeId(), channelConfigUrl }, async () => {
                            await this._announceToNode(individualNode.remoteNodeId(), channelConfigUrl);
                        }, async (err) => {
                            console.warn(`Problem announcing to individual node ${individualNode.remoteNodeId().slice(0, 6)} (${err.message})`);
                        });
                        /////////////////////////////////////////////////////////////////////////
                    }
                }
                lastIndividualNodeAnnounceTimestamp = core_1.nowTimestamp();
            }
            await util_1.sleepMsec(core_1.scaledDurationMsec(500), () => { return !__classPrivateFieldGet(this, _halted); });
        }
    }
}
exports.default = AnnounceService;
_node = new WeakMap(), _remoteNodeManager = new WeakMap(), _halted = new WeakMap(), _announceHistoryTimestamps = new WeakMap();
// thanks: https://gist.github.com/engelen/fbce4476c9e68c52ff7e5c2da5c24a28
function argMax(array) {
    if (array.length === 0)
        throw Error('Unexpected in argMax');
    return array.map((x, i) => [x, i]).reduce((r, a) => (a[0] > r[0] ? a : r))[1];
}
const selectNode = (nodes, channelConfigUrl, historyTimestamps) => {
    const n = nodes[0];
    if (!n)
        throw Error('Unexpected in selectNode');
    const timestamps = nodes.map(n => (historyTimestamps.getWithDefault(getCode(n.remoteNodeId(), channelConfigUrl), core_1.zeroTimestamp())));
    const elapsedTimes = timestamps.map(ts => (core_1.elapsedSince(ts)));
    const ind = argMax(elapsedTimes);
    return nodes[ind];
};
const getCode = (nodeId, channelConfigUrl) => {
    return nodeId.toString() + ':' + channelConfigUrl.toString();
};
//# sourceMappingURL=AnnounceService.js.map