"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _halted, _currentClientAuthCode;
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const child_process_1 = __importDefault(require("child_process"));
const fs_1 = __importDefault(require("fs"));
const os_1 = require("os");
class ClientAuthService {
    constructor(node, opts) {
        this.opts = opts;
        _node.set(this, void 0);
        _halted.set(this, false);
        _currentClientAuthCode.set(this, createClientAuthCode());
        __classPrivateFieldSet(this, _node, node);
        this._start();
    }
    stop() {
        __classPrivateFieldSet(this, _halted, true);
    }
    async _start() {
        const intervalMsec = core_1.scaledDurationMsec(1000 * 60 * 3);
        // this service should not wait before starting
        while (true) {
            if (__classPrivateFieldGet(this, _halted))
                return;
            const previous = __classPrivateFieldGet(this, _currentClientAuthCode);
            __classPrivateFieldSet(this, _currentClientAuthCode, createClientAuthCode());
            const clientAuthPath = __classPrivateFieldGet(this, _node).kacheryStorageManager().storageDir() + '/client-auth';
            await fs_1.default.promises.writeFile(clientAuthPath, __classPrivateFieldGet(this, _currentClientAuthCode), { mode: fs_1.default.constants.S_IRUSR | fs_1.default.constants.S_IRGRP | fs_1.default.constants.S_IWUSR });
            const group = this.opts.clientAuthGroup;
            if (group) {
                const user = os_1.userInfo().username;
                try {
                    child_process_1.default.execSync(`chown ${user}:${group} ${clientAuthPath}`);
                }
                catch (e) {
                    console.warn(`Problem setting ownership of client auth file. Perhaps you do not belong to group "${group}".`, e.message);
                    console.warn('ABORTING');
                    process.exit(1);
                }
            }
            __classPrivateFieldGet(this, _node).setClientAuthCode(__classPrivateFieldGet(this, _currentClientAuthCode), previous);
            await util_1.sleepMsec(intervalMsec, () => { return !__classPrivateFieldGet(this, _halted); });
        }
    }
}
exports.default = ClientAuthService;
_node = new WeakMap(), _halted = new WeakMap(), _currentClientAuthCode = new WeakMap();
const createClientAuthCode = () => {
    return util_1.randomAlphaString(12);
};
//# sourceMappingURL=ClientAuthService.js.map