"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _node, _halted;
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMirrorSourceConfig = exports.isJoinedChannelConfig = void 0;
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const isJoinedChannelConfig = (x) => {
    return core_1._validateObject(x, {
        channelConfigUrl: core_1.isChannelConfigUrl,
        isMessageProxy: core_1.optional(core_1.isBoolean),
        isDataProxy: core_1.optional(core_1.isBoolean),
        isPublic: core_1.optional(core_1.isBoolean)
    }, { allowAdditionalFields: true });
};
exports.isJoinedChannelConfig = isJoinedChannelConfig;
const isMirrorSourceConfig = (x) => {
    return core_1._validateObject(x, {
        uri: core_1.isString,
        label: core_1.optional(core_1.isString)
    }, { allowAdditionalFields: true });
};
exports.isMirrorSourceConfig = isMirrorSourceConfig;
const isJoinedChannelsConfig = (x) => {
    return core_1._validateObject(x, {
        joinedChannels: core_1.isArrayOf(exports.isJoinedChannelConfig)
    });
};
class ConfigUpdateService {
    constructor(node, opts) {
        this.opts = opts;
        _node.set(this, void 0);
        _halted.set(this, false);
        __classPrivateFieldSet(this, _node, node);
        this._start();
    }
    stop() {
        __classPrivateFieldSet(this, _halted, true);
    }
    async _start() {
        if (this.opts.staticConfigPathOrUrl) {
            const config = await util_1.loadYamlFromPathOrUrl(this.opts.staticConfigPathOrUrl);
            if (config.joinedChannels) {
                // todo: this code is duplicated below
                const joinedChannels = [];
                for (let joinedChannel of config.joinedChannels) {
                    if (exports.isJoinedChannelConfig(joinedChannel)) {
                        joinedChannels.push(joinedChannel);
                    }
                    else {
                        console.warn(joinedChannel);
                        console.warn('Invalid joined channel config');
                    }
                }
                __classPrivateFieldGet(this, _node).setJoinedChannels(joinedChannels);
            }
            if (config.mirrorSources) {
                const mirrorSources = [];
                for (let mirrorSource of config.mirrorSources) {
                    if (exports.isMirrorSourceConfig(mirrorSource)) {
                        mirrorSources.push(mirrorSource);
                    }
                    else {
                        console.warn(mirrorSource);
                        console.warn('Invalid mirror source config');
                    }
                }
                __classPrivateFieldGet(this, _node).setMirrorSources(mirrorSources);
            }
        }
        while (true) {
            if (__classPrivateFieldGet(this, _halted))
                return;
            if (!this.opts.staticConfigPathOrUrl) {
                const configFeedId = await __classPrivateFieldGet(this, _node).feedManager().getFeedId({ feedName: core_1.feedName('_kachery_p2p_config') });
                if (configFeedId) {
                    const joinedChannelsSubfeedHash = core_1.subfeedHash(core_1.sha1OfString('joined-channels'));
                    const joinedChannelsConfig = await __classPrivateFieldGet(this, _node).feedManager().getFinalLocalMessage({ feedId: configFeedId, subfeedHash: joinedChannelsSubfeedHash });
                    if (joinedChannelsConfig) {
                        if (isJoinedChannelsConfig(joinedChannelsConfig)) {
                            __classPrivateFieldGet(this, _node).setJoinedChannels(joinedChannelsConfig.joinedChannels);
                        }
                        else {
                            console.warn(joinedChannelsConfig);
                            console.warn('Invalid joined channels config');
                        }
                    }
                }
            }
            await util_1.sleepMsec(this.opts.intervalMsec, () => { return !__classPrivateFieldGet(this, _halted); });
        }
    }
}
exports.default = ConfigUpdateService;
_node = new WeakMap(), _halted = new WeakMap();
//# sourceMappingURL=ConfigUpdateService.js.map