"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _remoteNodeManager, _halted, _discoverHistoryTimestamps;
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = require("../common/action");
const constants_1 = require("../common/constants");
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const util_1 = require("../common/util");
const httpRequests_1 = require("../external/real/httpRequests");
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
class DiscoverService {
    constructor(node, opts) {
        this.opts = opts;
        _node.set(this, void 0);
        _remoteNodeManager.set(this, void 0);
        _halted.set(this, false);
        _discoverHistoryTimestamps.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
        __classPrivateFieldSet(this, _node, node);
        __classPrivateFieldSet(this, _remoteNodeManager, node.remoteNodeManager());
        __classPrivateFieldGet(this, _remoteNodeManager).onBootstrapNodeAdded((bootstrapNodeId) => {
            if (__classPrivateFieldGet(this, _halted))
                return;
            const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
            for (let channelConfigUrl of channelConfigUrls) {
                /////////////////////////////////////////////////////////////////////////
                action_1.action('discoverFromNewBootstrap', { context: 'DiscoverService', bootstrapNodeId, channelConfigUrl }, async () => {
                    await this._getChannelInfoFromNode(bootstrapNodeId, channelConfigUrl);
                }, null);
                /////////////////////////////////////////////////////////////////////////
            }
        });
        this._start();
    }
    stop() {
        __classPrivateFieldSet(this, _halted, true);
    }
    async _getChannelInfoFromNode(remoteNodeId, channelConfigUrl) {
        let numPasses = 0;
        while (!__classPrivateFieldGet(this, _remoteNodeManager).canSendRequestToNode(remoteNodeId, 'default')) {
            numPasses++;
            if (numPasses > 3)
                return;
            await util_1.sleepMsec(core_1.scaledDurationMsec(1500));
        }
        const requestData = {
            requestType: 'getChannelInfo',
            channelConfigUrl
        };
        let responseData;
        try {
            responseData = await __classPrivateFieldGet(this, _remoteNodeManager).sendRequestToNode(remoteNodeId, requestData, { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, method: 'default' });
        }
        catch (err) {
            if (err instanceof httpRequests_1.HttpPostJsonError) {
                // the node is probably not connected
                return;
            }
            else {
                throw err;
            }
        }
        if (!NodeToNodeRequest_1.isGetChannelInfoResponseData(responseData)) {
            throw Error(`Unexpected GetChannelInfoResponseData from node: ${remoteNodeId.slice(0, 6)}`);
        }
        const { channelInfo } = responseData;
        for (let channelNodeInfo of channelInfo.nodes) {
            if (channelNodeInfo.body.nodeId !== __classPrivateFieldGet(this, _node).nodeId()) {
                await __classPrivateFieldGet(this, _remoteNodeManager).setChannelNodeInfo(channelNodeInfo);
            }
        }
    }
    async _start() {
        const timestampStarted = core_1.nowTimestamp();
        // Get channel info from other nodes in our channels
        let lastBootstrapDiscoverTimestamp = core_1.zeroTimestamp();
        let lastIndividualNodeDiscoverTimestamp = core_1.zeroTimestamp();
        while (true) {
            if (__classPrivateFieldGet(this, _halted))
                return;
            // periodically get channel info from bootstrap nodes
            const startingUp = core_1.elapsedSince(timestampStarted) < core_1.durationMsecToNumber(core_1.scaledDurationMsec(15000));
            const discoverBootstrapIntervalMsec = startingUp ? core_1.scaledDurationMsec(2000) : this.opts.discoverBootstrapIntervalMsec;
            const discoverIndividualNodeIntervalMsec = startingUp ? core_1.scaledDurationMsec(500) : this.opts.discoverIndividualNodeIntervalMsec;
            const elapsedSinceLastBootstrapDiscover = core_1.elapsedSince(lastBootstrapDiscoverTimestamp);
            if (elapsedSinceLastBootstrapDiscover > core_1.durationMsecToNumber(discoverBootstrapIntervalMsec)) {
                const bootstrapNodes = __classPrivateFieldGet(this, _remoteNodeManager).getBootstrapRemoteNodes({ includeOffline: false });
                const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
                for (let bootstrapNode of bootstrapNodes) {
                    for (let channelConfigUrl of channelConfigUrls) {
                        /////////////////////////////////////////////////////////////////////////
                        await action_1.action('discoverFromBootstrapNode', { context: 'DiscoverService', bootstrapNodeId: bootstrapNode.remoteNodeId(), channelConfigUrl }, async () => {
                            await this._getChannelInfoFromNode(bootstrapNode.remoteNodeId(), channelConfigUrl);
                        }, async (err) => {
                            console.warn(`Problem discovering from bootstrap node ${bootstrapNode.remoteNodeId().slice(0, 6)} (${err.message})`);
                        });
                        /////////////////////////////////////////////////////////////////////////
                    }
                }
                lastBootstrapDiscoverTimestamp = core_1.nowTimestamp();
            }
            const elapsedSinceLastIndividualNodeDiscover = core_1.elapsedSince(lastIndividualNodeDiscoverTimestamp);
            if (elapsedSinceLastIndividualNodeDiscover > core_1.durationMsecToNumber(discoverIndividualNodeIntervalMsec)) {
                // for each channel, choose node with longest elapsed time of their channel node info and get the channel info from that node
                const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
                for (let channelConfigUrl of channelConfigUrls) {
                    let nodes = __classPrivateFieldGet(this, _remoteNodeManager).getRemoteNodesInChannel(channelConfigUrl, { includeOffline: false });
                    if (nodes.length > 0) {
                        var individualNode = selectNode(nodes, channelConfigUrl, __classPrivateFieldGet(this, _discoverHistoryTimestamps));
                        __classPrivateFieldGet(this, _discoverHistoryTimestamps).set(getCode(individualNode.remoteNodeId(), channelConfigUrl), core_1.nowTimestamp());
                        /////////////////////////////////////////////////////////////////////////
                        await action_1.action('discoverFromIndividualNode', { context: 'DiscoverService', remoteNodeId: individualNode.remoteNodeId(), channelConfigUrl }, async () => {
                            await this._getChannelInfoFromNode(individualNode.remoteNodeId(), channelConfigUrl);
                        }, async (err) => {
                            console.warn(`Problem discovering from individual node ${individualNode.remoteNodeId().slice(0, 6)} (${err.message})`);
                        });
                        /////////////////////////////////////////////////////////////////////////
                    }
                }
                lastIndividualNodeDiscoverTimestamp = core_1.nowTimestamp();
            }
            await util_1.sleepMsec(core_1.scaledDurationMsec(500), () => { return !__classPrivateFieldGet(this, _halted); });
        }
    }
}
exports.default = DiscoverService;
_node = new WeakMap(), _remoteNodeManager = new WeakMap(), _halted = new WeakMap(), _discoverHistoryTimestamps = new WeakMap();
// thanks: https://gist.github.com/engelen/fbce4476c9e68c52ff7e5c2da5c24a28
function argMax(array) {
    if (array.length === 0)
        throw Error('Unexpected in argMax');
    return array.map((x, i) => [x, i]).reduce((r, a) => (a[0] > r[0] ? a : r))[1];
}
const selectNode = (nodes, channelConfigUrl, historyTimestamps) => {
    const n = nodes[0];
    if (!n)
        throw Error('Unexpected in selectNode');
    const timestamps = nodes.map(n => (historyTimestamps.getWithDefault(getCode(n.remoteNodeId(), channelConfigUrl), core_1.zeroTimestamp())));
    const elapsedTimes = timestamps.map(ts => (core_1.elapsedSince(ts)));
    const ind = argMax(elapsedTimes);
    return nodes[ind];
};
const getCode = (nodeId, channelConfigUrl) => {
    return nodeId.toString() + ':' + channelConfigUrl.toString();
};
//# sourceMappingURL=DiscoverService.js.map