"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _node, _remoteNodeManager, _halted, _lastText, _lastDisplayTimestamp, _intervalMsec;
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("../common/util");
const createDownloader_1 = require("../downloadOptimizer/createDownloader");
const core_1 = require("../interfaces/core");
class DisplayStateService {
    constructor(node, opts) {
        this.opts = opts;
        _node.set(this, void 0);
        _remoteNodeManager.set(this, void 0);
        _halted.set(this, false);
        _lastText.set(this, '');
        _lastDisplayTimestamp.set(this, core_1.nowTimestamp());
        _intervalMsec.set(this, core_1.unscaledDurationMsec(10000));
        __classPrivateFieldSet(this, _node, node);
        __classPrivateFieldSet(this, _remoteNodeManager, node.remoteNodeManager());
        __classPrivateFieldGet(this, _remoteNodeManager).onBootstrapNodeAdded((bootstrapNodeId) => {
            const rn = __classPrivateFieldGet(this, _remoteNodeManager).getRemoteNode(bootstrapNodeId);
            if (!rn) {
                /* istanbul ignore next */
                throw Error('Bootstrap node added, but unable to find remote node');
            }
            console.info(`Bootstrap node added: ${bootstrapNodeId.slice(0, 6)}`); // to print label
            this._updateDisplay();
        });
        __classPrivateFieldGet(this, _remoteNodeManager).onNodeChannelAdded((remoteNodeId, channelConfigUrl) => {
            console.info(`Node added to channel ${channelConfigUrl}: ${remoteNodeId.slice(0, 6)}`);
            this._updateDisplay();
        });
        this._start();
    }
    stop() {
        __classPrivateFieldSet(this, _halted, true);
    }
    _updateDisplay() {
        const lines = [];
        lines.push('');
        lines.push('=======================================');
        lines.push(`NODE ${__classPrivateFieldGet(this, _node).nodeId().slice(0, 6)} (${__classPrivateFieldGet(this, _node).nodeLabel()})`);
        __classPrivateFieldGet(this, _remoteNodeManager).getBootstrapRemoteNodes({ includeOffline: true }).forEach(rn => {
            const connectionString = getConnectionString(__classPrivateFieldGet(this, _node), rn);
            lines.push(`BOOTSTRAP ${rn.remoteNodeId().slice(0, 6)} ${connectionString} (${rn.remoteNodeLabel() || ''})`);
        });
        __classPrivateFieldGet(this, _remoteNodeManager).getAllRemoteNodes({ includeOffline: true }).forEach((rn) => {
            const connectionString = getConnectionString(__classPrivateFieldGet(this, _node), rn);
            lines.push(`    ${rn.remoteNodeId().slice(0, 6)} ${connectionString} (${rn.remoteNodeLabel() || ''})`);
        });
        if (this.opts.daemonApiPort)
            lines.push(`http://localhost:${this.opts.daemonApiPort}/stats?format=html`);
        lines.push('=======================================');
        const txt = lines.join('\n');
        const elapsed = core_1.unscaledDurationMsec(core_1.elapsedSince(__classPrivateFieldGet(this, _lastDisplayTimestamp)));
        if ((txt !== __classPrivateFieldGet(this, _lastText)) || (core_1.durationGreaterThan(elapsed, core_1.unscaledDurationMsec(30000)))) {
            __classPrivateFieldSet(this, _lastText, txt);
            __classPrivateFieldSet(this, _lastDisplayTimestamp, core_1.nowTimestamp());
            console.info(txt);
            console.info(`Downloaded: ${createDownloader_1.formatByteCount(__classPrivateFieldGet(this, _node).getStats({ format: 'json' }).totalBytesReceived.total)}; Uploaded: ${createDownloader_1.formatByteCount(__classPrivateFieldGet(this, _node).getStats({ format: 'json' }).totalBytesSent.total)};`);
            console.info(`Memory used: ${createDownloader_1.formatByteCount(core_1.byteCount(process.memoryUsage().heapUsed))} (heap); ${createDownloader_1.formatByteCount(core_1.byteCount(process.memoryUsage().external))} (external); ${createDownloader_1.formatByteCount(core_1.byteCount(process.memoryUsage().arrayBuffers))} (arrayBuffers);`);
        }
    }
    async _start() {
        while (true) {
            if (__classPrivateFieldGet(this, _halted))
                return;
            this._updateDisplay();
            await util_1.sleepMsec(__classPrivateFieldGet(this, _intervalMsec), () => { return !__classPrivateFieldGet(this, _halted); });
        }
    }
}
exports.default = DisplayStateService;
_node = new WeakMap(), _remoteNodeManager = new WeakMap(), _halted = new WeakMap(), _lastText = new WeakMap(), _lastDisplayTimestamp = new WeakMap(), _intervalMsec = new WeakMap();
const getConnectionString = (node, rn) => {
    const onlineString = rn.isAuthorizedToCommunicate() ? (rn.isOnline() ? '' : '[offline] ') : '[no-auth] ';
    const candidateMethods = ['udp', 'http', 'http-proxy'];
    const methods = candidateMethods.filter(method => (rn.canSendRequest(method)));
    const channelLabels = [];
    for (let channelConfigUrl of rn.getJoinedChannelConfigUrls()) {
        const c = node.getChannelConfigSync(channelConfigUrl);
        if (c)
            channelLabels.push(c.channelLabel);
    }
    const other = [];
    if (rn.isBootstrap())
        other.push('bootstrap');
    const x = [...methods.map(m => rn.canSendData(m) ? m : m + '-'), ...channelLabels.map(cl => `ch:${cl}`), ...other];
    return `${onlineString}${x.join(' ')}`;
};
const formatMB = (numBytes) => {
    const a = numBytes / (1024 * 1024);
    return `${a.toFixed(1)} MB`;
};
//# sourceMappingURL=DisplayStateService.js.map