"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _node, _halted, _multicastSocket;
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = require("../common/action");
const crypto_util_1 = require("../common/crypto_util");
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const protocolVersion_1 = require("../protocolVersion");
class MulticastService {
    constructor(node, opts) {
        this.opts = opts;
        _node.set(this, void 0);
        _halted.set(this, false);
        _multicastSocket.set(this, null);
        __classPrivateFieldSet(this, _node, node);
        this._start();
    }
    stop() {
        if (__classPrivateFieldGet(this, _multicastSocket)) {
            __classPrivateFieldGet(this, _multicastSocket).close();
            __classPrivateFieldSet(this, _multicastSocket, null);
        }
        __classPrivateFieldSet(this, _halted, true);
    }
    async _start() {
        if (!__classPrivateFieldGet(this, _node).useMulticastUdp())
            return;
        // to find nodes on the local network
        __classPrivateFieldSet(this, _multicastSocket, __classPrivateFieldGet(this, _node).externalInterface().dgramCreateSocket({ type: "udp4", reuseAddr: true, nodeId: __classPrivateFieldGet(this, _node).nodeId(), firewalled: true }));
        const multicastAddress = this.opts.multicastAddress;
        const multicastPort = multicastAddress.port;
        if (!multicastAddress.hostName)
            throw Error('Unexpected multicast address');
        if (!multicastAddress.port)
            throw Error('Unexpected multicast address port');
        __classPrivateFieldGet(this, _multicastSocket).bind(core_1.portToNumber(multicastAddress.port));
        __classPrivateFieldGet(this, _multicastSocket).on('listening', () => {
            if (!multicastAddress.hostName)
                throw Error('Unexpected multicast address');
            if (__classPrivateFieldGet(this, _multicastSocket)) {
                __classPrivateFieldGet(this, _multicastSocket).addMembership(multicastAddress.hostName.toString());
            }
        });
        __classPrivateFieldGet(this, _multicastSocket).on('message', (message, rinfo) => {
            let msg = core_1.tryParseJsonObject(message.toString());
            if (core_1.isMulticastAnnounceMessage(msg)) {
                __classPrivateFieldGet(this, _node).stats().reportBytesReceived('multicastUdp', msg.body.fromNodeId, core_1.byteCount(message.length));
                const msg2 = msg;
                /////////////////////////////////////////////////////////////////////////
                action_1.action('handleMulticastAnnounceMessage', { fromNodeId: msg.body.fromNodeId }, async () => {
                    if (crypto_util_1.verifySignature(msg2.body, msg2.signature, core_1.nodeIdToPublicKey(msg2.body.fromNodeId), { checkTimestamp: true })) {
                        if (msg2.body.fromNodeId === __classPrivateFieldGet(this, _node).nodeId())
                            return;
                        const localUdpAddress = msg2.body.udpSocketPort ? ({
                            hostName: core_1.hostName(rinfo.address),
                            port: msg2.body.udpSocketPort
                        }) : null;
                        const response = __classPrivateFieldGet(this, _node).remoteNodeManager().handleAnnounceRequest({ fromNodeId: msg2.body.fromNodeId, requestData: msg2.body.requestData, localUdpAddress });
                        // don't do anything with response here
                    }
                    else {
                        throw Error('Problem verifying signature for multicast announce message');
                    }
                }, async (err) => {
                    //
                });
                /////////////////////////////////////////////////////////////////////////
            }
        });
        await util_1.sleepMsec(core_1.minDuration(core_1.scaledDurationMsec(1000), this.opts.intervalMsec));
        while (true) {
            if (__classPrivateFieldGet(this, _halted))
                return;
            const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
            for (let channelConfigUrl of channelConfigUrls) {
                const requestData = {
                    requestType: 'announce',
                    channelNodeInfo: await __classPrivateFieldGet(this, _node).getChannelNodeInfo(channelConfigUrl)
                };
                const body = {
                    protocolVersion: protocolVersion_1.protocolVersion(),
                    fromNodeId: __classPrivateFieldGet(this, _node).nodeId(),
                    messageType: 'announce',
                    requestData,
                    udpSocketPort: __classPrivateFieldGet(this, _node).udpSocketPort(),
                    timestamp: core_1.nowTimestamp()
                };
                const m = {
                    body,
                    signature: crypto_util_1.getSignature(body, __classPrivateFieldGet(this, _node).keyPair())
                };
                const mJson = JSON.stringify(m);
                /////////////////////////////////////////////////////////////////////////
                await action_1.action('sendMulticastAnnounceMessage', {}, async () => {
                    if (__classPrivateFieldGet(this, _multicastSocket)) {
                        __classPrivateFieldGet(this, _node).stats().reportBytesSent('multicastUdp', null, core_1.byteCount(mJson.length));
                        if (!multicastAddress.hostName)
                            throw Error('Unexpected multicast address');
                        if (!multicastAddress.port)
                            throw Error('Unexpected multicast address port');
                        __classPrivateFieldGet(this, _multicastSocket).send(Buffer.from(mJson), 0, mJson.length, core_1.portToNumber(multicastAddress.port), multicastAddress.hostName.toString());
                    }
                    else {
                        throw Error('No multicast socket');
                    }
                }, async (err) => {
                    console.warn(`Problem sending multicast announce message (${err.message})`);
                });
                /////////////////////////////////////////////////////////////////////////
            }
            await util_1.sleepMsec(this.opts.intervalMsec, () => { return !__classPrivateFieldGet(this, _halted); });
        }
    }
}
exports.default = MulticastService;
_node = new WeakMap(), _halted = new WeakMap(), _multicastSocket = new WeakMap();
//# sourceMappingURL=MulticastService.js.map