"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _remoteNodeManager, _proxyClientManager, _halted, _node_1, _failedConnectionAttemptTimestamps;
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = require("../common/action");
const constants_1 = require("../common/constants");
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const ProxyWebsocketConnection_1 = require("../proxyConnections/ProxyWebsocketConnection");
class ProxyClientService {
    constructor(node, opts) {
        this.opts = opts;
        _node.set(this, void 0);
        _remoteNodeManager.set(this, void 0);
        _proxyClientManager.set(this, void 0);
        _halted.set(this, false);
        __classPrivateFieldSet(this, _node, node);
        __classPrivateFieldSet(this, _remoteNodeManager, node.remoteNodeManager());
        __classPrivateFieldSet(this, _proxyClientManager, new ProxyClientManager(__classPrivateFieldGet(this, _node))
        // this.#remoteNodeManager.onBootstrapNodeAdded((bootstrapNodeId) => {
        //     const rn = this.#remoteNodeManager.getRemoteNode(bootstrapNodeId)
        //     if ((rn) && (rn.getRemoteNodeWebSocketAddress())) {
        //         /////////////////////////////////////////////////////////////////////////
        //         action('tryOutgoingProxyConnectionForNewBootstrapNode', {context: 'ProxyClientService', bootstrapNodeId}, async () => {
        //             await this.#proxyClientManager.tryConnection(bootstrapNodeId, {timeoutMsec: TIMEOUTS.websocketConnect});
        //         }, null)
        //         /////////////////////////////////////////////////////////////////////////
        //     }
        // })
        );
        // this.#remoteNodeManager.onBootstrapNodeAdded((bootstrapNodeId) => {
        //     const rn = this.#remoteNodeManager.getRemoteNode(bootstrapNodeId)
        //     if ((rn) && (rn.getRemoteNodeWebSocketAddress())) {
        //         /////////////////////////////////////////////////////////////////////////
        //         action('tryOutgoingProxyConnectionForNewBootstrapNode', {context: 'ProxyClientService', bootstrapNodeId}, async () => {
        //             await this.#proxyClientManager.tryConnection(bootstrapNodeId, {timeoutMsec: TIMEOUTS.websocketConnect});
        //         }, null)
        //         /////////////////////////////////////////////////////////////////////////
        //     }
        // })
        this._start();
    }
    stop() {
        __classPrivateFieldSet(this, _halted, true);
    }
    async _start() {
        // periodically try to establish proxy connections to the remote nodes
        while (true) {
            if (__classPrivateFieldGet(this, _halted))
                return;
            const channelConfigUrls = __classPrivateFieldGet(this, _node).joinedChannelConfigUrls();
            const messageProxyNodeIds = new Set();
            const dataProxyNodeIds = new Set();
            for (const channelConfigUrl of channelConfigUrls) {
                const channelConfig = await __classPrivateFieldGet(this, _node).getChannelConfig(channelConfigUrl);
                if (channelConfig) {
                    for (const nn of channelConfig.authorizedNodes) {
                        if (nn.isMessageProxy)
                            messageProxyNodeIds.add(nn.nodeId);
                        if (nn.isDataProxy)
                            dataProxyNodeIds.add(nn.nodeId);
                    }
                }
            }
            const remoteNodes = __classPrivateFieldGet(this, _remoteNodeManager).getAllRemoteNodes({ includeOffline: false });
            for (let remoteNode of remoteNodes) {
                const isMessageProxy = messageProxyNodeIds.has(remoteNode.remoteNodeId());
                const isDataProxy = dataProxyNodeIds.has(remoteNode.remoteNodeId());
                if ((isMessageProxy) || (isDataProxy)) {
                    if ((remoteNode.isOnline()) && (remoteNode.getRemoteNodeWebSocketAddress())) {
                        const remoteNodeId = remoteNode.remoteNodeId();
                        const c = __classPrivateFieldGet(this, _node).getProxyConnectionToServer(remoteNodeId);
                        if (!c) {
                            const elapsedMsec = __classPrivateFieldGet(this, _proxyClientManager).elapsedMsecSinceLastFailedOutgoingConnection(remoteNodeId);
                            if (core_1.durationGreaterThan(elapsedMsec, core_1.scaledDurationMsec(15000))) {
                                ///////////////////////////////////////////////////////////////////////
                                await action_1.action('tryOutgoingProxyConnection', { context: 'ProxyClientService', remoteNodeId }, async () => {
                                    await __classPrivateFieldGet(this, _proxyClientManager).tryConnection(remoteNodeId, { timeoutMsec: constants_1.TIMEOUTS.websocketConnect });
                                }, async (err) => {
                                    console.warn(`Problem establishing outgoing proxy connection to ${remoteNode.remoteNodeId().slice(0, 6)} (${err.message})`);
                                });
                                ///////////////////////////////////////////////////////////////////////
                            }
                        }
                    }
                }
            }
            await util_1.sleepMsec(this.opts.intervalMsec, () => { return !__classPrivateFieldGet(this, _halted); });
        }
    }
}
exports.default = ProxyClientService;
_node = new WeakMap(), _remoteNodeManager = new WeakMap(), _proxyClientManager = new WeakMap(), _halted = new WeakMap();
class ProxyClientManager {
    constructor(node) {
        _node_1.set(this, void 0);
        // #outgoingConnections = new Map<NodeId, ProxyConnectionToServer>()
        _failedConnectionAttemptTimestamps.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(120 * 1000)));
        __classPrivateFieldSet(this, _node_1, node);
    }
    async tryConnection(remoteNodeId, opts) {
        const remoteNode = __classPrivateFieldGet(this, _node_1).remoteNodeManager().getRemoteNode(remoteNodeId);
        if (!remoteNode)
            return;
        const webSocketAddress = remoteNode.getRemoteNodeWebSocketAddress();
        if (!webSocketAddress) {
            return;
        }
        try {
            const c = new ProxyWebsocketConnection_1.ProxyWebsocketConnection(__classPrivateFieldGet(this, _node_1), { connectionType: 'connectionToServer' });
            await c.initializeConnectionToServer(remoteNodeId, webSocketAddress, { timeoutMsec: opts.timeoutMsec });
            __classPrivateFieldGet(this, _node_1).setProxyConnectionToServer(remoteNodeId, c);
        }
        catch (err) {
            __classPrivateFieldGet(this, _failedConnectionAttemptTimestamps).set(remoteNodeId, core_1.nowTimestamp());
            throw (err);
        }
    }
    elapsedMsecSinceLastFailedOutgoingConnection(remoteNodeId) {
        const timestamp = __classPrivateFieldGet(this, _failedConnectionAttemptTimestamps).get(remoteNodeId) || core_1.zeroTimestamp();
        return core_1.unscaledDurationMsec(core_1.elapsedSince(timestamp));
    }
    getConnection(remoteNodeId) {
        return __classPrivateFieldGet(this, _node_1).getProxyConnectionToServer(remoteNodeId);
    }
}
_node_1 = new WeakMap(), _failedConnectionAttemptTimestamps = new WeakMap();
//# sourceMappingURL=ProxyClientService.js.map