"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _verbose, _app, _server;
Object.defineProperty(exports, "__esModule", { value: true });
exports.isPublicApiProbeResponse = void 0;
const express_1 = __importDefault(require("express"));
const action_1 = require("../common/action");
const crypto_util_1 = require("../common/crypto_util");
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
const protocolVersion_1 = require("../protocolVersion");
;
const isPublicApiProbeResponse = (x) => {
    return core_1._validateObject(x, {
        success: core_1.isBoolean,
        protocolVersion: core_1.isEqualTo(protocolVersion_1.protocolVersion()),
        daemonVersion: core_1.isDaemonVersion,
        nodeId: core_1.isNodeId,
        isBootstrapNode: core_1.isBoolean,
        webSocketAddress: core_1.isOneOf([core_1.isNull, core_1.isAddress]),
        publicUdpSocketAddress: core_1.isOneOf([core_1.isNull, core_1.isAddress])
    });
};
exports.isPublicApiProbeResponse = isPublicApiProbeResponse;
class PublicApiServer {
    // This is the public API server for communication between nodes
    constructor(node, opts = { verbose: 0 }) {
        _node.set(this, void 0);
        _verbose.set(this, void 0);
        _app.set(this, void 0);
        // #server: http.Server | https.Server | null = null
        _server.set(this, null
        // This is the public API server for communication between nodes
        );
        __classPrivateFieldSet(this, _node, node); // The kachery-p2p daemon
        __classPrivateFieldSet(// The kachery-p2p daemon
        this, _verbose, opts.verbose);
        __classPrivateFieldSet(this, _app, express_1.default()); // the express app
        __classPrivateFieldGet(this, _app).set('json spaces', 4); // when we respond with json, this is how it will be formatted
        // this.#app.use(cors()); // in the future, if we want to do this
        __classPrivateFieldGet(this, _app).use(express_1.default.json());
        __classPrivateFieldGet(this, _app).all('/*', (req, res, next) => {
            next();
        });
        // /probe - check whether the daemon is up and running and return info such as the node ID
        __classPrivateFieldGet(this, _app).get('/probe', async (req, res) => {
            /////////////////////////////////////////////////////////////////////////
            await action_1.action('/probe', { context: 'Public API' }, async () => {
                await this._apiProbe(req, res);
            }, async (err) => {
                await this._errorResponse(req, res, 500, err.message);
            });
            /////////////////////////////////////////////////////////////////////////
        });
        __classPrivateFieldGet(this, _app).post('/probe', async (req, res) => {
            /////////////////////////////////////////////////////////////////////////
            await action_1.action('/probe', { context: 'Public API' }, async () => {
                await this._apiProbe(req, res);
            }, async (err) => {
                await this._errorResponse(req, res, 500, err.message);
            });
            /////////////////////////////////////////////////////////////////////////
        });
        // /nodeToNodeRequest
        __classPrivateFieldGet(this, _app).post('/nodeToNodeRequest', async (req, res) => {
            const reqBody = req.body;
            if (!NodeToNodeRequest_1.isNodeToNodeRequest(reqBody)) {
                if ((reqBody.body) && (reqBody.body.protocolVersion === protocolVersion_1.protocolVersion())) {
                    await this._errorResponse(req, res, 500, 'Invalid node-to-node request');
                }
                return;
            }
            __classPrivateFieldGet(this, _node).stats().reportBytesReceived('http', reqBody.body.fromNodeId, core_1.byteCount(crypto_util_1.JSONStringifyDeterministic(reqBody).length));
            /////////////////////////////////////////////////////////////////////////
            await action_1.action('/nodeToNodeRequest', {
                context: 'Public API',
                fromNodeId: reqBody.body.fromNodeId,
                requestType: reqBody.body.requestData.requestType
            }, async () => {
                await this._apiNodeToNodeRequest(reqBody, res);
            }, async (err) => {
                await this._errorResponse(req, res, 500, err.message);
            });
            /////////////////////////////////////////////////////////////////////////
        });
        // /download
        __classPrivateFieldGet(this, _app).get('/download/:fromNodeId/:toNodeId/:streamId', async (req, res) => {
            const streamId = req.params.streamId;
            const fromNodeId = req.params.fromNodeId;
            const toNodeId = req.params.toNodeId;
            /////////////////////////////////////////////////////////////////////////
            await action_1.action('/download', {
                context: 'Public API',
                fromNodeId,
                toNodeId,
                streamId
            }, async () => {
                if (!NodeToNodeRequest_1.isStreamId(streamId)) {
                    /* istanbul ignore next */
                    throw Error('Invalid stream ID');
                }
                if (!core_1.isNodeId(fromNodeId)) {
                    /* istanbul ignore next */
                    throw Error('Invalid fromNodeId');
                }
                if (!core_1.isNodeId(toNodeId)) {
                    /* istanbul ignore next */
                    throw Error('Invalid toNodeId');
                }
                await this._apiDownload(fromNodeId, toNodeId, streamId, req, res);
            }, async (err) => {
                /* istanbul ignore next */
                await this._errorResponse(req, res, 500, err.message);
            });
            /////////////////////////////////////////////////////////////////////////
            req.params.streamId;
        });
    }
    stop() {
        if (__classPrivateFieldGet(this, _server)) {
            __classPrivateFieldGet(this, _server).close();
        }
    }
    async mockPostJson(path, data) {
        if (path === '/probe') {
            return await this._probe();
        }
        else if (path === '/NodeToNodeRequest') {
            if (!NodeToNodeRequest_1.isNodeToNodeRequest(data)) {
                /* istanbul ignore next */
                console.warn(JSON.stringify(data, null, 4));
                /* istanbul ignore next */
                throw Error('Invalid data for node to node request');
            }
            const ret = await this._nodeToNodeRequest(data);
            return ret;
        }
        else {
            /* istanbul ignore next */
            throw Error(`mock unexpected path: ${path}`);
        }
    }
    async mockGetDownload(path) {
        if (path.startsWith('/download/')) {
            const vals = path.split('/');
            const fromNodeId = vals[2];
            const toNodeId = vals[3];
            const streamId = vals[4];
            if (!core_1.isNodeId(fromNodeId)) {
                /* istanbul ignore next */
                throw Error('Invalid from node id in mock /download');
            }
            if (!core_1.isNodeId(toNodeId)) {
                /* istanbul ignore next */
                throw Error('Invalid to node id in mock /download');
            }
            if (!NodeToNodeRequest_1.isStreamId(streamId)) {
                /* istanbul ignore next */
                throw Error('Invalid stream id in mock /download');
            }
            const ds = await __classPrivateFieldGet(this, _node).streamDataForStreamId(fromNodeId, streamId);
            ds.onData(d => {
                __classPrivateFieldGet(this, _node).stats().reportBytesSent('http', fromNodeId, core_1.byteCount(d.length));
            });
            return ds;
        }
        else {
            /* istanbul ignore next */
            throw Error(`mock get download unexpected path: ${path}`);
        }
    }
    async _probe() {
        const response = {
            success: true,
            protocolVersion: protocolVersion_1.protocolVersion(),
            daemonVersion: protocolVersion_1.daemonVersion(),
            nodeId: __classPrivateFieldGet(this, _node).nodeId(),
            isBootstrapNode: __classPrivateFieldGet(this, _node).isBootstrapNode(),
            webSocketAddress: __classPrivateFieldGet(this, _node).webSocketAddress(),
            publicUdpSocketAddress: __classPrivateFieldGet(this, _node).publicUdpSocketAddress()
        };
        if (!core_1.isJSONObject(response))
            throw Error('Unexpected, not a JSON-serializable object');
        return response;
    }
    async _nodeToNodeRequest(reqBody) {
        const response = await __classPrivateFieldGet(this, _node).handleNodeToNodeRequest(reqBody);
        if (!core_1.isJSONObject(response))
            throw Error('Unexpected, not a JSON-serializable object');
        return response;
    }
    // /probe - check whether the daemon is up and running and return info such as the node ID
    async _apiProbe(req, res) {
        const response = await this._probe();
        res.json(response);
    }
    // /nodeToNodeRequest
    async _apiNodeToNodeRequest(reqBody, res) {
        const response = await this._nodeToNodeRequest(reqBody);
        res.json(response);
    }
    // /download
    async _apiDownload(fromNodeId, toNodeId, streamId, req, res) {
        const ds = await __classPrivateFieldGet(this, _node).streamDataForStreamId(fromNodeId, streamId);
        let started = false;
        ds.onStarted((size) => {
            started = true;
            res.writeHead(200, {
                'Content-Type': 'application/octet-stream',
                'Content-Length': size
            });
        });
        ds.onData((data) => {
            __classPrivateFieldGet(this, _node).stats().reportBytesSent('http', toNodeId, core_1.byteCount(data.length));
            res.write(data);
        });
        ds.onFinished(() => {
            res.end();
        });
        ds.onError((err) => {
            if (started) {
                console.warn(err);
                console.warn('Error in streaming file data');
                res.end();
            }
            else {
                res.status(500).send(err.message);
            }
        });
        req.on('close', () => {
            ds.cancel();
        });
    }
    // Helper function for returning http request with an error response
    async _errorResponse(req, res, code, errorString) {
        console.info(`Public responding with error: ${code} ${errorString}`);
        try {
            res.status(code).send(errorString);
        }
        catch (err) {
            console.warn(`Problem sending error`, { error: err.message });
        }
        await util_1.sleepMsec(core_1.scaledDurationMsec(100));
        try {
            req.connection.destroy();
        }
        catch (err) {
            console.warn('Problem destroying connection', { error: err.message });
        }
    }
    // Start listening via http/https
    async listen(port) {
        __classPrivateFieldSet(this, _server, await __classPrivateFieldGet(this, _node).externalInterface().startHttpServer(__classPrivateFieldGet(this, _app), port));
    }
}
exports.default = PublicApiServer;
_node = new WeakMap(), _verbose = new WeakMap(), _app = new WeakMap(), _server = new WeakMap();
//# sourceMappingURL=PublicApiServer.js.map