"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _node, _messagePartManager, _socket, _udpPacketSender, _udpPacketReceiver, _responseListeners, _incomingDataStreams, _receivedUdpPackets, _fallbackPacketSender, _stopped;
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodeIdFallbackAddress = exports.isNodeIdFallbackAddress = void 0;
const action_1 = require("../common/action");
const constants_1 = require("../common/constants");
const crypto_util_1 = require("../common/crypto_util");
const DataStreamy_1 = __importDefault(require("../common/DataStreamy"));
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const NodeToNodeRequest_1 = require("../interfaces/NodeToNodeRequest");
const UdpMessage_1 = require("../interfaces/UdpMessage");
const protocolVersion_1 = require("../protocolVersion");
const UdpMessagePartManager_1 = __importDefault(require("../udp/UdpMessagePartManager"));
const UdpPacketReceiver_1 = __importDefault(require("../udp/UdpPacketReceiver"));
const UdpPacketSender_1 = __importDefault(require("../udp/UdpPacketSender"));
const isStreamDataChunkMetaData = (x) => {
    return core_1._validateObject(x, {
        streamId: NodeToNodeRequest_1.isStreamId,
        dataChunkIndex: core_1.isNumber
    });
};
const isNodeIdFallbackAddress = (x) => {
    return core_1._validateObject(x, {
        nodeId: core_1.isNodeId
    });
};
exports.isNodeIdFallbackAddress = isNodeIdFallbackAddress;
const nodeIdFallbackAddress = (nodeId) => {
    return { nodeId };
};
exports.nodeIdFallbackAddress = nodeIdFallbackAddress;
const isStreamDataErrorMetaData = (x) => {
    return core_1._validateObject(x, {
        streamId: NodeToNodeRequest_1.isStreamId,
        errorMessage: core_1.isErrorMessage
    });
};
const isStreamDataEndMetaData = (x) => {
    return core_1._validateObject(x, {
        streamId: NodeToNodeRequest_1.isStreamId,
        numDataChunks: core_1.isNumber
    });
};
class PublicUdpSocketServer {
    constructor(node, firewalled) {
        this.firewalled = firewalled;
        _node.set(this, void 0);
        _messagePartManager.set(this, new UdpMessagePartManager_1.default());
        _socket.set(this, null);
        _udpPacketSender.set(this, null);
        _udpPacketReceiver.set(this, null);
        _responseListeners.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
        _incomingDataStreams.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(60 * 60 * 1000)));
        _receivedUdpPackets.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
        _fallbackPacketSender.set(this, void 0);
        _stopped.set(this, false);
        __classPrivateFieldSet(this, _node, node);
        __classPrivateFieldGet(this, _messagePartManager).onMessageComplete((header, buffer) => { this._handleCompleteMessage(header, buffer); });
        __classPrivateFieldSet(this, _fallbackPacketSender, new FallbackPacketSender(__classPrivateFieldGet(this, _node)));
    }
    stop() {
        __classPrivateFieldSet(this, _stopped, true);
        if (__classPrivateFieldGet(this, _socket)) {
            __classPrivateFieldGet(this, _socket).close();
        }
    }
    startListening(listenPort) {
        return new Promise((resolve, reject) => {
            try {
                __classPrivateFieldSet(this, _socket, __classPrivateFieldGet(this, _node).externalInterface().dgramCreateSocket({ type: "udp4", reuseAddr: false, nodeId: __classPrivateFieldGet(this, _node).nodeId(), firewalled: this.firewalled }));
                __classPrivateFieldGet(this, _socket).bind(core_1.portToNumber(listenPort));
                __classPrivateFieldGet(this, _socket).on("listening", () => {
                    /* istanbul ignore next */
                    if (__classPrivateFieldGet(this, _socket) === null)
                        throw Error('Unexpected null socket in startListening');
                    __classPrivateFieldSet(this, _udpPacketSender, new UdpPacketSender_1.default(__classPrivateFieldGet(this, _socket), __classPrivateFieldGet(this, _fallbackPacketSender), __classPrivateFieldGet(this, _node).stats(), { thisNodeId: __classPrivateFieldGet(this, _node).nodeId() }));
                    __classPrivateFieldSet(this, _udpPacketReceiver, new UdpPacketReceiver_1.default(__classPrivateFieldGet(this, _socket), () => (__classPrivateFieldGet(this, _node).getDefects()), __classPrivateFieldGet(this, _node).stats(), { thisNodeId: __classPrivateFieldGet(this, _node).nodeId() }));
                    __classPrivateFieldGet(this, _udpPacketReceiver).onPacket((packetId, packet, remoteInfo) => {
                        this._receiveUdpPacket(packetId, packet, remoteInfo, null);
                    });
                    __classPrivateFieldGet(this, _udpPacketReceiver).onConfirmation((packetId) => {
                        /////////////////////////////////////////////////////////////////////////
                        action_1.action('confirmUdpPacket', { packetId }, async () => {
                            /* istanbul ignore next */
                            if (__classPrivateFieldGet(this, _udpPacketSender) === null)
                                throw Error('Unexpected null udpPacketSender in startListening');
                            __classPrivateFieldGet(this, _udpPacketSender).receivePacketConfirmation(packetId);
                        }, async () => {
                        });
                        /////////////////////////////////////////////////////////////////////////
                    });
                    resolve();
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    async sendRequest(address, request, opts) {
        const fallbackAddress = exports.nodeIdFallbackAddress(request.body.toNodeId);
        return new Promise((resolve, reject) => {
            let complete = false;
            const _handleError = ((err) => {
                if (complete)
                    return;
                complete = true;
                if (__classPrivateFieldGet(this, _responseListeners).has(request.body.requestId)) {
                    __classPrivateFieldGet(this, _responseListeners).delete(request.body.requestId);
                }
                reject(err);
            });
            const _handleFinished = ((response, header) => {
                if (complete)
                    return;
                complete = true;
                if (__classPrivateFieldGet(this, _responseListeners).has(request.body.requestId)) {
                    __classPrivateFieldGet(this, _responseListeners).delete(request.body.requestId);
                }
                resolve({ response, header });
            });
            __classPrivateFieldGet(this, _responseListeners).set(request.body.requestId, {
                onResponse: (response, header) => {
                    _handleFinished(response, header);
                }
            });
            const timestamp = core_1.nowTimestamp();
            const checkTimeout = () => {
                if (complete)
                    return;
                const elapsed = core_1.elapsedSince(timestamp);
                if (elapsed > core_1.durationMsecToNumber(opts.timeoutMsec)) {
                    _handleError(new util_1.RequestTimeoutError(`Timeout waiting for response for request: ${request.body.requestData.requestType} (${elapsed} > ${opts.timeoutMsec} msec)`));
                }
                else {
                    setTimeout(() => {
                        checkTimeout();
                    }, core_1.durationMsecToNumber(core_1.scaledDurationMsec(500)));
                }
            };
            checkTimeout();
            this._sendMessage(address, fallbackAddress, "NodeToNodeRequest", request, UdpMessage_1.udpMessageMetaData({}), { timeoutMsec: opts.timeoutMsec, toNodeId: request.body.toNodeId }).catch(err => {
                _handleError(err);
            });
        });
    }
    getIncomingDataStream(streamId) {
        if (__classPrivateFieldGet(this, _incomingDataStreams).has(streamId)) {
            /* istanbul ignore next */
            throw Error('Cannot get incoming data stream twice');
        }
        const ds = new DataStreamy_1.default();
        __classPrivateFieldGet(this, _incomingDataStreams).set(streamId, ds);
        ds.producer().onCancelled(() => {
            this._cancelIncomingDataStream(streamId);
        });
        return ds;
    }
    setOutgoingDataStream(address, fallbackAddress, streamId, dataStream, opts) {
        let dataChunkIndex = 0;
        dataStream.onFinished(() => {
            const metaData = {
                streamId,
                numDataChunks: dataChunkIndex
            };
            this._sendMessage(address, fallbackAddress, 'streamDataEnd', Buffer.alloc(0), UdpMessage_1.udpMessageMetaData(metaData), { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, toNodeId: opts.toNodeId }).catch((err) => {
                // todo
            });
        });
        dataStream.onError((err) => {
            const metaData = {
                streamId,
                errorMessage: core_1.errorMessage(err.message)
            };
            this._sendMessage(address, fallbackAddress, 'streamDataError', Buffer.alloc(0), UdpMessage_1.udpMessageMetaData(metaData), { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, toNodeId: opts.toNodeId }).catch((err) => {
                // todo
            });
        });
        dataStream.onData((dataChunk) => {
            const metaData = {
                streamId,
                dataChunkIndex
            };
            this._sendMessage(address, fallbackAddress, 'streamDataChunk', dataChunk, UdpMessage_1.udpMessageMetaData(metaData), { timeoutMsec: constants_1.TIMEOUTS.defaultRequest, toNodeId: opts.toNodeId }).catch((err) => {
                // todo
            });
            dataChunkIndex++;
        });
    }
    receiveFallbackUdpPacket(fromNodeId, packetId, packet) {
        this._receiveUdpPacket(packetId, packet, null, fromNodeId);
    }
    _cancelIncomingDataStream(streamId) {
        const ds = __classPrivateFieldGet(this, _incomingDataStreams).get(streamId);
        if (!ds)
            return;
    }
    _receiveUdpPacket(packetId, packet, remoteInfo, checkFromNodeId) {
        if (__classPrivateFieldGet(this, _receivedUdpPackets).has(packetId)) {
            return;
        }
        __classPrivateFieldGet(this, _receivedUdpPackets).set(packetId, true);
        const fromAddress = remoteInfo ? {
            port: core_1.toPort(remoteInfo.port),
            hostName: core_1.hostName(remoteInfo.address)
        } : null;
        const headerTxt = packet.slice(0, UdpMessage_1.UDP_MESSAGE_HEADER_SIZE).toString().trimEnd();
        const dataBuffer = packet.slice(UdpMessage_1.UDP_MESSAGE_HEADER_SIZE);
        const header = core_1.tryParseJsonObject(headerTxt);
        if (header === null) {
            return;
        }
        if (!UdpMessage_1.isUdpHeader(header)) {
            /* istanbul ignore next */
            console.warn(header);
            /* istanbul ignore next */
            console.warn('Problem with udp header');
            return;
        }
        if (checkFromNodeId) {
            if (checkFromNodeId !== header.body.fromNodeId) {
                /* istanbul ignore next */
                console.warn('fromNodeId does not match in udp packet');
                /* istanbul ignore next */
                return;
            }
        }
        if (remoteInfo !== null) {
            __classPrivateFieldGet(this, _node).stats().reportBytesReceived('udp', header.body.fromNodeId, core_1.byteCount(packet.length));
        }
        /////////////////////////////////////////////////////////////////////////
        action_1.action('handleUdpMessagePart', { fromAddress, fromNodeId: header.body.fromNodeId, udpMessageType: header.body.udpMessageType }, async () => {
            this._handleMessagePart(fromAddress, header, dataBuffer);
        }, async () => {
        });
        /////////////////////////////////////////////////////////////////////////
    }
    async _sendMessage(address, fallbackAddress, messageType, messageData, metaData, opts) {
        if ((__classPrivateFieldGet(this, _socket) === null) || (__classPrivateFieldGet(this, _udpPacketSender) === null)) {
            /* istanbul ignore next */
            throw Error("Cannot _sendMessage before calling startListening()");
        }
        let payloadIsJson;
        let messageBuffer;
        if (Buffer.isBuffer(messageData)) {
            payloadIsJson = false;
            messageBuffer = messageData;
        }
        else {
            payloadIsJson = true;
            messageBuffer = Buffer.from(JSON.stringify(messageData));
        }
        const parts = this._createUdpMessageParts(messageType, address, messageBuffer, metaData, { payloadIsJson });
        const packets = [];
        for (let part of parts) {
            const b = Buffer.concat([
                Buffer.from(JSON.stringify(part.header).padEnd(UdpMessage_1.UDP_MESSAGE_HEADER_SIZE, ' ')),
                part.dataBuffer
            ]);
            packets.push(b);
        }
        try {
            await __classPrivateFieldGet(this, _udpPacketSender).sendPackets(address, fallbackAddress, packets, { timeoutMsec: opts.timeoutMsec, toNodeId: opts.toNodeId });
        }
        catch (err) {
            if (__classPrivateFieldGet(this, _stopped)) {
                // do not throw the error if stopped
                return;
            }
            throw (err);
        }
    }
    _handleMessagePart(remoteAddress, header, dataBuffer) {
        if (!crypto_util_1.verifySignature(header.body, header.signature, core_1.nodeIdToPublicKey(header.body.fromNodeId))) {
            /* istanbul ignore next */
            throw Error('Error verifying signature in udp message');
        }
        __classPrivateFieldGet(this, _messagePartManager).addMessagePart(remoteAddress, header.body.udpMessageId, header.body.partIndex, header.body.numParts, header, dataBuffer);
    }
    _handleCompleteMessage(header, dataBuffer) {
        const mt = header.body.udpMessageType;
        if (mt === "NodeToNodeRequest") {
            const req = core_1.tryParseJsonObject(dataBuffer.toString());
            if (!NodeToNodeRequest_1.isNodeToNodeRequest(req)) {
                // throw error? ban peer?
                /* istanbul ignore next */
                return;
            }
            /////////////////////////////////////////////////////////////////////////
            action_1.action('/Udp/handleNodeToNodeRequest', { fromNodeId: req.body.fromNodeId, requestId: req.body.requestId, requestType: req.body.requestData.requestType }, async () => {
                const response = await __classPrivateFieldGet(this, _node).handleNodeToNodeRequest(req);
                const fallbackAddress = exports.nodeIdFallbackAddress(req.body.fromNodeId);
                const remoteAddress = __classPrivateFieldGet(this, _messagePartManager).getRemoteAddressForNodeId(req.body.fromNodeId);
                await this._sendMessage(remoteAddress, fallbackAddress, "NodeToNodeResponse", response, UdpMessage_1.udpMessageMetaData({}), { timeoutMsec: constants_1.TIMEOUTS.defaultResponse, toNodeId: req.body.fromNodeId });
            }, async () => {
            });
            /////////////////////////////////////////////////////////////////////////
        }
        else if (mt === "NodeToNodeResponse") {
            const res = core_1.tryParseJsonObject(dataBuffer.toString());
            if (!NodeToNodeRequest_1.isNodeToNodeResponse(res)) {
                // throw error? ban peer?
                /* istanbul ignore next */
                return;
            }
            /////////////////////////////////////////////////////////////////////////
            action_1.action('/Udp/handleNodeToNodeResponse', { fromNodeId: res.body.fromNodeId, requestId: res.body.requestId, requestType: res.body.responseData.requestType }, async () => {
                const responseListener = __classPrivateFieldGet(this, _responseListeners).get(res.body.requestId);
                if (responseListener) {
                    responseListener.onResponse(res, header);
                    // response listener will be delete that way we will not handle the same response twice
                }
                else {
                    // Do we need to warn here? Can this reasonably happen - like if we get two responses (maybe the first was lost?)
                    console.warn(`No response listener found for response: ${res.body.requestId} ${res.body.responseData.requestType}`);
                }
            }, async () => {
            });
            /////////////////////////////////////////////////////////////////////////
        }
        else if (mt === "KeepAlive") {
            // todo
        }
        else if (mt === 'streamDataChunk') {
            const metaData = header.body.metaData;
            if (!isStreamDataChunkMetaData(metaData)) {
                // do we want to throw error here? ban node?
                /* istanbul ignore next */
                throw Error('Invalid meta data for stream data chunk message');
            }
            const ds = __classPrivateFieldGet(this, _incomingDataStreams).get(metaData.streamId);
            if (!ds) {
                // do we want to throw error here?
                /* istanbul ignore next */
                throw Error('No incoming data stream found for data chunk message');
            }
            ds.producer().unorderedData(metaData.dataChunkIndex, dataBuffer);
        }
        else if (mt === 'streamDataError') {
            const metaData = header.body.metaData;
            if (!isStreamDataErrorMetaData(metaData)) {
                // do we want to throw error here? ban node?
                throw Error('Invalid meta data for stream data error message');
            }
            const ds = __classPrivateFieldGet(this, _incomingDataStreams).get(metaData.streamId);
            if (!ds) {
                // do we want to throw error here?
                /* istanbul ignore next */
                throw Error('No incoming data stream found for data chunk error');
            }
            ds.producer().error(Error(metaData.errorMessage.toString()));
        }
        else if (mt === 'streamDataEnd') {
            const metaData = header.body.metaData;
            if (!isStreamDataEndMetaData(metaData)) {
                // do we want to throw error here? ban node?
                /* istanbul ignore next */
                throw Error('Invalid meta data for stream data end message');
            }
            const ds = __classPrivateFieldGet(this, _incomingDataStreams).get(metaData.streamId);
            if (!ds) {
                // do we want to throw error here?
                /* istanbul ignore next */
                throw Error('No incoming data stream found for data end error');
            }
            ds.producer().unorderedEnd(metaData.numDataChunks);
        }
    }
    _createUdpMessageParts(udpMessageType, toAddress, messageData, metaData, opts) {
        const parts = [];
        const partSize = UdpMessage_1.UDP_PACKET_SIZE - UdpMessage_1.UDP_MESSAGE_HEADER_SIZE;
        const buffers = [];
        let i = 0;
        while (i < messageData.length) {
            buffers.push(messageData.slice(i, i + partSize));
            i += partSize;
        }
        if (buffers.length === 0) {
            // important to have at least one so that the header gets sent (some messages are header-only)
            buffers.push(Buffer.alloc(0));
        }
        const udpMessageId = UdpMessage_1.createUdpMessageId();
        buffers.forEach((b, ii) => {
            const body = {
                udpMessageId,
                protocolVersion: protocolVersion_1.protocolVersion(),
                fromNodeId: __classPrivateFieldGet(this, _node).nodeId(),
                toAddress,
                udpMessageType: udpMessageType,
                metaData,
                partIndex: UdpMessage_1.partIndex(ii),
                numParts: UdpMessage_1.numParts(buffers.length),
                payloadIsJson: opts.payloadIsJson
            };
            const header = {
                body,
                signature: crypto_util_1.getSignature(body, __classPrivateFieldGet(this, _node).keyPair())
            };
            parts.push({
                header,
                dataBuffer: b
            });
        });
        return parts;
    }
}
exports.default = PublicUdpSocketServer;
_node = new WeakMap(), _messagePartManager = new WeakMap(), _socket = new WeakMap(), _udpPacketSender = new WeakMap(), _udpPacketReceiver = new WeakMap(), _responseListeners = new WeakMap(), _incomingDataStreams = new WeakMap(), _receivedUdpPackets = new WeakMap(), _fallbackPacketSender = new WeakMap(), _stopped = new WeakMap();
class FallbackPacketSender {
    constructor(node) {
        this.node = node;
    }
    async sendPacket(fallbackAddress, packetId, packet) {
        if (!exports.isNodeIdFallbackAddress(fallbackAddress)) {
            /* istanbul ignore next */
            throw Error('Invalid fallback address');
        }
        const requestData = {
            requestType: 'fallbackUdpPacket',
            dataBase64: packet.toString('base64'),
            packetId
        };
        await this.node.remoteNodeManager().sendRequestToNode(fallbackAddress.nodeId, requestData, { timeoutMsec: constants_1.TIMEOUTS.fallbackUdpPacket, method: 'udp-fallback' });
    }
}
//# sourceMappingURL=PublicUdpSocketServer.js.map