"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var _maxNumBytesPerSecondToSend, _estimatedRoundtripLatencyMsec, _trialDurationMsec, _currentTrialData, _queuedPackets, _timestampStarted, _sentPacketInternalIds, _numSentPackets, _numSentBytes, _numConfirmedPackets, _numConfirmedBytes, _numTimedOutPackets, _numTimedOutBytes, _numErrorPackets, _numErrorBytes, _roundtripLatenciesMsec, _peakNumUnconfirmedBytes;
Object.defineProperty(exports, "__esModule", { value: true });
exports.createInternalId = exports.packetCount = exports.packetCountToNumber = exports.isPacketCount = exports.examplePacketCount = exports.exampleByteCountPerSec = exports.byteCountPerSec = exports.byteCountPerSecToNumber = exports.isByteCountPerSec = exports.UdpTimeoutError = void 0;
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
// this is the target fraction of udp packets lost
const TARGET_FRAC_LOST_BYTES = 0.02;
class UdpTimeoutError extends Error {
    constructor(errorString) {
        super(errorString);
    }
}
exports.UdpTimeoutError = UdpTimeoutError;
const isByteCountPerSec = (x) => {
    if (!core_1.isNumber(x))
        return false;
    if (x < 0)
        return false;
    return true;
};
exports.isByteCountPerSec = isByteCountPerSec;
const byteCountPerSecToNumber = (x) => {
    return x;
};
exports.byteCountPerSecToNumber = byteCountPerSecToNumber;
const byteCountPerSec = (n) => {
    return n;
};
exports.byteCountPerSec = byteCountPerSec;
exports.exampleByteCountPerSec = exports.byteCountPerSec(400);
exports.examplePacketCount = 40;
const isPacketCount = (x) => {
    if (!core_1.isNumber(x))
        return false;
    if (x < 0)
        return false;
    return true;
};
exports.isPacketCount = isPacketCount;
const packetCountToNumber = (x) => {
    return x;
};
exports.packetCountToNumber = packetCountToNumber;
const packetCount = (n) => {
    return n;
};
exports.packetCount = packetCount;
const addPacketCount = (n1, n2) => {
    return exports.packetCount(exports.packetCountToNumber(n1) + exports.packetCountToNumber(n2));
};
const createInternalId = () => {
    return util_1.randomAlphaString(10);
};
exports.createInternalId = createInternalId;
class UdpCongestionManager {
    constructor() {
        _maxNumBytesPerSecondToSend.set(this, exports.byteCountPerSec(3 * 1000 * 1000)); // current number of bytes per second allowed to send -- this will get adjusted based on the udp packet loss rate
        _estimatedRoundtripLatencyMsec.set(this, core_1.scaledDurationMsec(200)); // current estimated roundtrip latency
        _trialDurationMsec.set(this, core_1.scaledDurationMsec(5000)); // duration of a single trial
        _currentTrialData.set(this, new TrialData());
        _queuedPackets.set(this, new Queue());
    }
    async sendPacket(packetId, packetSize, send) {
        return new Promise((resolve, reject) => {
            __classPrivateFieldGet(this, _queuedPackets).enqueue({
                internalId: exports.createInternalId(),
                packetId,
                packetSize,
                send,
                onFinished: () => { resolve(); },
                onError: (err) => {
                    reject(err);
                }
            });
            this._handleNextPackets();
        });
    }
    _sendPacket(p) {
        const timer = core_1.nowTimestamp();
        let complete = false;
        const _onConfirmed = () => {
            if (complete)
                return;
            complete = true;
            const elapsedMsec = core_1.unscaledDurationMsec(core_1.elapsedSince(timer));
            __classPrivateFieldGet(this, _currentTrialData).reportConfirmed(p.internalId, p.packetSize, elapsedMsec);
            p.onFinished();
            this._handleNextPackets();
        };
        const _onTimedOut = () => {
            if (complete)
                return;
            complete = true;
            __classPrivateFieldGet(this, _currentTrialData).reportTimedOut(p.internalId, p.packetSize);
            this._handleNextPackets();
            p.onError(new UdpTimeoutError('Timeout while waiting for udp confirmation'));
        };
        const _onError = (err) => {
            if (complete)
                return;
            complete = true;
            __classPrivateFieldGet(this, _currentTrialData).reportError(p.internalId, p.packetSize);
            this._handleNextPackets();
            p.onError(err);
        };
        const timeoutMsec = core_1.scaleDurationBy(__classPrivateFieldGet(this, _estimatedRoundtripLatencyMsec), 5);
        p.send(timeoutMsec).then(() => {
            _onConfirmed();
        }).catch((err) => {
            if (err instanceof UdpTimeoutError) {
                _onTimedOut();
            }
            else {
                _onError(err);
            }
        });
        __classPrivateFieldGet(this, _currentTrialData).reportSent(p.internalId, p.packetSize);
    }
    _handleNextPackets() {
        if (__classPrivateFieldGet(this, _currentTrialData).elapsedMsec() > core_1.durationMsecToNumber(__classPrivateFieldGet(this, _trialDurationMsec))) {
            __classPrivateFieldSet(this, _estimatedRoundtripLatencyMsec, __classPrivateFieldGet(this, _currentTrialData).estimateRoundtripLatency(__classPrivateFieldGet(this, _estimatedRoundtripLatencyMsec)));
            __classPrivateFieldSet(this, _maxNumBytesPerSecondToSend, __classPrivateFieldGet(this, _currentTrialData).determineMaxNumBytesPerSecondToSend(__classPrivateFieldGet(this, _maxNumBytesPerSecondToSend), __classPrivateFieldGet(this, _estimatedRoundtripLatencyMsec)));
            __classPrivateFieldGet(this, _currentTrialData).reset();
        }
        while (this._handleNextPacket())
            ;
    }
    _handleNextPacket() {
        if (__classPrivateFieldGet(this, _queuedPackets).isEmpty()) {
            return false;
        }
        const p = __classPrivateFieldGet(this, _queuedPackets).peek();
        const packetSize = p.packetSize;
        const numUnconfirmedBytes = __classPrivateFieldGet(this, _currentTrialData).computeNumUnconfirmedBytes();
        // determine how many unconfirmed bytes we are allowed to have at any given moment
        const maxNumUnconfirmedBytesAllowed = core_1.byteCount(exports.byteCountPerSecToNumber(__classPrivateFieldGet(this, _maxNumBytesPerSecondToSend)) / 1000 * core_1.durationMsecToNumber(__classPrivateFieldGet(this, _estimatedRoundtripLatencyMsec)));
        if (numUnconfirmedBytes < maxNumUnconfirmedBytesAllowed) {
            __classPrivateFieldGet(this, _queuedPackets).dequeue();
            this._sendPacket(p);
            return true;
        }
        else {
            return false;
        }
    }
}
exports.default = UdpCongestionManager;
_maxNumBytesPerSecondToSend = new WeakMap(), _estimatedRoundtripLatencyMsec = new WeakMap(), _trialDurationMsec = new WeakMap(), _currentTrialData = new WeakMap(), _queuedPackets = new WeakMap();
class TrialData {
    constructor() {
        _timestampStarted.set(this, core_1.nowTimestamp());
        _sentPacketInternalIds.set(this, new Map());
        _numSentPackets.set(this, void 0);
        _numSentBytes.set(this, void 0);
        _numConfirmedPackets.set(this, void 0);
        _numConfirmedBytes.set(this, void 0);
        _numTimedOutPackets.set(this, void 0);
        _numTimedOutBytes.set(this, void 0);
        _numErrorPackets.set(this, void 0);
        _numErrorBytes.set(this, void 0);
        _roundtripLatenciesMsec.set(this, void 0);
        _peakNumUnconfirmedBytes.set(this, void 0);
        this.reset();
    }
    reset() {
        __classPrivateFieldSet(this, _timestampStarted, core_1.nowTimestamp());
        __classPrivateFieldGet(this, _sentPacketInternalIds).clear();
        __classPrivateFieldSet(this, _numSentPackets, exports.packetCount(0));
        __classPrivateFieldSet(this, _numSentBytes, core_1.byteCount(0));
        __classPrivateFieldSet(this, _numConfirmedPackets, exports.packetCount(0));
        __classPrivateFieldSet(this, _numConfirmedBytes, core_1.byteCount(0));
        __classPrivateFieldSet(this, _numTimedOutPackets, exports.packetCount(0));
        __classPrivateFieldSet(this, _numTimedOutBytes, core_1.byteCount(0));
        __classPrivateFieldSet(this, _numErrorPackets, exports.packetCount(0));
        __classPrivateFieldSet(this, _numErrorBytes, core_1.byteCount(0));
        __classPrivateFieldSet(this, _roundtripLatenciesMsec, []); // update this
        __classPrivateFieldSet(// update this
        this, _peakNumUnconfirmedBytes, core_1.byteCount(0));
    }
    computeNumUnconfirmedBytes() {
        return core_1.byteCount(core_1.byteCountToNumber(__classPrivateFieldGet(this, _numSentBytes)) -
            core_1.byteCountToNumber(__classPrivateFieldGet(this, _numConfirmedBytes)) -
            core_1.byteCountToNumber(__classPrivateFieldGet(this, _numTimedOutBytes)) -
            core_1.byteCountToNumber(__classPrivateFieldGet(this, _numErrorBytes)));
    }
    reportSent(internalId, packetSize) {
        __classPrivateFieldSet(this, _numSentPackets, addPacketCount(__classPrivateFieldGet(this, _numSentPackets), exports.packetCount(1)));
        __classPrivateFieldSet(this, _numSentBytes, core_1.addByteCount(__classPrivateFieldGet(this, _numSentBytes), packetSize));
        __classPrivateFieldGet(this, _sentPacketInternalIds).set(internalId, true);
        const u = this.computeNumUnconfirmedBytes();
        if (u > __classPrivateFieldGet(this, _peakNumUnconfirmedBytes)) {
            __classPrivateFieldSet(this, _peakNumUnconfirmedBytes, u);
        }
    }
    reportConfirmed(internalId, packetSize, roundTripLatencyMsec) {
        if (__classPrivateFieldGet(this, _sentPacketInternalIds).has(internalId)) {
            __classPrivateFieldSet(this, _numConfirmedPackets, addPacketCount(__classPrivateFieldGet(this, _numConfirmedPackets), exports.packetCount(1)));
            __classPrivateFieldSet(this, _numConfirmedBytes, core_1.addByteCount(__classPrivateFieldGet(this, _numConfirmedBytes), packetSize));
            __classPrivateFieldGet(this, _roundtripLatenciesMsec).push(roundTripLatencyMsec);
        }
    }
    reportTimedOut(internalId, packetSize) {
        if (__classPrivateFieldGet(this, _sentPacketInternalIds).has(internalId)) {
            __classPrivateFieldSet(this, _numTimedOutPackets, addPacketCount(__classPrivateFieldGet(this, _numTimedOutPackets), exports.packetCount(1)));
            __classPrivateFieldSet(this, _numTimedOutBytes, core_1.addByteCount(__classPrivateFieldGet(this, _numTimedOutBytes), packetSize));
        }
    }
    reportError(internalId, packetSize) {
        if (__classPrivateFieldGet(this, _sentPacketInternalIds).has(internalId)) {
            __classPrivateFieldSet(this, _numErrorPackets, addPacketCount(__classPrivateFieldGet(this, _numErrorPackets), exports.packetCount(1)));
            __classPrivateFieldSet(this, _numErrorBytes, core_1.addByteCount(__classPrivateFieldGet(this, _numErrorBytes), packetSize));
        }
    }
    estimateRoundtripLatency(oldEstimateMsec) {
        if (__classPrivateFieldGet(this, _roundtripLatenciesMsec).length < 5) {
            // not enough to estimate
            return oldEstimateMsec;
        }
        // adjust the round-trip latency by taking a weighted average of existing estimate with new estimate (median)
        const alpha = 0.3; // how much to weight the new estimate
        let estimatedRoundtripLatencyMsec = core_1.unscaledDurationMsec(median(__classPrivateFieldGet(this, _roundtripLatenciesMsec).map(l => core_1.durationMsecToNumber(l))) * alpha + core_1.durationMsecToNumber(oldEstimateMsec) * (1 - alpha));
        // constrain the estimate
        const minEstLatency = core_1.scaledDurationMsec(20);
        const maxEstLatency = core_1.scaledDurationMsec(1000);
        if (core_1.durationMsecToNumber(estimatedRoundtripLatencyMsec) < core_1.durationMsecToNumber(minEstLatency))
            estimatedRoundtripLatencyMsec = minEstLatency;
        if (core_1.durationMsecToNumber(estimatedRoundtripLatencyMsec) > core_1.durationMsecToNumber(maxEstLatency))
            estimatedRoundtripLatencyMsec = maxEstLatency;
        return estimatedRoundtripLatencyMsec;
    }
    determineMaxNumBytesPerSecondToSend(oldMaxNumBytesPerSecondToSend, estimatedRoundtripLatencyMsec) {
        if (core_1.byteCountToNumber(__classPrivateFieldGet(this, _numSentBytes)) < 10 * 1000) {
            // not enough bytes sent
            return oldMaxNumBytesPerSecondToSend;
        }
        // The target of % bytes we want to lose
        const targetFracLostBytes = TARGET_FRAC_LOST_BYTES;
        let maxNumBytesPerSecondToSend = oldMaxNumBytesPerSecondToSend;
        // Fraction of lost bytes
        const fracConfirmedBytes = core_1.byteCountToNumber(__classPrivateFieldGet(this, _numConfirmedBytes)) / core_1.byteCountToNumber(__classPrivateFieldGet(this, _numSentBytes));
        const fracLostBytes = 1 - fracConfirmedBytes;
        if (fracLostBytes < targetFracLostBytes) {
            // didn't lose enough bytes... let's see if we were actually limited
            // calculate the expected peak number of outstanding bytes if we were at full load
            const a = exports.byteCountPerSecToNumber(oldMaxNumBytesPerSecondToSend) / 1000 * core_1.durationMsecToNumber(estimatedRoundtripLatencyMsec);
            if (core_1.byteCountToNumber(__classPrivateFieldGet(this, _peakNumUnconfirmedBytes)) > 0.6 * a) {
                // it looks like we were indeed limited
                // okay, let's increase the rate
                maxNumBytesPerSecondToSend = exports.byteCountPerSec(exports.byteCountPerSecToNumber(maxNumBytesPerSecondToSend) * 1.2);
            }
        }
        else {
            // We lost too many bytes, so let's decrease the rate
            // question: should we check whether the loss was likely due to rate?
            maxNumBytesPerSecondToSend = exports.byteCountPerSec(exports.byteCountPerSecToNumber(maxNumBytesPerSecondToSend) / 1.2);
        }
        if (fracLostBytes < 0.2) {
            // don't let the rate get too low (unless we are losing a very high fraction)
            const lowerBound = exports.byteCountPerSec(1 * 1000 * 1000);
            if (exports.byteCountPerSecToNumber(maxNumBytesPerSecondToSend) < exports.byteCountPerSecToNumber(lowerBound)) {
                maxNumBytesPerSecondToSend = lowerBound;
            }
        }
        return maxNumBytesPerSecondToSend;
    }
    elapsedMsec() {
        return core_1.elapsedSince(__classPrivateFieldGet(this, _timestampStarted));
    }
}
_timestampStarted = new WeakMap(), _sentPacketInternalIds = new WeakMap(), _numSentPackets = new WeakMap(), _numSentBytes = new WeakMap(), _numConfirmedPackets = new WeakMap(), _numConfirmedBytes = new WeakMap(), _numTimedOutPackets = new WeakMap(), _numTimedOutBytes = new WeakMap(), _numErrorPackets = new WeakMap(), _numErrorBytes = new WeakMap(), _roundtripLatenciesMsec = new WeakMap(), _peakNumUnconfirmedBytes = new WeakMap();
class Queue extends Array {
    enqueue(val) {
        this.push(val);
    }
    dequeue() {
        return this.shift();
    }
    peek() {
        return this[0];
    }
    isEmpty() {
        return this.length === 0;
    }
}
const median = (x) => {
    const values = [...x];
    values.sort(function (a, b) { return a - b; });
    var half = Math.floor(values.length / 2);
    if (values.length % 2)
        return values[half];
    else
        return (values[half - 1] + values[half]) / 2.0;
};
//# sourceMappingURL=UdpCongestionManager.js.map