"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _messageParts, _remoteAddressesByNodeId, _onMessageCompleteCallbacks;
Object.defineProperty(exports, "__esModule", { value: true });
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const core_1 = require("../interfaces/core");
const UdpMessage_1 = require("../interfaces/UdpMessage");
class UdpMessagePartManager {
    constructor() {
        _messageParts.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
        _remoteAddressesByNodeId.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(30 * 60 * 1000)));
        _onMessageCompleteCallbacks.set(this, []);
    }
    addMessagePart(remoteAddress, udpMessageId, partInd, numParts, header, buffer) {
        __classPrivateFieldGet(this, _messageParts).set(UdpMessage_1.udpMessagePartId(udpMessageId, partInd, numParts), { header, buffer });
        let complete = true;
        const buffers = [];
        const headers = [];
        for (let i = 0; i < UdpMessage_1.numPartsToNumber(numParts); i++) {
            const d = __classPrivateFieldGet(this, _messageParts).get(UdpMessage_1.udpMessagePartId(udpMessageId, UdpMessage_1.partIndex(i), numParts));
            if (d) {
                buffers.push(d.buffer);
                headers.push(d.header);
            }
            else {
                complete = false;
                break;
            }
        }
        if (complete) {
            for (let i = 0; i < UdpMessage_1.numPartsToNumber(numParts); i++) {
                __classPrivateFieldGet(this, _messageParts).delete(UdpMessage_1.udpMessagePartId(udpMessageId, UdpMessage_1.partIndex(i), numParts));
            }
            const fullBuffer = Buffer.concat(buffers);
            __classPrivateFieldGet(this, _onMessageCompleteCallbacks).forEach(cb => {
                cb(headers[0], fullBuffer);
            });
        }
        if (remoteAddress) {
            __classPrivateFieldGet(this, _remoteAddressesByNodeId).set(header.body.fromNodeId, remoteAddress);
        }
    }
    onMessageComplete(callback) {
        __classPrivateFieldGet(this, _onMessageCompleteCallbacks).push(callback);
    }
    getRemoteAddressForNodeId(nodeId) {
        return __classPrivateFieldGet(this, _remoteAddressesByNodeId).get(nodeId) || null;
    }
}
exports.default = UdpMessagePartManager;
_messageParts = new WeakMap(), _remoteAddressesByNodeId = new WeakMap(), _onMessageCompleteCallbacks = new WeakMap();
//# sourceMappingURL=UdpMessagePartManager.js.map