"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _socket, _onPacketCallbacks, _onConfirmationCallbacks, _numPacketsReceived;
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("../interfaces/core");
const protocolVersion_1 = require("../protocolVersion");
const UdpPacketSender_1 = require("./UdpPacketSender");
class UdpPacketReceiver {
    constructor(socket, getDefects, stats, opts) {
        this.getDefects = getDefects;
        _socket.set(this, void 0);
        _onPacketCallbacks.set(this, []);
        _onConfirmationCallbacks.set(this, []);
        _numPacketsReceived.set(this, 0);
        __classPrivateFieldSet(this, _socket, socket);
        __classPrivateFieldGet(this, _socket).on('message', (message, remoteInfo) => {
            __classPrivateFieldSet(this, _numPacketsReceived, +__classPrivateFieldGet(this, _numPacketsReceived) + 1);
            const udpPacketLossNum = this.getDefects().udpPacketLossNum || null;
            if (udpPacketLossNum !== null) {
                if (((__classPrivateFieldGet(this, _numPacketsReceived) - 1) % udpPacketLossNum) === (udpPacketLossNum - 1)) {
                    // lose this message
                    return;
                }
            }
            const headerTxt = message.slice(0, UdpPacketSender_1.UDP_PACKET_HEADER_SIZE).toString().trimEnd();
            const dataBuffer = message.slice(UdpPacketSender_1.UDP_PACKET_HEADER_SIZE);
            const header = core_1.tryParseJsonObject(headerTxt);
            if (header === null) {
                return;
            }
            if ((header.protocolVersion + '') !== protocolVersion_1.protocolVersion() + '') {
                // just ignore if incorrect protocol version
                return;
            }
            if (!UdpPacketSender_1.isUdpPacketSenderHeader(header)) {
                console.warn('Unexpected udp packet header');
                return;
            }
            if (header.isConfirmation) {
                __classPrivateFieldGet(this, _onConfirmationCallbacks).forEach(cb => { cb(header.packetId); });
                return;
            }
            // send confirmation
            const h = {
                protocolVersion: protocolVersion_1.protocolVersion(),
                packetId: header.packetId,
                fromNodeId: opts.thisNodeId,
                toNodeId: header.fromNodeId,
                isConfirmation: true
            };
            const buf = Buffer.from(JSON.stringify(h).padEnd(UdpPacketSender_1.UDP_PACKET_HEADER_SIZE));
            stats.reportBytesSent('udp', header.fromNodeId, core_1.byteCount(buf.length));
            __classPrivateFieldGet(this, _socket).send(buf, 0, buf.length, remoteInfo.port, remoteInfo.address);
            __classPrivateFieldGet(this, _onPacketCallbacks).forEach(cb => {
                cb(header.packetId, dataBuffer, remoteInfo);
            });
        });
    }
    onPacket(callback) {
        __classPrivateFieldGet(this, _onPacketCallbacks).push(callback);
    }
    onConfirmation(callback) {
        __classPrivateFieldGet(this, _onConfirmationCallbacks).push(callback);
    }
    socket() {
        return __classPrivateFieldGet(this, _socket);
    }
    numPacketsReceived() {
        return __classPrivateFieldGet(this, _numPacketsReceived);
    }
}
exports.default = UdpPacketReceiver;
_socket = new WeakMap(), _onPacketCallbacks = new WeakMap(), _onConfirmationCallbacks = new WeakMap(), _numPacketsReceived = new WeakMap();
//# sourceMappingURL=UdpPacketReceiver.js.map