"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _socket, _congestionManagers, _unconfirmedOutgoingPackets, _debugId, _packetSender, _packetId, _address, _fallbackAddress, _buffer, _onConfirmed, _onCancelled, _confirmed, _cancelled, _timeoutMsec;
Object.defineProperty(exports, "__esModule", { value: true });
exports.isUdpPacketSenderHeader = exports.UDP_PACKET_HEADER_SIZE = exports.createPacketId = exports.isPacketId = void 0;
const GarbageMap_1 = __importDefault(require("../common/GarbageMap"));
const util_1 = require("../common/util");
const core_1 = require("../interfaces/core");
const protocolVersion_1 = require("../protocolVersion");
const UdpCongestionManager_1 = __importStar(require("./UdpCongestionManager"));
const examplePacketId = "packetId";
const isPacketId = (x) => {
    if (!core_1.isString(x))
        return false;
    return (/^[A-Za-z]{10}$/.test(x));
};
exports.isPacketId = isPacketId;
const createPacketId = () => {
    return util_1.randomAlphaString(10);
};
exports.createPacketId = createPacketId;
exports.UDP_PACKET_HEADER_SIZE = 500;
const isUdpPacketSenderHeader = (x) => {
    return core_1._validateObject(x, {
        protocolVersion: core_1.isEqualTo(protocolVersion_1.protocolVersion()),
        fromNodeId: core_1.isNodeId,
        toNodeId: core_1.isNodeId,
        packetId: exports.isPacketId,
        isConfirmation: core_1.isBoolean
    });
};
exports.isUdpPacketSenderHeader = isUdpPacketSenderHeader;
class UdpPacketSender {
    constructor(socket, fallbackPacketSender, stats, opts) {
        this.fallbackPacketSender = fallbackPacketSender;
        this.stats = stats;
        this.opts = opts;
        _socket.set(this, void 0);
        _congestionManagers.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(5 * 60 * 1000)));
        _unconfirmedOutgoingPackets.set(this, new GarbageMap_1.default(core_1.scaledDurationMsec(5 * 60 * 1000)));
        _debugId.set(this, util_1.randomAlphaString(4));
        __classPrivateFieldSet(this, _socket, socket);
    }
    socket() {
        return __classPrivateFieldGet(this, _socket);
    }
    async sendPackets(address, fallbackAddress, packets, opts) {
        const outgoingPackets = packets.map(p => {
            const pkt = new OutgoingPacket(this, address, fallbackAddress, p, opts.timeoutMsec, this.stats, { thisNodeId: this.opts.thisNodeId, toNodeId: opts.toNodeId });
            __classPrivateFieldGet(this, _unconfirmedOutgoingPackets).set(pkt.packetId(), pkt);
            return pkt;
        });
        const promises = outgoingPackets.map(pkt => {
            this.stats.reportBytesSent('udp', opts.toNodeId, pkt.size());
            return pkt.send();
        }); // send the packets and await all the promises
        try {
            for (let p of promises) {
                await p;
            }
        }
        catch (err) {
            outgoingPackets.forEach(pkt => { pkt.cancel(); });
            throw (err);
        }
    }
    receivePacketConfirmation(packetId) {
        const p = __classPrivateFieldGet(this, _unconfirmedOutgoingPackets).get(packetId);
        if (p) {
            p.confirm();
            __classPrivateFieldGet(this, _unconfirmedOutgoingPackets).delete(packetId);
        }
        else {
            console.warn(`Unable to confirm packet with ID: ${packetId}`, __classPrivateFieldGet(this, _debugId));
        }
    }
    congestionManagers(address) {
        const addressHash = core_1.sha1OfObject(address).toString();
        const c = __classPrivateFieldGet(this, _congestionManagers).get(addressHash) || new UdpCongestionManager_1.default();
        // do it this way so that garbage collection of GarbageMap will function
        __classPrivateFieldGet(this, _congestionManagers).set(addressHash, c);
        return c;
    }
    async _fallbackSendPacket(fallbackAddress, packetId, buffer) {
        await this.fallbackPacketSender.sendPacket(fallbackAddress, packetId, buffer);
    }
}
exports.default = UdpPacketSender;
_socket = new WeakMap(), _congestionManagers = new WeakMap(), _unconfirmedOutgoingPackets = new WeakMap(), _debugId = new WeakMap();
class OutgoingPacket {
    constructor(packetSender, address, fallbackAddress, buffer, timeoutMsec, stats, opts) {
        this.stats = stats;
        this.opts = opts;
        _packetSender.set(this, void 0);
        _packetId.set(this, void 0);
        _address.set(this, void 0);
        _fallbackAddress.set(this, void 0);
        _buffer.set(this, void 0);
        _onConfirmed.set(this, void 0);
        _onCancelled.set(this, void 0);
        _confirmed.set(this, false);
        _cancelled.set(this, false);
        _timeoutMsec.set(this, void 0);
        __classPrivateFieldSet(this, _packetSender, packetSender);
        __classPrivateFieldSet(this, _address, address);
        __classPrivateFieldSet(this, _fallbackAddress, fallbackAddress);
        __classPrivateFieldSet(this, _buffer, buffer);
        __classPrivateFieldSet(this, _timeoutMsec, timeoutMsec);
        __classPrivateFieldSet(this, _packetId, exports.createPacketId());
    }
    packetId() {
        return __classPrivateFieldGet(this, _packetId);
    }
    confirm() {
        __classPrivateFieldGet(this, _onConfirmed) && __classPrivateFieldGet(this, _onConfirmed).call(this);
    }
    async send() {
        if (__classPrivateFieldGet(this, _address) === null) {
            await __classPrivateFieldGet(this, _packetSender)._fallbackSendPacket(__classPrivateFieldGet(this, _fallbackAddress), __classPrivateFieldGet(this, _packetId), __classPrivateFieldGet(this, _buffer));
            return;
        }
        const cm = __classPrivateFieldGet(this, _packetSender).congestionManagers(__classPrivateFieldGet(this, _address));
        try {
            await cm.sendPacket(__classPrivateFieldGet(this, _packetId), core_1.byteCount(__classPrivateFieldGet(this, _buffer).length), async (timeoutMsec) => {
                await this._trySend(timeoutMsec);
            });
        }
        catch (err) {
            await __classPrivateFieldGet(this, _packetSender)._fallbackSendPacket(__classPrivateFieldGet(this, _fallbackAddress), __classPrivateFieldGet(this, _packetId), __classPrivateFieldGet(this, _buffer));
        }
    }
    size() {
        return core_1.byteCount(__classPrivateFieldGet(this, _buffer).length);
    }
    async _trySend(timeoutMsec) {
        const socket = __classPrivateFieldGet(this, _packetSender).socket();
        const b = __classPrivateFieldGet(this, _buffer);
        return new Promise((resolve, reject) => {
            /* istanbul ignore next */
            if (__classPrivateFieldGet(this, _confirmed)) {
                resolve();
                return;
            }
            /* istanbul ignore next */
            if (__classPrivateFieldGet(this, _cancelled)) {
                reject(Error('Cancelled'));
                return;
            }
            let completed = false;
            __classPrivateFieldSet(this, _onConfirmed, () => {
                if (completed)
                    return;
                completed = true;
                resolve();
            });
            __classPrivateFieldSet(this, _onCancelled, () => {
                if (completed)
                    return;
                completed = true;
                reject(Error('Canceled'));
            });
            const h = {
                protocolVersion: protocolVersion_1.protocolVersion(),
                packetId: __classPrivateFieldGet(this, _packetId),
                fromNodeId: this.opts.thisNodeId,
                toNodeId: this.opts.toNodeId,
                isConfirmation: false
            };
            const b2 = Buffer.concat([
                Buffer.from(JSON.stringify(h).padEnd(exports.UDP_PACKET_HEADER_SIZE)),
                b
            ]);
            /* istanbul ignore next */
            if (!__classPrivateFieldGet(this, _address))
                throw Error('Unexpected address in _trySend');
            this.stats.reportBytesSent('udp', this.opts.toNodeId, core_1.byteCount(b2.length));
            if (!__classPrivateFieldGet(this, _address).hostName)
                throw Error('Unexpected udp address');
            if (!__classPrivateFieldGet(this, _address).port)
                throw Error('Unexpected udp address port');
            socket.send(b2, 0, b2.length, core_1.portToNumber(__classPrivateFieldGet(this, _address).port), __classPrivateFieldGet(this, _address).hostName.toString(), (err, numBytesSent) => {
                /* istanbul ignore next */
                if (err) {
                    if (completed)
                        return;
                    completed = true;
                    reject(Error(`Failed to send udp message to remote: ${err.message}`));
                }
                /* istanbul ignore next */
                if (numBytesSent !== b2.length) {
                    if (completed)
                        return;
                    completed = true;
                    console.warn(__classPrivateFieldGet(this, _address));
                    reject(Error(`Failed to send udp message to remote: unexpected numBytesSent: ${numBytesSent} <> ${b.length}`));
                }
            });
            setTimeout(() => {
                if (!completed) {
                    completed = true;
                    reject(new UdpCongestionManager_1.UdpTimeoutError('Timed out'));
                }
            }, core_1.durationMsecToNumber(timeoutMsec));
        });
    }
    cancel() {
        __classPrivateFieldGet(this, _onCancelled) && __classPrivateFieldGet(this, _onCancelled).call(this);
    }
}
_packetSender = new WeakMap(), _packetId = new WeakMap(), _address = new WeakMap(), _fallbackAddress = new WeakMap(), _buffer = new WeakMap(), _onConfirmed = new WeakMap(), _onCancelled = new WeakMap(), _confirmed = new WeakMap(), _cancelled = new WeakMap(), _timeoutMsec = new WeakMap();
//# sourceMappingURL=UdpPacketSender.js.map